#include "stdafx.h"

// tests ValueInspectorBE

#include "test.h"
#include "grt/grt_value_inspector.h"
#include "grtpp_value.h"

using namespace grt;
using namespace bec;

#include "grt_values_test_data.h"
#include "structs.test.h"

BEGIN_TEST_DATA_CLASS(be_inspector_value)
public:
  GRT grt;

END_TEST_DATA_CLASS;



TEST_MODULE(be_inspector_value, "grt value inspector (ValueInspectorBE) backend");



inline bool get_row(ValueInspectorBE *insp, const NodeId &node, std::string &name, std::string &value)
{
  return insp->get_field(node, ValueInspectorBE::Name, name) && insp->get_field(node, ValueInspectorBE::Value, value);
}

// Test Lists
// ----------

TEST_FUNCTION(1)
{
  bool flag;
  // test inspection of list

  // create a test list
  BaseListRef list(create_list_with_varied_data(&grt));

  ensure_equals("initial list size", (int)list.count(), 10);
  ensure_equals("list type", list.type(), ListType);

  ValueInspectorBE *vinsp= ValueInspectorBE::create(&grt, list, false);

  // test listing
  int c= vinsp->count();
  ensure_equals("begin listing", c, 10);

  std::string name, value;
  Type type;
  NodeId node;
  
  node= vinsp->get_node(0);
  flag= get_row(vinsp, node, name, value);
  ensure("first item", flag);
  ensure_equals("1) name", name, "[1]");
  ensure_equals("1) value", value, "111");
  type= vinsp->get_field_type(node, ValueInspectorBE::Value);
  ensure_equals("1) type", type, IntegerType);

  node= vinsp->get_node(1);
  flag= get_row(vinsp, node, name, value);
  ensure("item2", flag);
  ensure_equals("2) name", name, "[2]");
  ensure_equals("2) value", value, "222");
  type= vinsp->get_field_type(node, ValueInspectorBE::Value);
  ensure_equals("2) type", type, IntegerType);

  node= vinsp->get_node(2);
  flag= get_row(vinsp, node, name, value);
  ensure("item3", flag);
  ensure_equals("3) name", name, "[3]");
  ensure_equals("3) value", value, "test");
  type= vinsp->get_field_type(node, ValueInspectorBE::Value);
  ensure_equals("3) type", type, StringType);

  node= vinsp->get_node(3);
  flag= get_row(vinsp, node, name, value);
  ensure("item4", flag);
  ensure_equals("4) name", name, "[4]");
  ensure_equals("4) value", value, "123.456");
  type= vinsp->get_field_type(node, ValueInspectorBE::Value);
  ensure_equals("4) type", type, DoubleType);

  node= vinsp->get_node(4);
  flag= get_row(vinsp, node, name, value);
  ensure("item5", flag);
  ensure_equals("5) name", name, "[5]");
  ensure_equals("5) value", value, "test");
  type= vinsp->get_field_type(node, ValueInspectorBE::Value);
  ensure_equals("5) type", type, StringType);

  node= vinsp->get_node(5);
  flag= get_row(vinsp, node, name, value);
  ensure("item6", flag);
  ensure_equals("6) name", name, "[6]");
  ensure_equals("6) value", value, "[ 0, 1, 2, 3, 4 ]");
  type= vinsp->get_field_type(node, ValueInspectorBE::Value);
  ensure_equals("6) type", type, ListType);

  node= vinsp->get_node(6);
  flag= get_row(vinsp, node, name, value);
  ensure("item7", flag);
  ensure_equals("7) name", name, "[7]");
  ensure_equals("7) value", value, "{ item1 = 1, item2 = 2.2, item3 = test }");
  type= vinsp->get_field_type(node, ValueInspectorBE::Value);
  ensure_equals("7) type", type, DictType);

  // XXX test with objects
  node= vinsp->get_node(7);
  flag= get_row(vinsp, node, name, value);
  ensure_equals("8) name", name, "[8]");
  ensure_equals("8) value", value, "test");
  type= vinsp->get_field_type(node, ValueInspectorBE::Value);
  ensure_equals("8) type", type, StringType);

  node= vinsp->get_node(8);
  flag= get_row(vinsp, node, name, value);
  ensure_equals("9) name", name, "[9]");
  ensure_equals("9) value", value, "test");
  type= vinsp->get_field_type(node, ValueInspectorBE::Value);
  ensure_equals("9) type", type, StringType);

  node= vinsp->get_node(9);
  flag= get_row(vinsp, node, name, value);
  ensure_equals("10) name", name, "[10]");
  ensure_equals("10) value", value, "test");
  type= vinsp->get_field_type(node, ValueInspectorBE::Value);
  ensure_equals("10) type", type, StringType);

  node= vinsp->get_node(10);
  flag= get_row(vinsp, node, name, value);
  ensure("last+1 item", !flag);


  // test get random item

  node= vinsp->get_child(vinsp->get_root(), 6);

  flag= vinsp->get_field(node, ValueInspectorBE::Name, name);
  ensure("get", flag);
  flag= vinsp->get_field(node, ValueInspectorBE::Value, value);
  ensure("get", flag);
  type= vinsp->get_field_type(node, ValueInspectorBE::Value);
  ensure("get", flag);
  ensure_equals("7) value", name, "[7]");
  ensure_equals("7) value", value, "{ item1 = 1, item2 = 2.2, item3 = test }");
  ensure_equals("7) type", type, DictType);

  node= vinsp->get_node(-1);
  ensure("bad get", !node.is_valid());

  node= vinsp->get_node(11);
  ensure("bad get", !node.is_valid());

  // test change int value
  flag= vinsp->set_field(0, ValueInspectorBE::Value, 112211);
  ensure("set 1", flag);

  flag= vinsp->get_field(0, ValueInspectorBE::Value, value);
  ensure("get/set 1", flag);
  ensure_equals("get/set 1", value, "112211");
  type= vinsp->get_field_type(0, ValueInspectorBE::Value);
  ensure_equals("get/set 1", type, IntegerType);

  flag= vinsp->set_field(9, ValueInspectorBE::Value, 8888);
  ensure("set 10", flag);

  flag= get_row(vinsp, 9, name, value);
  ensure("get/set 10", flag);
  ensure_equals("get/set 10", name, "[10]");
  ensure_equals("get/set 10", value, "8888");
  type= vinsp->get_field_type(9, ValueInspectorBE::Value);
  ensure_equals("get/set 10", type, IntegerType);

  // test change string value
  flag= vinsp->set_field(0, ValueInspectorBE::Value, "hello");
  ensure("set 1", flag);

  flag= get_row(vinsp, 0, name, value);
  ensure("get/set 1", flag);
  ensure_equals("get/set 1", name, "[1]");
  ensure_equals("get/set 1", value, "hello");
  type= vinsp->get_field_type(0, ValueInspectorBE::Value);
  ensure_equals("get/set 1", type, StringType);

  flag= vinsp->set_field(9, ValueInspectorBE::Value, "world");
  ensure("set 10", flag);

  flag= get_row(vinsp, 9, name, value);
  ensure("get/set 10", flag);
  ensure_equals("get/set 10", name, "[10]");
  ensure_equals("get/set 10", value, "world");
  type= vinsp->get_field_type(9, ValueInspectorBE::Value);
  ensure_equals("get/set 10", type, StringType);

  // test change double value
  flag= vinsp->set_field(0, ValueInspectorBE::Value, 1.234);
  ensure("set 1", flag);

  flag= get_row(vinsp, 0, name, value);
  ensure("get/set 1", flag);
  ensure_equals("get/set 1", name, "[1]");
  ensure_equals("get/set 1", value, "1.234");
  type= vinsp->get_field_type(0, ValueInspectorBE::Value);
  ensure_equals("get/set 1", type, DoubleType);

  flag= vinsp->set_field(8, ValueInspectorBE::Value, 5.1);
  ensure("set 9", flag);

  flag= get_row(vinsp, 8, name, value);
  ensure("get/set 9", flag);
  ensure_equals("get/set 9", name, "[9]");
  ensure_equals("get/set 9", value, "5.1");
  type= vinsp->get_field_type(8, ValueInspectorBE::Value);
  ensure_equals("get/set 9", type, DoubleType);

  // test change int value from string
  flag= vinsp->set_convert_field(1, ValueInspectorBE::Value, "112233");
  ensure("set 2", flag);

  // in this case, the list is untyped, so it should just appear as string
  flag= get_row(vinsp, 1, name, value);
  ensure("get/set 2", flag);
  ensure_equals("get/set 2", value, "112233");
  type= vinsp->get_field_type(1, ValueInspectorBE::Value);
  ensure_equals("get/set 2", type, StringType);

  // item count still ok?

  ensure_equals("item count after changes", (int)list.count(), 10);
  
  // test add new value
  NodeId nkey;
  flag= vinsp->add_item(nkey);
  ensure("add ok", flag);
  ensure_equals("add", nkey[0], 10);
  flag= vinsp->set_field(nkey, ValueInspectorBE::Value, "new value");
  ensure("set item", flag);

  ensure_equals("new count", (int)list.count(), 11);

  flag= get_row(vinsp, nkey, name, value);
  ensure("get new", flag);
  ensure_equals("new value", name, "[11]");
  ensure_equals("new value", value, "new value");
  type= vinsp->get_field_type(nkey, ValueInspectorBE::Value);
  ensure_equals("new type", type, StringType);

  // test delete value
  flag= vinsp->delete_item(3);
  ensure("del", flag);
  
  ensure_equals("count", (int)list.count(), 10);

  flag= get_row(vinsp, 3, name, value);
  ensure("get/del", flag);

  ensure_equals("get/del value", value, "test");
  
  flag= vinsp->delete_item(10);
  ensure("bad del", !flag);
  
  delete vinsp;
}


TEST_FUNCTION(2)
{
  bool flag;
  
  // test inspection of string typed list

  // create a test list
  BaseListRef list(create_string_list(&grt, 10));

  ensure_equals("initial list size", (int)list.count(), 10);
  ensure_equals("list type", list.type(), ListType);
  ensure_equals("list content type", list.content_type(), StringType);

  ValueInspectorBE *vinsp= ValueInspectorBE::create(&grt, list, false);

  // test listing
  int c= vinsp->count();
  ensure_equals("begin listing", c, 10);

  std::string name, value;
  Type type;

  for (int i= 0; i < 10; i++)
  {
    char buffer[5];
    char buffer2[10];
    sprintf(buffer, "[%i]", i+1);
    sprintf(buffer2, "test%i", i+1);

    flag= get_row(vinsp, i, name, value);
    ensure("item", flag);
    ensure_equals("name", name, std::string(buffer));
    ensure_equals("value", value, std::string(buffer2));
    type= vinsp->get_field_type(i, ValueInspectorBE::Value);
    ensure_equals("type", type, StringType);
  }
  flag= get_row(vinsp, 10, name, value);
  ensure("end item", !flag);

  // test get random item

  flag= get_row(vinsp, 6, name, value);
  ensure("get", flag);
  ensure_equals("7) value", name, "[7]");
  ensure_equals("7) value", value, "test7");
  type= vinsp->get_field_type(6, ValueInspectorBE::Value);
  ensure_equals("7) type", type, StringType);

  NodeId node= vinsp->get_node(-1);
  ensure("bad get", !node.is_valid());

  flag= get_row(vinsp, 11, name, value);
  ensure("bad get", !flag);

  flag= vinsp->get_field(11, ValueInspectorBE::Name, name);
  ensure("bad get", !flag);

  flag= vinsp->get_field(11, ValueInspectorBE::Value, value);
  ensure("bad get", !flag);

  // test change int value
  flag= vinsp->set_field(0, ValueInspectorBE::Value, 112211);
  ensure("set 1", !flag);

  flag= get_row(vinsp, 0, name, value);
  ensure("get/set 1", flag);
  ensure_equals("get/set 1", value, "test1");
  type= vinsp->get_field_type(0, ValueInspectorBE::Value);
  ensure_equals("get/set 1", type, StringType);

  // test change string value
  flag= vinsp->set_field(0, ValueInspectorBE::Value, "hello");
  ensure("set 1", flag);

  flag= get_row(vinsp, 0, name, value);
  ensure("get/set 1", flag);
  ensure_equals("get/set 1", value, "hello");
  type= vinsp->get_field_type(0, ValueInspectorBE::Value);
  ensure_equals("get/set 1", type, StringType);

  // test change double value
  flag= vinsp->set_field(1, ValueInspectorBE::Value, 1.234);
  ensure("set 2", !flag);

  flag= get_row(vinsp, 1, name, value);
  ensure("get/set 2", flag);
  ensure_equals("get/set 2", value, "test2");
  type= vinsp->get_field_type(1, ValueInspectorBE::Value);
  ensure_equals("get/set 2", type, StringType);

  // test change int value from string
  flag= vinsp->set_convert_field(2, ValueInspectorBE::Value, "112233");
  ensure("set 3", flag);

  flag= get_row(vinsp, 2, name, value);
  ensure("get/set 3", flag);
  ensure_equals("get/set 3", value, "112233");
  type= vinsp->get_field_type(2, ValueInspectorBE::Value);
  ensure_equals("get/set 3", type, StringType);

  // item count still ok?
  delete vinsp;

  ensure_equals("item count after changes", (int)list.count(), 10);
}


TEST_FUNCTION(3)
{
  bool flag;
  
  // test inspection of int typed list

  // create a test list
  BaseListRef list(create_int_list(&grt, 10));

  ensure_equals("initial list size", (int)list.count(), 10);
  ensure_equals("list type", list.type(), ListType);
  ensure_equals("list content type", list.content_type(), IntegerType);

  ValueInspectorBE *vinsp= ValueInspectorBE::create(&grt, list, false);

  // test listing
  int c= vinsp->count();
  ensure_equals("begin listing", c, 10);

  std::string name, value;
  Type type;

  for (int i= 0; i < 10; i++)
  {
    char buffer[5];
    char buffer2[10];
    sprintf(buffer, "[%i]", i+1);
    sprintf(buffer2, "%i", i);

    flag= get_row(vinsp, i, name, value);
    ensure("item", flag);
    ensure_equals(" name", name, std::string(buffer));
    ensure_equals(" value", value, std::string(buffer2));
    type= vinsp->get_field_type(i, ValueInspectorBE::Value);
    ensure_equals(" type", type, IntegerType);
  }
  flag= get_row(vinsp, 10, name, value);
  ensure("end item", !flag);

  // test get random item

  // 7 must be 6 otherwise there might be a off by 1 error
  flag= get_row(vinsp, 6, name, value);
  ensure("get", flag);
  ensure_equals("7) value", name, "[7]");
  ensure_equals("7) value", value, "6");
  type= vinsp->get_field_type(6, ValueInspectorBE::Value);
  ensure_equals("7) type", type, IntegerType);

  flag= get_row(vinsp, -1, name, value);
  ensure("bad get", !flag);

  flag= get_row(vinsp, 10, name, value);
  ensure("bad get", !flag);

  NodeId node= vinsp->get_node(11);
  ensure("bad get", !node.is_valid());

  //node= vinsp->get_child(4, 0);
  //ensure("bad get", !node.is_valid());

  //node= vinsp->get_child(1, 1);
  //ensure("bad get", !node.is_valid());

  flag= vinsp->set_field(0, ValueInspectorBE::Name, 123);
  ensure("bad set", !flag);

  // test change int value
  flag= vinsp->set_field(0, ValueInspectorBE::Value, 112211);
  ensure("set 1", flag);

  flag= get_row(vinsp, 0, name, value);
  ensure("get/set 1", flag);
  ensure_equals("get/set 1", value, "112211");
  ensure_equals("get/set 1", type, IntegerType);

  // test change string value
  flag= vinsp->set_field(0, ValueInspectorBE::Value, "hello");
  ensure("set 1", !flag);

  flag= get_row(vinsp, 0, name, value);
  ensure("get/set 1", flag);
  ensure_equals("get/set 1", value, "112211");
  type= vinsp->get_field_type(0, ValueInspectorBE::Value);
  ensure_equals("get/set 1", type, IntegerType);

  // test change double value
  flag= vinsp->set_field(1, ValueInspectorBE::Value, 1.234);
  ensure("set 2", !flag);

  flag= get_row(vinsp, 1, name, value);
  ensure("get/set 2", flag);
  ensure_equals("get/set 2", value, "1");
  type= vinsp->get_field_type(1, ValueInspectorBE::Value);
  ensure_equals("get/set 2", type, IntegerType);

  // test change int value from string
  flag= vinsp->set_convert_field(2, ValueInspectorBE::Value, "112233");
  ensure("set 3", flag);

  flag= get_row(vinsp, 2, name, value);
  ensure("get/set 2", flag);
  ensure_equals("get/set 2", value, "112233");
  type= vinsp->get_field_type(2, ValueInspectorBE::Value);
  ensure_equals("get/set 2", type, IntegerType);

  // item count still ok?

  delete vinsp;
  
  ensure_equals("item count after changes", (int)list.count(), 10);
}


// Test Dicts
// ----------

TEST_FUNCTION(10)
{
  bool flag;
  
  // test inspection of dict

  // create a test dict
  DictRef dict(create_dict_with_varied_data(&grt));

  ensure_equals("initial dict size", (int)dict.count(), 6);
  ensure_equals("dict type", dict.type(), DictType);

  ValueInspectorBE *vinsp= ValueInspectorBE::create(&grt, dict, false);

  // test listing
  int c= vinsp->count();
  ensure_equals("begin listing", c, 6);

  std::string name, value;
  Type type;

  flag= get_row(vinsp, 0, name, value);
  ensure("first item", flag);
  ensure_equals("1) name", name, "k1");
  ensure_equals("1) value", value, "111");
  type= vinsp->get_field_type(0, ValueInspectorBE::Value);
  ensure_equals("1) type", type, IntegerType);

  flag= get_row(vinsp, 1, name, value);
  ensure(" item 2", flag);
  ensure_equals("2) name", name, "k2");
  ensure_equals("2) value", value, "123.456");
  type= vinsp->get_field_type(1, ValueInspectorBE::Value);
  ensure_equals("2) type", type, DoubleType);

  flag= get_row(vinsp, 2, name, value);
  ensure("item 3", flag);
  ensure_equals("3) name", name, "k3");
  ensure_equals("3) value", value, "test3");
  type= vinsp->get_field_type(2, ValueInspectorBE::Value);
  ensure_equals("3) type", type, StringType);

  flag= get_row(vinsp, 3, name, value);
  ensure("item 4", flag);
  ensure_equals("4) name", name, "k4");
  ensure_equals("4) value", value, "444");
  type= vinsp->get_field_type(3, ValueInspectorBE::Value);
  ensure_equals("4) type", type, IntegerType);

  flag= get_row(vinsp, 4, name, value);
  ensure("item 5", flag);
  ensure_equals("5) name", name, "k5");
  ensure_equals("5) value", value, "test5");
  type= vinsp->get_field_type(4, ValueInspectorBE::Value);
  ensure_equals("5) type", type, StringType);

  flag= get_row(vinsp, 5, name, value);
  ensure("item 6", flag);
  ensure_equals("6) name", name, "k6");
  ensure_equals("6) value", value, "[ 1, 2, 3, 4, 5 ]");
  type= vinsp->get_field_type(5, ValueInspectorBE::Value);
  ensure_equals("6) type", type, ListType);

  flag= get_row(vinsp, 6, name, value);
  ensure("end item", !flag);
  

  // test get random item
  NodeId node= vinsp->get_node(4);
  flag= vinsp->get_field(node, ValueInspectorBE::Name, name);
  ensure("get", flag);
  flag= vinsp->get_field(node, ValueInspectorBE::Value, value);
  ensure("get", flag);
  type= vinsp->get_field_type(node, ValueInspectorBE::Value);

  ensure_equals("get", name, "k5");
  ensure_equals("get", value, "test5");
  ensure_equals("get", type, StringType);

  node= vinsp->get_node(-1);
  ensure("bad get", !node.is_valid());

//  node= vinsp->get_child(1, 0);
//  ensure("bad get", !node.is_valid());

  // test change int value
  flag= vinsp->set_field(0, ValueInspectorBE::Value, 112211);
  ensure("set 1", flag);

  flag= get_row(vinsp, 0, name, value);
  ensure("get/set k1", flag);
  ensure_equals("get/set k1", name, "k1");
  ensure_equals("get/set k1", value, "112211");
  type= vinsp->get_field_type(0, ValueInspectorBE::Value);
  ensure_equals("get/set k1", type, IntegerType);

  flag= vinsp->set_field(4, ValueInspectorBE::Value, 8888);
  ensure("set k5", flag);

  flag= get_row(vinsp, 4, name, value);
  ensure("get/set k5", flag);
  ensure_equals("get/set k5", name, "k5");
  ensure_equals("get/set k5", value, "8888");
  type= vinsp->get_field_type(4, ValueInspectorBE::Value);
  ensure_equals("get/set k5", type, IntegerType);

  // test change string value
  flag= vinsp->set_field(0, ValueInspectorBE::Value, "hello");
  ensure("set k1", flag);

  flag= get_row(vinsp, 0, name, value);
  ensure("get/set k1", flag);
  ensure_equals("get/set k1", name, "k1");
  ensure_equals("get/set k1", value, "hello");
  type= vinsp->get_field_type(0, ValueInspectorBE::Value);
  ensure_equals("get/set k1", type, StringType);

  // test change double value
  flag= vinsp->set_field(0, ValueInspectorBE::Value, 1.234);
  ensure("set k1", flag);

  flag= get_row(vinsp, 0, name, value);
  ensure("get/set k1", flag);
  ensure_equals("get/set k1", value, "1.234");
  type= vinsp->get_field_type(0, ValueInspectorBE::Value);
  ensure_equals("get/set k1", type, DoubleType);

  // test change int value from string
  flag= vinsp->set_convert_field(1, ValueInspectorBE::Value, "112233");
  ensure("set k2", flag);

  // in this case, the dict is untyped, so it should just appear as string
  flag= get_row(vinsp, 1, name, value);
  ensure("get/set k2", flag);
  ensure_equals("get/set k2", value, "112233");
  type= vinsp->get_field_type(1, ValueInspectorBE::Value);
  ensure_equals("get/set k2", type, StringType);

  // item count still ok?

  ensure_equals("item count after changes", (int)dict.count(), 6);
  
  // test add new value
  NodeId nkey;
  flag= vinsp->add_item(nkey);
  ensure("add ok", flag);
  ensure_equals("add", nkey[0], 6);

  flag= vinsp->set_field(nkey, ValueInspectorBE::Name, "newk");
  ensure("set item name", flag);
  flag= vinsp->set_field(nkey, ValueInspectorBE::Value, "new value");
  ensure("set item", flag);

  ensure_equals("new count", (int)dict.count(), 7);

  flag= get_row(vinsp, nkey, name, value);
  ensure("get new", flag);
  ensure_equals("new value", value, "new value");
  type= vinsp->get_field_type(nkey, ValueInspectorBE::Value);
  ensure_equals("new type", type, StringType);

  // test delete value
  flag= vinsp->delete_item(3);
  ensure("del", flag);
  
  ensure_equals("count", (int)dict.count(), 6);

  get_row(vinsp, 3, name, value);
  ensure("delete k4", name != "k4");

  flag= get_row(vinsp, 4, name, value);
  ensure("get/del", flag);
  ensure_equals("get/del value", value, "[ 1, 2, 3, 4, 5 ]");
  
  flag= vinsp->delete_item(8);
  ensure("bad del", !flag);

  delete vinsp;
  
  ensure_equals("final count", (int)dict.count(), 6);
}



TEST_FUNCTION(11)
{
  bool flag;
  
  // test inspection of typed dict

  // create a test dict
  DictRef dict(create_dict_with_int_data(&grt));

  ensure_equals("initial dict size", (int)dict.count(), 9);
  ensure_equals("dict type", dict.type(), DictType);
  ensure_equals("dict content type", dict.content_type(), IntegerType);

  ValueInspectorBE *vinsp= ValueInspectorBE::create(&grt, dict, false);

  // test listing
  int c= vinsp->count();
  ensure_equals("begin listing", c, 9);

  std::string name, value;
  Type type;

  for (int i= 0; i < 9; i++)
  {
    char buffer[10];
    char buffer2[10];
    NodeId node;

    sprintf(buffer, "k%i", i+1);
    sprintf(buffer2, "%i", i+1);

    node= vinsp->get_node(i);

    flag= get_row(vinsp, node, name, value);
    ensure("item", flag);
    ensure_equals("name", name, buffer);
    ensure_equals("value", value, buffer2);
    type= vinsp->get_field_type(node, ValueInspectorBE::Value);
    ensure_equals("type", type, IntegerType);
  }
  flag= get_row(vinsp, 9, name, value);
  ensure("end item", !flag);
  
  // test get random item
  flag= get_row(vinsp, 4, name, value);
  ensure("get", flag);
  ensure_equals("get", value, "5");
  type= vinsp->get_field_type(4, ValueInspectorBE::Value);
  ensure_equals("get", type, IntegerType);

  flag= get_row(vinsp, 9, name, value);
  ensure("bad get", !flag);

  flag= get_row(vinsp, -1, name, value);
  ensure("bad get", !flag);

  // test change int value
  flag= vinsp->set_field(0, ValueInspectorBE::Value, 112211);
  ensure("set 1", flag);

  flag= get_row(vinsp, 0, name, value);
  ensure("get/set k1", flag);
  ensure_equals("get/set k1", name, "k1");
  ensure_equals("get/set k1", value, "112211");
  type= vinsp->get_field_type(0, ValueInspectorBE::Value);
  ensure_equals("get/set k1", type, IntegerType);

  flag= vinsp->set_field(4, ValueInspectorBE::Value, 8888);
  ensure("set k5", flag);

  int ivalue;
  flag= vinsp->get_field(4, ValueInspectorBE::Value, ivalue);
  ensure("get/set k5", flag);
  ensure_equals("get/set k5", ivalue, 8888);
  type= vinsp->get_field_type(4, ValueInspectorBE::Value);
  ensure_equals("get/set k5", type, IntegerType);

  // test change string value
  flag= vinsp->set_field(0, ValueInspectorBE::Value, "hello");
  ensure("set k1", !flag);

  flag= get_row(vinsp, 0, name, value);
  ensure("get/set k1", flag);
  ensure_equals("get/set k1", value, "112211");
  type= vinsp->get_field_type(0, ValueInspectorBE::Value);
  ensure_equals("get/set k1", type, IntegerType);

  // test change double value
  flag= vinsp->set_field(0, ValueInspectorBE::Value, 1.234);
  ensure("set k1", !flag);

  flag= get_row(vinsp, 0, name, value);
  ensure("get/set k1", flag);
  ensure_equals("get/set k1", value, "112211");
  type= vinsp->get_field_type(0, ValueInspectorBE::Value);
  ensure_equals("get/set k1", type, IntegerType);

  // test change int value from string
  flag= vinsp->set_convert_field(1, ValueInspectorBE::Value, "112233");
  ensure("set k2", flag);

  // in this case, the dict is untyped, so it should just appear as string
  flag= get_row(vinsp, 1, name, value);
  ensure("get/set k2", flag);
  ensure_equals("get/set k2", value, "112233");
  type= vinsp->get_field_type(1, ValueInspectorBE::Value);
  ensure_equals("get/set k2", type, IntegerType);

  // item count still ok?

  ensure_equals("item count after changes", (int)dict.count(), 9);
  
  // test add new value
  NodeId nkey;
  flag= vinsp->add_item(nkey);
  ensure("add ok", flag);
  ensure_equals("add", nkey[0], 9);

  flag= vinsp->set_field(nkey, ValueInspectorBE::Value, "new value");
  ensure("set item", !flag);

  flag= vinsp->set_field(nkey, ValueInspectorBE::Value, "1234");
  ensure("set item", !flag);

  flag= vinsp->set_field(nkey, ValueInspectorBE::Name, "anewk");
  ensure("set item", flag);

  flag= vinsp->set_field(nkey, ValueInspectorBE::Value, 1234);
  ensure("set item", flag);

  ensure_equals("new count", (int)dict.count(), 10);

  flag= get_row(vinsp, nkey, name, value);
  ensure("get new", flag);
  ensure_equals("new value", name, "anewk");
  ensure_equals("new value", value, "1234");
  type= vinsp->get_field_type(nkey, ValueInspectorBE::Value);
  ensure_equals("new type", type, IntegerType);

  flag= vinsp->add_item(nkey);
  ensure("add", flag);

  flag= vinsp->add_item(nkey);
  ensure("add ng", !flag);

  vinsp->refresh();

  flag= vinsp->add_item(nkey);
  ensure("add", flag);

  flag= get_row(vinsp, nkey, name, value);
  ensure("get new", flag);
  ensure_equals("new value", name, "");
  ensure_equals("new value", value, "NULL");
  type= vinsp->get_field_type(nkey, ValueInspectorBE::Value);
  ensure_equals("new type", type, IntegerType);

  flag= vinsp->add_item(nkey);
  ensure("add again", !flag);

  flag= vinsp->set_convert_field(nkey, ValueInspectorBE::Name, "aaa");
  flag= vinsp->set_convert_field(nkey, ValueInspectorBE::Value, "22");
  ensure("set item", flag);

  flag= get_row(vinsp, nkey, name, value);
  ensure("get new 2", flag);
  ensure_equals("new value", name, "aaa");
  ensure_equals("new value", value, "22");
  type= vinsp->get_field_type(nkey, ValueInspectorBE::Value);
  ensure_equals("new type", type, IntegerType);
  
  
  // test delete value
  flag= vinsp->delete_item(3);
  ensure("del", flag);

  ensure_equals("count", (int)dict.count(), 10);

  flag= vinsp->delete_item(10);
  ensure("bad del", !flag);

  flag= vinsp->delete_item(9);
  ensure("del last", flag);

  delete vinsp;
  
  ensure_equals("final count", (int)dict.count(), 9);
}


// Test Objects
// ------------

TEST_FUNCTION(20)
{
  // test inspection of object (ungrouped)

  // Load test meta classes.
  grt.load_metaclasses("data/structs.test.xml");

  // Consolidate the loaded classes.
  grt.end_loading_metaclasses();

  test_BookRef book(&grt);
  ensure("object valid", book.is_valid());

  ValueInspectorBE *vinsp= ValueInspectorBE::create(&grt, book, false);
  bool flag;
  int c;

  // test listing
  c= vinsp->count();
  ensure_equals("begin listing", c, 6);
  
  std::string name, value;
  Type type;

  NodeId node;

  node= vinsp->get_node(0);
  flag= get_row(vinsp, node, name, value);
  ensure("item", flag);
  ensure_equals("name", name, "authors");
  ensure_equals("value", value, "[  ]");
  type= vinsp->get_field_type(0, ValueInspectorBE::Value);
  ensure_equals("type", type, ListType);
  c= vinsp->count_children(node);
  ensure_equals("children", c, 0);

  node= vinsp->get_node(1);
  flag= get_row(vinsp, node, name, value);
  ensure("item", flag);
  ensure_equals("name", name, "extras");
  ensure_equals("value", value, "{  }");
  type= vinsp->get_field_type(1, ValueInspectorBE::Value);
  ensure_equals("type", type, DictType);
  c= vinsp->count_children(node);
  ensure_equals("children", c, 0);

  node= vinsp->get_node(2);
  flag= get_row(vinsp, node, name, value);
  ensure("item", flag);
  ensure_equals("name", name, "pages");
  ensure_equals("value", value, "NULL");
  type= vinsp->get_field_type(2, ValueInspectorBE::Value);
  ensure_equals("type", type, IntegerType);
  c= vinsp->count_children(node);
  ensure_equals("children", c, 0);

  node= vinsp->get_node(3);
  flag= get_row(vinsp, node, name, value);
  ensure("item", flag);
  ensure_equals("name", name, "price");
  ensure_equals("value", value, "NULL");
  type= vinsp->get_field_type(3, ValueInspectorBE::Value);
  ensure_equals("type", type, DoubleType);
  c= vinsp->count_children(node);
  ensure_equals("children", c, 0);

  node= vinsp->get_node(4);
  flag= get_row(vinsp, node, name, value);
  ensure("item", flag);
  ensure_equals("name", name, "publisher");
  ensure_equals("value", value, "NULL");
  c= vinsp->count_children(node);
  ensure_equals("children", c, 0);

  node= vinsp->get_node(5);
  flag= get_row(vinsp, node, name, value);
  ensure("item", flag);
  ensure_equals("name", name, "title");
  ensure_equals("value", value, "");
  type= vinsp->get_field_type(5, ValueInspectorBE::Value);
  ensure_equals("type", type, StringType);
  c= vinsp->count_children(node);
  ensure_equals("children", c, 0);

  // bad value
  flag= get_row(vinsp, 6, name, value);
  ensure("end item", !flag);

  // test get random item
  flag= get_row(vinsp, 2, name, value);
  ensure("get", flag);
  ensure_equals("get", name, "pages");
  ensure_equals("get", value, "NULL");
  type= vinsp->get_field_type(2, ValueInspectorBE::Value);
  ensure_equals("get", type, IntegerType);

  flag= get_row(vinsp, 9, name, value);
  ensure("bad get", !flag);

  flag= get_row(vinsp, -1, name, value);
  ensure("bad get", !flag);

  // test bad change int value
  flag= vinsp->set_field(5, ValueInspectorBE::Value, 112211);
  ensure("set invalid 1", !flag);

  flag= get_row(vinsp, 5, name, value);
  ensure("get/set title", flag);
  ensure_equals("get/set title", name, "title");
  ensure_equals("get/set title", value, "");
  type= vinsp->get_field_type(5, ValueInspectorBE::Value);
  ensure_equals("get/set title", type, StringType);

  flag= vinsp->set_field(5, ValueInspectorBE::Value, "The Illiad");
  ensure("set title", flag);

  flag= vinsp->get_field(5, ValueInspectorBE::Value, value);
  ensure("get/set title", flag);
  ensure_equals("get/set title", value, "The Illiad");

  // test change int value
  flag= vinsp->set_field(2, ValueInspectorBE::Value, 123);
  ensure("set pages", flag);

  flag= get_row(vinsp, 2, name, value);
  ensure("get/set pages", flag);
  ensure_equals("get/set pages", name, "pages");
  ensure_equals("get/set pages", value, "123");
  type= vinsp->get_field_type(2, ValueInspectorBE::Value);
  ensure_equals("get/set pages", type, IntegerType);

  // test change double value
  flag= vinsp->set_field(3, ValueInspectorBE::Value, 1.234);
  ensure("set price", flag);

  flag= get_row(vinsp, 3, name, value);
  ensure("get/set price", flag);
  ensure_equals("get/set price", value, "1.234");
  type= vinsp->get_field_type(3, ValueInspectorBE::Value);
  ensure_equals("get/set price", type, DoubleType);

  // test change int value from string
  flag= vinsp->set_convert_field(2, ValueInspectorBE::Value, "112233");
  ensure("set convert pages", flag);

  // in this case, the dict is untyped, so it should just appear as string
  flag= get_row(vinsp, 2, name, value);
  ensure("get/set price", flag);
  ensure_equals("get/set price", value, "112233");
  type= vinsp->get_field_type(2, ValueInspectorBE::Value);
  ensure_equals("get/set pages", type, IntegerType);

  // test add new value
  NodeId nkey;
  flag= vinsp->add_item(nkey);
  ensure("add ok", !flag);

  // test delete value
  flag= vinsp->delete_item(3);
  ensure("del", !flag);

  delete vinsp;
}




TEST_FUNCTION(21)
{
  // test inspection of object (grouped)
  
  test_BookRef book(&grt);
  ensure("object valid", book.is_valid());

  ValueInspectorBE *vinsp= ValueInspectorBE::create(&grt, book, true);

  // test listing
  int c= vinsp->count();
  ensure_equals("top listing", c, 3);

  std::string name, value;
  Type type;
  bool flag;

  NodeId node, gnode;
  
  // ungrouped
  gnode= vinsp->get_node(0);
  flag= vinsp->get_field(gnode, ValueInspectorBE::Name, name);
  ensure("group", flag);
  ensure_equals("group name", name, "");
  c= vinsp->count_children(gnode);
  ensure_equals("group children", c, 1);

  node= vinsp->get_child(gnode, 0);
  flag= get_row(vinsp, node, name, value);
  ensure("item", flag);
  ensure_equals("name", name, "title");
  ensure_equals("value", value, "");
  type= vinsp->get_field_type(node, ValueInspectorBE::Value);
  ensure_equals("type", type, StringType);
  c= vinsp->count_children(node);
  ensure_equals("children", c, 0);

  // group1

  gnode= vinsp->get_node(1);
  flag= vinsp->get_field(gnode, ValueInspectorBE::Name, name);
  ensure("group", flag);
  ensure_equals("group name", name, "group1");
  c= vinsp->count_children(gnode);
  ensure_equals("group children", c, 3);

  node= vinsp->get_child(gnode, 0);
  flag= get_row(vinsp, node, name, value);
  ensure("item", flag);
  ensure_equals("name", name, "authors");
  ensure_equals("value", value, "[  ]");
  type= vinsp->get_field_type(node, ValueInspectorBE::Value);
  ensure_equals("type", type, ListType);
  c= vinsp->count_children(node);
  ensure_equals("children", c, 0);

  node= vinsp->get_child(gnode, 1);
  flag= get_row(vinsp, node, name, value);
  ensure("item", flag);
  ensure_equals("name", name, "pages");
  ensure_equals("value", value, "NULL");
  type= vinsp->get_field_type(node, ValueInspectorBE::Value);
  ensure_equals("type", type, IntegerType);
  c= vinsp->count_children(node);
  ensure_equals("children", c, 0);

  node= vinsp->get_child(gnode, 2);
  flag= get_row(vinsp, node, name, value);
  ensure("item", flag);
  ensure_equals("name", name, "price");
  ensure_equals("value", value, "NULL");
  type= vinsp->get_field_type(node, ValueInspectorBE::Value);
  ensure_equals("type", type, DoubleType);
  c= vinsp->count_children(node);
  ensure_equals("children", c, 0);


  // group2

  gnode= vinsp->get_node(2);
  flag= vinsp->get_field(gnode, ValueInspectorBE::Name, name);
  ensure("group", flag);
  ensure_equals("group name", name, "group2");
  c= vinsp->count_children(gnode);
  ensure_equals("group children", c, 2);


  node= vinsp->get_child(gnode, 0);
  flag= get_row(vinsp, node, name, value);
  ensure("item", flag);
  ensure_equals("name", name, "extras");
  ensure_equals("value", value, "{  }");
  type= vinsp->get_field_type(node, ValueInspectorBE::Value);
  ensure_equals("type", type, DictType);
  c= vinsp->count_children(node);
  ensure_equals("children", c, 0);


  node= vinsp->get_child(gnode, 1);
  flag= get_row(vinsp, node, name, value);
  ensure("item", flag);
  ensure_equals("name", name, "publisher");
  ensure_equals("value", value, "NULL");
  c= vinsp->count_children(node);
  ensure_equals("children", c, 0);


  // bad value
  flag= get_row(vinsp, 4, name, value);
  ensure("end item", !flag);

  // test get random item
  flag= get_row(vinsp, vinsp->get_child(NodeId(1), 1), name, value);
  ensure("get", flag);
  ensure_equals("get", name, "pages");
  ensure_equals("get", value, "NULL");
  type= vinsp->get_field_type(vinsp->get_child(NodeId(1), 1), ValueInspectorBE::Value);
  ensure_equals("get", type, IntegerType);

  try {
    get_row(vinsp, vinsp->get_child(NodeId(1), 3), name, value);
    fail("Invalid item");
  } catch (std::range_error)
  {
    // expected
  }

  flag= get_row(vinsp, -1, name, value);
  ensure("bad get", !flag);

  // test bad change int value
  flag= vinsp->set_field(0, ValueInspectorBE::Value, 112211);
  ensure("set invalid 1", !flag);

  node= vinsp->get_child(0, 0);

  flag= get_row(vinsp, node, name, value);
  ensure("get/set title", flag);
  ensure_equals("get/set title", name, "title");
  ensure_equals("get/set title", value, "");
  type= vinsp->get_field_type(node, ValueInspectorBE::Value);
  ensure_equals("get/set title", type, StringType);

  flag= vinsp->set_field(node, ValueInspectorBE::Value, "The Illiad");
  ensure("set title", flag);

  flag= vinsp->get_field(node, ValueInspectorBE::Value, value);
  ensure("get/set title", flag);
  ensure_equals("get/set title", value, "The Illiad");

  // test change int value
  node= vinsp->get_child(1, 1);
  flag= vinsp->set_field(node, ValueInspectorBE::Value, 123);
  ensure("set pages", flag);

  flag= get_row(vinsp, node, name, value);
  ensure("get/set pages", flag);
  ensure_equals("get/set pages", name, "pages");
  ensure_equals("get/set pages", value, "123");
  type= vinsp->get_field_type(node, ValueInspectorBE::Value);
  ensure_equals("get/set pages", type, IntegerType);

  // test change double value
  node= vinsp->get_child(1, 2);
  flag= vinsp->set_field(node, ValueInspectorBE::Value, 1.234);
  ensure("set price", flag);

  flag= get_row(vinsp, node, name, value);
  ensure("get/set price", flag);
  ensure_equals("get/set price", value, "1.234");
  type= vinsp->get_field_type(node, ValueInspectorBE::Value);
  ensure_equals("get/set price", type, DoubleType);

  // test change int value from string
  node= vinsp->get_child(1, 1);
  flag= vinsp->set_convert_field(node, ValueInspectorBE::Value, "112233");
  ensure("set convert pages", flag);

  // in this case, the dict is untyped, so it should just appear as string
  flag= get_row(vinsp, node, name, value);
  ensure("get/set price", flag);
  ensure_equals("get/set price", value, "112233");
  type= vinsp->get_field_type(node, ValueInspectorBE::Value);
  ensure_equals("get/set pages", type, IntegerType);

  // test change group name
  node= vinsp->get_node(1);
  flag= vinsp->set_convert_field(node, ValueInspectorBE::Value, "HELLO");
  ensure("set group name", !flag);

  
  // test add new value
  NodeId nkey;
  flag= vinsp->add_item(nkey);
  ensure("add ok", !flag);

  // test delete value
  flag= vinsp->delete_item(3);
  ensure("del", !flag);

  delete vinsp;
}





END_TESTS
