#include "stdafx.h"
#include "test.h"
#include "mdc.h"
#include <myx_grtpp.h>
#include <mdc_canvas_view_image.h>

#include "wbcanvas/model_layer.h"
#include "wbcanvas/workbench_physical_model.h"
#include "wbcanvas/workbench_physical_view.h"
#include "wbcanvas/workbench_physical_tablefigure.h"
#include "wbcanvas/workbench_physical_viewfigure.h"
#include "wbcanvas/workbench_physical_routinegroupfigure.h"

#include "grts/structs.db.mysql.h"
#include "grts/structs.workbench.model.h"

using namespace grt;
using namespace mdc;


class Delegate : public wbbridge::physical::ModelBridgeDelegate
{
public:
  ImageCanvasView *view;

  Delegate(ImageCanvasView *aView) : view(aView) {}
  virtual mdc::CanvasView *create_canvas_view(const std::string &id, const std::string &name)
  {
    return view;
  }
  
  virtual void free_canvas_view(mdc::CanvasView *view)
  {
    delete view;
  }
  
  virtual void activate_canvas_figure(const model_Figure &figure)
  {
  }
  
  virtual void handle_fklist_change(const db_Table&, const db_ForeignKey&, bool)
  {
  }

  virtual void notify_figure_change(mdc::CanvasView*, const model_Figure&) {}
  virtual void notify_layer_change(mdc::CanvasView*, const model_Layer&) {}
  virtual void notify_selection_change(mdc::CanvasView*, const model_View&) {}
  virtual bec::UndoManager* get_model_undo_manager() { return 0; }
  virtual void notify_dbobject_figure(const db_DatabaseObject&, const model_Figure&, bool) {}

};

BEGIN_TEST_DATA_CLASS(wbcanvas_bridge)
public:
  GRT grt;

  workbench_physical_Model model;
  workbench_physical_View model_view;

  ImageCanvasView view;
  Layer *layer;

  workbench_physical_TableFigure table_item_grt;
  mdc::CanvasItem *table_item;

  Delegate delegate;

TEST_DATA_CONSTRUCTOR(wbcanvas_bridge)
  : view(0, 500, 400), delegate(&view)
{
  view.initialize();
  layer= view.get_current_layer();
  
  table_item= 0;
}
END_TEST_DATA_CLASS


TEST_MODULE(wbcanvas_bridge, "wbcanvas bridge");


TEST_FUNCTION(1)
{ // initialize

  int i= grt.scan_structs_from("../../res/grt/");
  ensure("scan structs", i>0);

  ObjectBridgeBase::register_bridge<wbbridge::physical::Model>(&grt, &delegate);
  ObjectBridgeBase::register_bridge<wbbridge::physical::View>(&grt);
  ObjectBridgeBase::register_bridge<wbbridge::physical::TableFigure>(&grt);
  
  ObjectBridgeBase::register_bridge<wbbridge::Layer>(&grt);
}


TEST_FUNCTION(2)
{ // setup the model and a view

  model= workbench_physical_Model(&grt);

  ensure("physical model", model.is_valid());
  ensure("model has a bridge", model.get_bridge_private()!=0);
  ensure_equals("model has no view", model.views().count(), 0U);

  // now add a view wrapping the pre-created one

  DictValue args(&grt);

  args.set("*createnow", IntValue(1));
  args.set("owner", model);

  workbench_physical_View pview(&grt, args);
  
  model_view= pview;

  wbbridge::physical::View *vbridge= dynamic_cast<wbbridge::physical::View*>(pview.get_bridge_private());

  ensure("view bridge", vbridge != 0);
  ensure_equals("wrapped canvas view", vbridge->get_canvas_view(), &view);


  workbench_model_Layer layer;

  layer= workbench_model_Layer::cast_from(pview.rootLayer());
  ensure("root layer auto-creation", layer.is_valid());
}



TEST_FUNCTION(3)
{ // add a table

  ensure("check model", model.is_valid());

  DictValue args(&grt);

  args.set("owner", model_view);

  workbench_physical_TableFigure table(&grt, args);

  table_item_grt= table;
  
  ensure("table figure", table.is_valid());

  // set the table
  db_mysql_Table mtable(&grt);

  mtable.name("employee");

  table.color("#aabbcc");
  table.table(mtable);

  ensure("table figure table", table.table().is_valid());
  ensure("table figure table", table.table()==mtable);

  wbbridge::physical::TableFigure *tabbri= dynamic_cast<wbbridge::physical::TableFigure*>(table.get_bridge_private());
  ensure("table bridge", tabbri!=0);

  table.layer(model_view.rootLayer());

  ensure("table is in view", tabbri->get_canvas_view()!=0);
  
 //check ensure("table is in layer", tabbri->get_layer_bridge()!=0);

  ensure("table is realizable", tabbri->is_realizable());

  wbfig::Table *tabfig= dynamic_cast<wbfig::Table*>(tabbri->get_canvas_item());
  ensure("table figure realized", tabfig!=0);

  ensure_equals("table title", 
                tabfig->get_title()->get_title(), "employee");


  table_item= tabfig;
  
  ensure_equals("rootLayer figures", model_view.rootLayer().figures().count(), 1U);

  // change grt object

  mtable.name("person");

  grt.flush_notifications();

  ensure_equals("new table name", 
                *mtable.name(), "person");
  
  ensure_equals("new table title", 
                tabfig->get_title()->get_title(), "person");

  table.top(50);
  table.left(50);


  // add some columns
  db_mysql_Column columns[3]= { db_mysql_Column(&grt), db_mysql_Column(&grt), db_mysql_Column(&grt) };

  columns[0].owner(table);
  columns[0].name("id");

  columns[1].owner(table);
  columns[1].name("email");

  columns[2].owner(table);
  columns[2].name("address");

  mtable.columns().insert(columns[0]);
  mtable.columns().insert(columns[1]);
  mtable.columns().insert(columns[2]);



  // change canvas object

  tabfig->move_to(mdc::Point(100, 100));

  ensure_equals("grt update after canvas change", *table.top(), 100.0);
  ensure_equals("grt update after canvas change", *table.left(), 100.0);

  // dump final state
  view.save_to("bridge_table.png");
}



TEST_FUNCTION(4)
{ // add a layer

  DictValue args(&grt);

  args.set("owner", model_view);
  args.set("top", DoubleValue(10));
  args.set("left", DoubleValue(200));
  args.set("width", DoubleValue(200));
  args.set("height", DoubleValue(300));
  args.set("color", StringValue("#ffffdd"));

  workbench_model_Layer layer(&grt, args);

  mdc::AreaGroup *ag= dynamic_cast<wbbridge::Layer*>(layer.get_bridge_private())->get_area_group();
  ensure("area group", ag!=0);

  layer.name("test layer");

  model_view.rootLayer().subLayers().insert(layer);
  
  ensure_equals("layer count", model_view.layers().count(), 2U);

  ensure_equals("layer 0 name", *model_view.layers().get(0).name(), "");
  ensure_equals("layer 1 name", *model_view.layers().get(1).name(), "test layer");

  ensure_equals("area group pos", ag->get_position().x, *layer.left());
  ensure_equals("area group pos", ag->get_position().y, *layer.top());

  ensure_equals("area group size", ag->get_size().width, *layer.width());
  ensure_equals("area group size", ag->get_size().height, *layer.height());

  // move the layer from the grt
  layer.left(150);
  ensure_equals("changed layer pos", ag->get_position().x, 150.0);

  layer.width(210);
  ensure_equals("resize layer", ag->get_size().width, 210.0);

  // move layer from canvas
  ag->move_to(mdc::Point(180, 10));
  ensure_equals("moved layer", ag->get_position().x, 180.0);
  ensure_equals("moved layer", *layer.left(), 180.0);



  // add a table in the layer
  args.init(&grt);

  args.set("owner", model_view);

  workbench_physical_TableFigure table(&grt, args);
  ensure("table figure", table.is_valid());

  // set the table
  db_mysql_Table mtable(&grt);
  mtable.name("deparment");
  table.color("#aabbcc");
  table.table(mtable);

  table.layer(layer);

  table.top(10);
  table.left(10);

  ensure_equals("model elem count", model_view.figures().count(), 2U);
  ensure_equals("model layer count", model_view.layers().count(), 2U);

  ensure("figure0 layer", model_view.figures().get(0).layer() == model_view.rootLayer());

  ensure_equals("rootLayer elem count", model_view.rootLayer().figures().count(), 1U);
  
  ensure_equals("rootlayer layer count", model_view.rootLayer().subLayers().count(), 1U);

  ensure_equals("new layer elem count", layer.figures().count(), 1U);
  ensure_equals("new layer sublayer count", layer.subLayers().count(), 0U);

  view.save_to("bridge_layer.png");
}


TEST_FUNCTION(5)
{ // test add/remove item from canvas

  std::string id;
  
  ensure_equals("model elem count", model_view.figures().count(), 2U);

  {
    DictValue args= DictValue::create(&grt, "owner", model_view.grt_value(), NULL);
    workbench_physical_TableFigure table(&grt, args);
  
    db_mysql_Table db_table(&grt);
    table.table(db_table);

    table.layer(model_view.rootLayer());
    
    mdc::CanvasItem *item= dynamic_cast<wbbridge::FigureBase*>(table.get_bridge_private())->get_canvas_item();

    id= table.id();
    
    ensure("table canvas item", item!=0);
    ensure_equals("check tag", item->get_tag(), id);

    ensure_equals("model elem count", model_view.figures().count(), 3U);
  
    mdc::CanvasItem *tmp;
    tmp= view.find_item_with_tag(id);
    ensure("table in canvas", tmp!=0);

    // remove
    model_view.figures().remove_value(table);
    table.layer().figures().remove_value(table);

    ensure_equals("elem count in view", model_view.figures().count(), 2U);
  }

  mdc::CanvasItem *tmp;
  tmp= view.find_item_with_tag(id);
  ensure("check if table deleted from canvas", tmp==0);
  
}


TEST_FUNCTION(10)
{ // test canvas <-> GRT selection updating
  
  ensure_equals("model elem count", model_view.figures().count(), 2U);

  ensure("table item handle", table_item!=0);

  size_t c;

  // With 1 item
  
  // select table in canvas, check GRT
  view.get_selection()->add(table_item);

  c= view.get_selected_items().size();
  ensure_equals("selection count", c, 1U);

  ensure("view selection contents", *view.get_selected_items().begin()==table_item);

  c= model_view.selection().count();
  ensure_equals("grt selection size", c, 1U);

  // unselect in canvas, check GRT
  view.get_selection()->remove(table_item);
  c= view.get_selected_items().size();
  ensure_equals("selection count", c, 0U);
  c= model_view.selection().count();
  ensure_equals("grt selection size", c, 0U);

  // select again in canvas
  view.get_selection()->add(table_item);

  c= view.get_selected_items().size();
  ensure_equals("selection count", c, 1U);

  // unselect from GRT
  model_view.selection().remove_value(table_item_grt);

  c= model_view.selection().count();
  ensure_equals("grt selection size", c, 0U);

  c= view.get_selected_items().size();
  ensure_equals("selection count", c, 0U);

  // select from GRT
  model_view.selection().insert(table_item_grt);
  
  c= model_view.selection().count();
  ensure_equals("grt selection size", c, 1U);

  c= view.get_selected_items().size();
  ensure_equals("selection count", c, 1U);
  
  
  // With 2 items
  DictValue args= DictValue::create(&grt, "owner", model_view.grt_value(), NULL);
  workbench_physical_TableFigure table2_grt(&grt, args);
 // mdc::CanvasItem *table2;
  
  ensure_equals("model elem count", model_view.figures().count(), 3U);
}


END_TESTS
