#include "stdafx.h"

#pragma warning(disable: 4793)  // 'vararg' causes native code generation

#include "Grt.h"
#include "IconManager.h"
#include "TreeModel.h"
#include "GrtValueTree.h"
#include "GrtValueInspector.h"
#include "GrtStructsTree.h"
#include "GrtModulesTree.h"
#include "GrtShell.h"
#include "GrtManager.h"
#include "Exceptions.h"
#include "UiForm.h"
#include "BaseEditor.h"


namespace MySQL {
namespace Grt {


String^ GrtUtil::CppStringToNative(const std::string &input)
{
  return GrtUtil::CharToString(input.c_str(), input.size());
}

std::string GrtUtil::NativeToCppString(String^ input)
{
  char *c_str= GrtUtil::StringToChar(input);
  std::string result= c_str;
  delete[] c_str;
  return result;
}

char* GrtUtil::StringToChar(String^ str)
  {
    if (!str)
      return NULL;

    // Pin memory so GC can't move it while native function is called
    pin_ptr<const wchar_t> wch = PtrToStringChars(str);

    // Conversion to char* :
    // Can just convert wchar_t* to char* using one of the 
    // conversion functions such as: 
    // WideCharToMultiByte()
    // wcstombs_s()
    // ... etc

    int sizeInBytes = ((str->Length + 1) * 4); // utf8 can have up to 4 bytes per char (not 2)
    errno_t err = 0;
    char    *ch = new char[sizeInBytes];

    size_t convertedChars = WideCharToMultiByte(CP_UTF8, 0, wch, -1, ch, sizeInBytes, 0, 0);
    //if (err != 0)
    //  throw ...

    // compress \r\n to \n
    char *from, *to;

    for (from= to= ch; *from != '\0'; ++from, ++to)
    {
      if (*from == '\r' && *(from+1)=='\n')
        ++from;
      *to= *from;
    }
    *to= 0;

    return ch;
  }

  String^ GrtUtil::CharToString(const char *str)
  {
    // TODO optimize here - do not copy string creating std::string
    std::string res= bec::replace_string(str, "\n", "\r\n");

    return gcnew String(res.c_str(), 0, (int) res.size(), gcnew System::Text::UTF8Encoding(true, true));
  }

  String^ GrtUtil::CharToString(const char *str, size_t length)
  {
    // TODO optimize here - do not copy string creating std::string
    std::string res= bec::replace_string(str, "\n", "\r\n");

    // TODO is there a faster way to cache this encoding obj?
    return gcnew String(res.c_str(), 0, (int) res.size(), gcnew System::Text::UTF8Encoding(true, true));
  }
}

}