#ifndef __GRT_TEMPLATES_H__
#define __GRT_TEMPLATES_H__

// This file should not contain any compilable .net public code
// because it is included by many projects, and any .net public code
// will cause conflicts
using namespace System;
using namespace System::Collections::Generic;

namespace MySQL {
namespace Grt {

static String ^ CppStringToNative(const std::string &input)
{
  return GrtUtil::CharToString(input.c_str());
}

static std::string NativeToCppString(String ^input)
{
  char *c_str= GrtUtil::StringToChar(input);
  std::string result= c_str;
  delete[] c_str;
  return result;
}

static List<String^>^ CppStringListToNative(const std::vector<std::string> &input)
{
  int cap = static_cast<int>(input.size());
  List<String^>^ result= gcnew List<String^>(cap);
  std::vector<std::string>::const_iterator e= input.end();
  
  for (std::vector<std::string>::const_iterator i= input.begin(); i != e; i++)
    result->Add(GrtUtil::CharToString(i->c_str()));
  return result;
}

static std::vector<std::string> NativeToCppStringList(List<String^>^ input)
{
  std::vector<std::string> result;
  result.reserve(input->Count);
  
  for each(String ^str_item in input)
  {
    char *c_str= GrtUtil::StringToChar(str_item);
    result.push_back(c_str);
    delete[] c_str;
  }
  
  return result;
}

static std::list<std::string> NativeToCppStringList2(List<String^>^ input)
{
  std::list<std::string> result;
  
  for each(String ^str_item in input)
  {
    char *c_str= GrtUtil::StringToChar(str_item);
    result.push_back(c_str);
    delete[] c_str;
  }
  
  return result;
}

static List<String^>^ CppStringListToNative2(const std::list<std::string> &input)
{
  int cap = static_cast<int>(input.size());
  List<String^>^ result= gcnew List<String^>(cap);
  std::list<std::string>::const_iterator e= input.end();
  
  for (std::list<std::string>::const_iterator i= input.begin(); i != e; i++)
    result->Add(GrtUtil::CharToString(i->c_str()));
  return result;
}

static List<int>^ CppVectorToIntList(const std::vector<int>& input)
{
  typedef const std::vector<int> SourceContainerType;
  typedef List<int> TargetContainerType;

  TargetContainerType^ result= gcnew TargetContainerType(static_cast<int>(input.size()));
  SourceContainerType::const_iterator e= input.end();
  
  for(SourceContainerType::const_iterator i= input.begin(); i != e; i++)
    result->Add(*i);

  return result;
}

static std::vector<int> IntListToCppVector(List<int>^ input)
{
  typedef const List<int>^ SourceContainerType;
  typedef std::vector<int> TargetContainerType;

  TargetContainerType result;
  result.reserve(static_cast<int>(input->Count));
  
  for each(int listitem in input)
    result.push_back(listitem);

  return result;
}

template<typename S, typename T>
static List<T^>^ CppVectorToObjectList(const std::vector<S>& input)
{
  typedef const std::vector<S> SourceContainerType;
  typedef List<T^> TargetContainerType;

  TargetContainerType^ result= gcnew TargetContainerType(static_cast<int>(input.size()));
  SourceContainerType::const_iterator e= input.end();
  
  for(SourceContainerType::const_iterator i= input.begin(); i != e; i++)
    result->Add(gcnew T(*i));

  return result;
}

// this conversion function is for simple types that map managed objects to
// a native c++ object used by value, such as e.g. ::bec::NodeId

template<typename S, typename T>
static std::vector<T> ObjectListToCppVector(List<S^>^ input)
{
  typedef const List<S^>^ SourceContainerType;
  typedef std::vector<T> TargetContainerType;

  TargetContainerType result;
  result.reserve(static_cast<int>(input->Count));
  
  for each(S^ listitem in input)
    result.push_back(*listitem->get_unmanaged_object());

  return result;
}

template<typename S, typename T>
static List<T^>^ CppListToObjectList(const std::list<S>& input)
{
  typedef const std::list<S> SourceContainerType;
  typedef List<T^> TargetContainerType;

  TargetContainerType^ result= gcnew TargetContainerType(static_cast<int>(input.size()));
  SourceContainerType::const_iterator e= input.end();
  
  for(SourceContainerType::const_iterator i= input.begin(); i != e; i++)
    result->Add(gcnew T(*i));

  return result;
}

// this conversion function is for simple types that map managed objects to
// a native c++ object used by value, such as e.g. ::bec::NodeId

template<typename S, typename T>
static std::list<T> ObjectListToCppList(List<S^>^ input)
{
  typedef const List<S^>^ SourceContainerType;
  typedef std::list<T> TargetContainerType;

  TargetContainerType result(static_cast<int>(input->Count));
  
  for each(S^ listitem in input)
    result.push_back(*listitem->get_unmanaged_object());

  return result;
}


template<typename S, typename T>
static List<T>^ CppListToNativeList(const std::list<S>& input)
{
  typedef const std::list<S> SourceContainerType;
  typedef List<T> TargetContainerType;

  TargetContainerType^ result= gcnew TargetContainerType(static_cast<int>(input.size()));
  SourceContainerType::const_iterator e= input.end();
  
  for(SourceContainerType::const_iterator i= input.begin(); i != e; i++)
    result->Add(T(*i));

  return result;
}

// this conversion function is for simple types that map managed objects to
// a native c++ object used by value, such as e.g. ::bec::NodeId

template<typename S, typename T>
static std::list<T> NativeListToCppList(List<S>^ input)
{
  typedef const List<S>^ SourceContainerType;
  typedef std::list<T> TargetContainerType;

  TargetContainerType result(static_cast<int>(input->Count));

  for each(S listitem in input)
    result.push_back(listitem);

  return result;
}


template <class T>
struct EditorCentry
{
  T *editor;
  EditorCentry(T *editor)
    : editor(editor)
  {
    editor->DisableAutoRefresh();
  }
  ~EditorCentry()
  {
    editor->EnableAutoRefresh();
  }
  T& operator *() { return *editor; }
  T& operator ->() { return *editor; }
  operator T* () {return editor;}
  operator T& () {return *editor;}
};

} // namespace Grt
} // namespace MySQL

#endif // __GRT_TEMPLATES_H__
