#include "stdafx.h"

#include "wb_context_basic.h"

#include "grts/structs.workbench.model.h"

#include "wbcanvas/model_layer.h"
#include "wbcanvas/model_view.h"
#include "wbcanvas/workbench_model_notefigure.h"
#include "wbcanvas/workbench_model_imagefigure.h"


using namespace wb;
using namespace grt;
using namespace bec;


struct wb::HandToolContext
{
  mdc::Point mouse_pos;
  mdc::Point viewport_pos;
  bool moving;

  HandToolContext()
    : mouse_pos(0., 0.), viewport_pos(0, 0), moving(false)
  {
  }

};



WBContextBasic::WBContextBasic(bool verbose)
: WBContextBase(verbose)
{
}


WBContextBasic::~WBContextBasic()
{
}


void WBContextBasic::init_canvas_grt(grt::GRT *grt, WBOptions *options)
{
  ObjectBridgeBase::register_bridge<wbbridge::Layer>(grt);
  ObjectBridgeBase::register_bridge<wbbridge::NoteFigure>(grt);
  ObjectBridgeBase::register_bridge<wbbridge::ImageFigure>(grt);
}


void WBContextBasic::load_app_options(bool update)
{
  if (!update)
  {
    app_ToolbarRef tools_toolbar(app_ToolbarRef::cast_from(
      _manager->get_grt()->unserialize("data/tools_toolbar_basic.xml")));

    _toolbars[tools_toolbar.name()]= tools_toolbar;

    app_ToolbarRef options_toolbar;

    options_toolbar= app_ToolbarRef::cast_from(
      _manager->get_grt()->unserialize("data/main_toolbar_layer.xml"));
    _toolbars[options_toolbar.name()]= options_toolbar;

    options_toolbar= app_ToolbarRef::cast_from(
      _manager->get_grt()->unserialize("data/main_toolbar_note.xml"));
    _toolbars[options_toolbar.name()]= options_toolbar;
  }
}


void WBContextBasic::set_toolbar_option(const std::string &option, const std::string &item)
{
  set_canvas_tool_argument(get_active_canvas_view(), option, item);
}


void WBContextBasic::set_canvas_tool(mdc::CanvasView *view, const std::string &tool)
{
  if (tool == WB_TOOL_HAND)
  {
    _view_data[view].hcontext= new HandToolContext;
    _view_data[view].cursor= "hand";
    show_status_text(_("Drag the canvas to move it around."));
  }
  else if (tool == WB_TOOL_DELETE)
  {
    _view_data[view].cursor= "rubber";
    show_status_text(_("Click the object to delete."));
  }
  else if (tool == WB_TOOL_LAYER)
  {
    _view_data[view].cursor= "layer";
    show_status_text(_("Select an area for the new layer."));
  }
  else if (tool == WB_TOOL_NOTE)
  {
    _view_data[view].cursor= "note";
    show_status_text(_("Select an area for a note object."));
  }
  else if (tool == WB_TOOL_IMAGE)
  {
    _view_data[view].cursor= "image";
    show_status_text(_("Select a location for the image object."));
  }
  else
  {
    show_status_text("Invalid tool "+tool);
    return;
  }

  _view_data[view].handle_button= sigc::mem_fun(this, &WBContextBasic::handle_button_event);
  _view_data[view].handle_motion= sigc::mem_fun(this, &WBContextBasic::handle_motion_event);
  _view_data[view].reset_tool= sigc::mem_fun(this, &WBContextBasic::reset_tool);
}


grt::ListRef<app_ToolbarItem> WBContextBasic::get_toolbar_items_for_view(const std::string &toolbar)
{
  if (toolbar == WB_TOOLBAR_MAIN)
  {
    if (_toolbars.find(toolbar+"/"+get_canvas_tool(get_active_canvas_view())) == _toolbars.end())
      return grt::ListRef<app_ToolbarItem>();
    return _toolbars[toolbar+"/"+get_canvas_tool(get_active_canvas_view())].items();
  }
  else
    return _toolbars[toolbar].items();
}


std::vector<std::string> WBContextBasic::get_dropdown_items(const std::string &option, std::string &selected)
{
  std::vector<std::string> items;
  if (has_suffix(option, ":Color"))
  {
    items.push_back("#ffaaaa");
    items.push_back("#ffffaa");
    items.push_back("#aaffff");

    selected= get_canvas_tool_argument(get_active_canvas_view(), option);
    if (selected.empty())
      selected= get_wb_options().get_string(option);

    if (!selected.empty() && std::find(items.begin(), items.end(), selected) == items.end())
      items.push_back(selected);
  }
  else
    throw std::logic_error("Unknown option "+option);
  return items;
}


//--------------------------------------------------------------------------------
// Object Placement


grt::ValueRef WBContextBasic::place_layer_grt(grt::GRT *grt, const std::string &view_id,
                                             const mdc::Rect &rect)
{
  model_ViewRef view(get_view_with_id(view_id));

//  _layer_place_pos= pos;

  mdc::Point offset;
  // find what layer is it inside of
  model_LayerRef parent_layer(get_layer_at(view, rect.pos, offset));

  mdc::CanvasView *cview;
  cview= dynamic_cast<wbbridge::ViewBase*>(view.get_bridge_private())->get_canvas_view();

  std::string color;

  if (_view_data[cview].args["workbench.model.Layer:Color"].empty())
    color= get_wb_options().get_string("workbench.model.Layer:Color");
  else
    color= _view_data[cview].args["workbench.model.Layer:Color"];

  // create the layer object
  model_LayerRef layer= 
    grt->create_object<model_Layer>("workbench.model.Layer",
                                    "owner", view.valueptr(),
                                    "top", grt::DoubleRef(rect.ymin()).valueptr(),
                                    "left", grt::DoubleRef(rect.xmin()).valueptr(),
                                    "width", grt::DoubleRef(rect.width()).valueptr(),
                                    "height", grt::DoubleRef(rect.height()).valueptr(),
                                    "color", grt::StringRef(color).valueptr(),
                                    "name", grt::StringRef(_("New Layer")).valueptr(),
                                    NULL);

  _undo_manager->begin_undo_group();
  _undo_manager->add_undo(new UndoListAddAction(parent_layer.subLayers()));

  parent_layer.subLayers().insert(layer);

  mdc::Rect bounds(*layer.left(), *layer.top(), *layer.width(), *layer.height());
  // check for objects in the parent layer that fall inside this layer and add them
  for (size_t c= parent_layer.figures().count(), i= 0; i < c; i++)
  {
    model_FigureRef fig(parent_layer.figures().get(c-i-1));
    mdc::Rect fbounds(*fig.left(), *fig.top(), *fig.width(), *fig.height());

    if (mdc::bounds_contain_bounds(bounds, fbounds))
    {
      _undo_manager->add_undo(new UndoObjectChangeAction(fig, "layer"));
      fig.layer(layer);
    }
  }

  _undo_manager->end_undo_group();
  _undo_manager->set_action_description(_("Place New Layer"));

  return layer;
}


void WBContextBasic::place_layer(const std::string &view_id,
                            const mdc::Rect &rect)
{
  try
  {
    execute_in_grt_thread("Place new layer in model",
                        sigc::bind<std::string,mdc::Rect>(sigc::mem_fun(this, &WBContextBasic::place_layer_grt),
                                                           view_id,
                                                           rect));
    show_status_text(_("Created new layer."));
  }
  catch (std::exception &exc)
  {
    show_exception(_("Place new layer"), exc);
  }
}


//-------------------------------------------------------------------------------------
// Deletion


void WBContextBasic::delete_selection()
{
  if (_active_view)
  {
    bool view_only;
    grt::ListRef<model_Object> selection= get_selection(_active_view);
    int ok= execute_in_main_thread<int>("ask delete",
      sigc::bind<std::string, std::string, std::string, std::string>(confirm_action, 
                _("Delete Selection"), 
                selection.count() > 1 ?
                _("Should attached database objects be deleted with the figures?") :
                _("Should the attached database object be deleted with the figure?"),
                _("Delete"), _("Keep"), _("Cancel")));
    if (ok < 0)
      return;
    if (ok == 1)
      view_only= false;
    else
      view_only= true;

    std::vector<model_ObjectRef> objects;

    for (size_t c= selection.count(), i= 0; i < c; i++)
    {
      if (selection.get(i).is_instance(model_Object::static_class_name()))
        objects.push_back(model_ObjectRef::cast_from(selection.get(i)));
    }

    for (size_t c= objects.size(), i= 0; i < c; i++)
      delete_model_object(objects[i], view_only);
  }
}

// delete_basic_model_figure()

void WBContextBasic::delete_object(mdc::CanvasView *view, const mdc::Point &pos)
{
  model_ObjectRef object= get_object_at(get_view_with_id(get_id_of_view(view)), view, pos);

  if (object.is_valid())
  {
    if (delete_model_object(object, true))
      show_status_text(fmt(_("Deleted %s"), object.get_attribute("caption").c_str()));
  }
}


//-------------------------------------------------------------------------------------
// Handlers

bool WBContextBasic::handle_button_event(mdc::CanvasView *view, mdc::MouseButton button, bool press, mdc::Point pos, mdc::EventState state)
{
  std::string tool= get_canvas_tool(view);

  if (button != mdc::ButtonLeft)
    return false;

  if (tool == WB_TOOL_HAND)
  {
    _view_data[view].hcontext->moving= press;
    if(press)
    {
      mdc::Rect r= view->get_viewport();
      int wx, wy;
      _view_data[view].hcontext->viewport_pos= r.pos;
      view->canvas_to_window(pos, wx, wy);
      _view_data[view].hcontext->mouse_pos= mdc::Point(wx, wy);;
    }
    return true;
  }
  else if (tool == WB_TOOL_DELETE)
  {
    if (press)
    {
      delete_object(view, pos);
    }
    return true;
  }
  else if (tool == WB_TOOL_LAYER)
  {
    if (press)
      view->start_dragging_rectangle(pos);
    else
    {
      mdc::Rect rect= view->finish_dragging_rectangle();

      if (rect.width() < 5 || rect.height() < 5)
      {
        show_status_text(_("Please select a larger area."));
      }
      else
      {
        place_layer(_view_data[view].id, rect);
        reset_canvas_tool(view, true);
      }
    }
    return true;
  }
  else if (tool == WB_TOOL_NOTE)
  {
    if (press)
      view->start_dragging_rectangle(pos);
    else
    {
      mdc::Rect rect= view->finish_dragging_rectangle();

      if (rect.width() < 5 || rect.height() < 5)
      {
        show_status_text(_("Please select a larger area."));
      }
      else
      {
        workbench_model_NoteFigureRef note(workbench_model_NoteFigureRef::cast_from(
          place_object(_view_data[view].id, rect.pos, "workbench.model.NoteFigure")));
        if (note.is_valid())
        {
          note.text("Text Note");
          note.width(rect.width());
          note.height(rect.height());
        }
        reset_canvas_tool(view, true);
      }
    }
    return true;
  }
  else if (tool == WB_TOOL_IMAGE)
  {
    workbench_model_ImageFigureRef image;

    image= workbench_model_ImageFigureRef::cast_from(place_object(_view_data[view].id, pos, "workbench.model.ImageFigure"));

    if (image.is_valid())
    {
      std::string filename= show_file_dialog("open", _("Place Image"), "png");
      image.filename(filename);
    }

    reset_canvas_tool(view, true);
    return true;
  }
  return false;
}


bool WBContextBasic::handle_motion_event(mdc::CanvasView *cview, mdc::Point pos, mdc::EventState state)
{
  std::string tool= get_canvas_tool(cview);

  if (tool == WB_TOOL_HAND)
  {
    if(_view_data[cview].hcontext->moving)
    {
      int wx, wy;
      cview->canvas_to_window(pos, wx, wy);
      mdc::Point wpos(wx, wy);

      mdc::Point mouse_pos= _view_data[cview].hcontext->mouse_pos;
      mdc::Point viewport_pos= _view_data[cview].hcontext->viewport_pos;

      mdc::Point newp= viewport_pos - wpos + mouse_pos;

      mdc::Rect r= cview->get_viewport();
      mdc::Size sz= cview->get_total_view_size();

      if(newp.x < 0)
        newp.x= 0;
      if(newp.y < 0)
        newp.y= 0;
      if(newp.x > (sz.width - r.size.width))
        newp.x= sz.width - r.size.width;
      if(newp.y > (sz.height - r.size.height))
        newp.y= sz.height - r.size.height;

      cview->set_offset(newp);
      
      return true;
    }
  }
  return false;
}


void WBContextBasic::reset_tool(mdc::CanvasView *view)
{
  if (_view_data[view].hcontext)
  {
    delete _view_data[view].hcontext;
    _view_data[view].hcontext= 0;
  }
}
