#ifndef _WB_CONTEXT_BASIC_H_
#define _WB_CONTEXT_BASIC_H_

// Basic Model Handling (layers etc)

#include "wb_context_base.h"


namespace wb {

  // basic tool names
  #define WB_TOOL_HAND       "basic/hand"
  #define WB_TOOL_DELETE     "basic/delete"
  #define WB_TOOL_LAYER      "basic/layer"
  #define WB_TOOL_NOTE       "basic/note"
  #define WB_TOOL_IMAGE      "basic/image"

  class WBContextBasic : public virtual WBContextBase
  {
  public:
    WBContextBasic(bool verbose);
    virtual ~WBContextBasic();


    void delete_selection();


  protected:
    void init_canvas_grt(grt::GRT *grt, WBOptions *options);
    void load_app_options(bool update);

    // Canvas Handling
    void set_canvas_tool(mdc::CanvasView *view, const std::string &tool);

    // Toolbar Handling
    grt::ListRef<app_ToolbarItem> get_toolbar_items_for_view(const std::string &toolbar);

    std::vector<std::string> get_dropdown_items(const std::string &option, std::string &selected);
    void set_toolbar_option(const std::string &option, const std::string &item);

  private:
    std::map<std::string, app_Toolbar> _toolbars;

    // Model Objects
    void place_layer(const std::string &view_id, const mdc::Rect &rect);
    grt::ValueRef place_layer_grt(grt::GRT *grt, const std::string &view_id, 
                                      const mdc::Rect &rect);

    void delete_object(mdc::CanvasView *view, const mdc::Point &pos);


    void reset_tool(mdc::CanvasView *view);
    bool handle_motion_event(mdc::CanvasView*, mdc::Point, mdc::EventState);
    bool handle_button_event(mdc::CanvasView*, mdc::MouseButton, bool, mdc::Point, mdc::EventState);
  };

};


#endif 