#ifndef _WB_CONTEXT_PHYSICAL_H_
#define _WB_CONTEXT_PHYSICAL_H_

// Physical Model Handling

#include "wb_context_base.h"

#include "grt/icon_manager.h"

#include "wbcanvas/workbench_physical_model.h"

namespace wb {
  
  
  enum RelationshipType {
    Relationship11Id,
    Relationship1nId,
    RelationshipnmId,
    Relationship11NonId,
    Relationship1nNonId,
  };


  enum ObjectType {
    ObjectTable,
    ObjectView,
    ObjectRoutineGroup
  };



  #define WB_TOOL_PTABLE "physical/table"
  #define WB_TOOL_PROUTINEGROUP "physical/routinegroup"
  #define WB_TOOL_PVIEW "physical/view"

  #define WB_TOOL_PREL11_NOID "physical/rel11_noid"
  #define WB_TOOL_PREL1n_NOID "physical/rel1n_noid"
  #define WB_TOOL_PREL11 "physical/rel11"
  #define WB_TOOL_PREL1n "physical/rel1n"
  #define WB_TOOL_PRELnm "physical/relnm"
  

  class MYSQLWBBACKEND_PUBLIC_FUNC WBContextPhysical : virtual public WBContextBase, virtual public wbbridge::physical::ModelBridgeDelegate
  {
  public:
    WBContextPhysical(bool verbose);
    virtual ~WBContextPhysical();

    // Model
    void add_new_db_schema(const std::string &model_id);
    void delete_db_schema(const std::string &schema_id);

    void add_new_db_table(const std::string &schema_id);

    void add_new_db_view(const std::string &schema_id);

    void add_new_db_routine(const std::string &schema_id);

    void add_new_db_routine_group(const std::string &schema_id);


    void place_db_object(mdc::CanvasView *view, int x, int y, const db_DatabaseObjectRef &object);
    void place_db_object(mdc::CanvasView *view, const mdc::Point &pos, const db_DatabaseObjectRef &object);

    // Catalog Tree
    bec::ValueTreeBE *get_catalog_tree(const std::string &path);

    std::string get_catalog_path_for_view(mdc::CanvasView *view);

    std::string reverse_engineer_mysql_script(const std::string &file);

    void export_mysql_sql_script(const std::string &file);
    grt::ValueRef export_mysql_sql_script_task(grt::GRT*, grt::StringRef file);
    void export_mysql_sql_script_task_finished_cb(grt::ValueRef res);

    void delete_db_object(const db_DatabaseObjectRef &object);

    //XXX TODO delme
    std::string import_DBD4(const std::string &file);

  protected:
    void init_canvas_grt(grt::GRT *grt, WBOptions *options);
    void load_app_options(bool update);
    void save_app_options();

    void setup_context_grt(grt::GRT *grt, WBOptions *options);

    void init_catalog_grt(const db_mgmt_RdbmsRef &rdbms,
                          const std::string &db_versionRef,
                          workbench_physical_ModelRef &model);

    void setup_physical_model(grt::GRT *grt, workbench_DocumentRef &doc, 
                              const std::string &rdbms_name, const std::string &rdbms_version);



    void set_canvas_tool(mdc::CanvasView *view, const std::string &tool);

    grt::ListRef<app_ToolbarItem> get_toolbar_items_for_view(const std::string &toolbar);


    void reset_schemata_listeners();

    void activate_figure(const model_FigureRef &figure);
    bool delete_physical_model_object(const model_ObjectRef &object, bool view_only);


    // Toolbar Handling
    std::vector<std::string> get_dropdown_items(const std::string &option, std::string &selected);
    void set_toolbar_option(const std::string &option, const std::string &item);

  private:
    grt::ValueRef add_new_db_schema_grt(grt::GRT *grt,
                                       const std::string &model_id);

    grt::ValueRef delete_db_schema_grt(grt::GRT *grt, const std::string &schema_id, bool check_empty);

    grt::ValueRef add_new_db_table_grt(grt::GRT *grt,
                                       const std::string &schema_id);

    grt::ValueRef add_new_db_view_grt(grt::GRT *grt,
                                       const std::string &schema_id);

    grt::ValueRef add_new_db_routine_group_grt(grt::GRT *grt,
                                       const std::string &schema_id);

    grt::ValueRef add_new_db_routine_grt(grt::GRT *grt,
                                       const std::string &schema_id);


    void place_new_db_object(mdc::CanvasView *cview, const mdc::Point &pos, ObjectType type);
    grt::ValueRef place_new_db_object_grt(grt::GRT *grt, mdc::CanvasView *cview, 
                                              const mdc::Point &pos, ObjectType type);
    
    void start_relationship(mdc::CanvasView *cview, const mdc::Point &pos, RelationshipType type);
    bool update_relationship(mdc::CanvasView *cview, const mdc::Point &pos);
    void cancel_relationship(mdc::CanvasView *cview);

    grt::ValueRef update_relationship_grt(grt::GRT *grt, const std::string &view_id, 
                                              const mdc::Point &pos);

    bool create_nm_relationship_grt(grt::GRT *grt, mdc::CanvasView *view, workbench_physical_TableFigureRef table1, 
                                    workbench_physical_TableFigureRef table2,
                                    bool imandatory, bool fmandatory);

    grt::ValueRef place_db_object_grt(grt::GRT *grt, mdc::CanvasView *view, 
                                          const mdc::Point &pos, const db_DatabaseObjectRef &object);

//    void finder(FindDialogBE *finder, const model_ModelRef &model);

  private:
    enum RelationshipToolState
    {
      RIdle,
      RPickingStart,
      RPickingEnd,
      RFinished,
      RCancelled
    };

    struct RelationshipToolContext
    {
      RelationshipToolState state;
      std::string last_message;
      RelationshipType type;

      workbench_physical_TableFigureRef itable;

      RelationshipToolContext()
        : state(RIdle) {}
    };

    std::map<mdc::CanvasView*, RelationshipToolContext> _relationship_context;

    std::multimap<std::string, model_Figure> _dbobject_figures;

    std::map<std::string, app_Toolbar> _toolbars;
    std::vector<std::string> _collation_list;

    bec::ValueTreeBE *_catalog_tree;
    int _schemata_listener;

    bool filter_catalog_tree(const bec::NodeId &node, const std::string &key, const grt::ValueRef &value, 
      std::string &label_ret, bec::IconId &icon_ret);

    // delegate for PhysicalModelBridgeDelegate
    virtual void handle_fklist_change(const db_TableRef &table, const db_ForeignKeyRef &fk, bool added);
    virtual void notify_dbobject_figure(const db_DatabaseObjectRef &object, const model_FigureRef &figure, bool attached);

    // Listeners
    void schema_list_changed(MYX_GRT_NOTIFICATION *notif);
    void schema_changed(MYX_GRT_NOTIFICATION *notif);
    void schema_object_list_changed(MYX_GRT_NOTIFICATION *notif);

    void db_object_changed(MYX_GRT_NOTIFICATION *notif);

    bool handle_button_event(mdc::CanvasView*, mdc::MouseButton, bool, mdc::Point, mdc::EventState);
  };

};


#endif
