#ifndef _PREFERENCES_FORM_H_
#define _PREFERENCES_FORM_H_

#include <mforms/mforms.h>

#include "workbench/wb_context_ui.h"

class PreferencesForm : public mforms::Form 
{
public:
  struct Option
  {
    mforms::View *view;
    sigc::slot<void> show_value;
    sigc::slot<void> update_value;
    
    ~Option() { delete view; }
  };

private:
  std::list<Option*> _options;
  std::list<mforms::View*> _widgets;

  mforms::Box _top_box;
  mforms::Box _bottom_box;
  mforms::TabView _tabview;

  mforms::Box _button_box;
  mforms::Button _ok_button;
  mforms::Button _cancel_button;

  mforms::TreeView _font_list;
  std::vector<std::string> _font_options;

  std::string _model_id; // "" unless we're showing model specific options
  wb::WBContextUI *_wbui;

  sigc::slot<std::string,std::string,std::string> _edit_font;
  sigc::slot<void> _handle_close;
  
  template <class C>
    C* add_view(C *w)
    {
      _widgets.push_back(w);
      return w;
    }
  
  mforms::Label *new_label(const std::string &text, bool right_align= false);

  mforms::TextEntry *new_entry_option(const std::string &option, bool numeric);
  mforms::CheckBox *new_checkbox_option(const std::string &option);
  mforms::Selector *new_selector_option(const std::string &option);

  void ok_clicked();
  void cancel_clicked();
  
  void show_values();
  void update_values();

  void show_colors();
  void update_colors();

  void create_general_page();
  void create_model_page();
  void create_mysql_page();
  void create_diagram_page();
  void create_appearance_page();

  grt::DictRef get_options();
  
public:
  PreferencesForm(wb::WBContextUI *wbui, const std::string &model_id= "");
  virtual ~PreferencesForm();
  
  // returned font, title, current font
  void set_font_panel_function(const sigc::slot<std::string,std::string,std::string> &edit_font);

  void set_closed_handler(const sigc::slot<void> &slot);
};


#endif /* _PREFERENCES_FORM_H_ */
