
#include "multi_view.h"


MultiView::MultiView(bool tree_view, bool icon_view)
  : Gtk::VBox(false, 0)
{
  if (tree_view)
  {
    _tree_view= Gtk::manage(new Gtk::TreeView());
    pack_start(*_tree_view, false, false);

    _tree_view->signal_row_activated().connect(sigc::mem_fun(this, &MultiView::tree_row_activated));
    _tree_view->signal_button_release_event().
      connect_notify(sigc::mem_fun(*this, &MultiView::tree_button_release_event));

    _tree_view->get_selection()->signal_changed().connect(sigc::mem_fun(this, &MultiView::tree_selection_changed));
  }
  else
    _tree_view= 0;


  if (icon_view)
  {
    _icon_view= Gtk::manage(new Gtk::IconView());
    pack_start(*_icon_view, false, false);
    _icon_view->set_orientation(Gtk::ORIENTATION_HORIZONTAL);

    _icon_view->signal_item_activated().connect(sigc::mem_fun(this, &MultiView::icon_activated));
    _icon_view->signal_button_release_event().
      connect_notify(sigc::mem_fun(*this, &MultiView::icon_button_release_event));
    _icon_view->signal_selection_changed().connect(sigc::mem_fun(this, &MultiView::icon_selection_changed));
  }
  else
    _icon_view= 0;
  
  if (tree_view)
    _tree_view->show();
  else if (icon_view)
    _icon_view->show();
}


MultiView::~MultiView()
{
}


void MultiView::set_icon_mode(bool flag, bool horizontal_icons)
{
  if (_tree_view && _icon_view)
  {
    if (flag)
    {
      _tree_view->hide();
      _icon_view->show();

      _icon_view->set_orientation(horizontal_icons ? Gtk::ORIENTATION_HORIZONTAL : Gtk::ORIENTATION_VERTICAL);
    }
    else
    {
      _tree_view->show();
      _icon_view->hide();
    }
  }
  else if (_icon_view)
    _icon_view->set_orientation(horizontal_icons ? Gtk::ORIENTATION_HORIZONTAL : Gtk::ORIENTATION_VERTICAL);
}


void MultiView::set_model(const Glib::RefPtr<TreeModelWrapper> &model)
{
  _model= model;
  
  if (_tree_view)
    _tree_view->set_model(model);
  if (_icon_view)
    _icon_view->set_model(model);
}


void MultiView::refresh()
{
 if (_tree_view)
  {
    _tree_view->unset_model();
    _tree_view->set_model(_model);
  }
  if (_icon_view)
  {
    _icon_view->set_model(Glib::RefPtr<Gtk::TreeModel>());
    _icon_view->set_model(_model);
  }
}


void MultiView::tree_row_activated(const Gtk::TreeModel::Path &path,
                                   const Gtk::TreeViewColumn  *column)
{
  _activate_item(path);
}


void MultiView::icon_activated(const Gtk::TreeModel::Path &path)
{
  _activate_item(path);
}



void MultiView::icon_button_release_event(GdkEventButton* event)
{
  if ( GDK_BUTTON_RELEASE == event->type && 3 == event->button )
  {
    Gtk::TreeModel::Path path;
    
    if (_icon_view->get_item_at_pos(event->x, event->y, path))
      _icon_view->select_path(path);
    
    std::vector<Gtk::TreeModel::Path> selected_items = _icon_view->get_selected_items();
    if ( selected_items.size() > 0 )
    {
      _popup_menu.emit(selected_items[0], event->time);
      return; 
    }

    _popup_menu.emit(Gtk::TreeModel::Path(), event->time);
  }
}


void MultiView::tree_button_release_event(GdkEventButton* event)
{
  if ( GDK_BUTTON_RELEASE == event->type && 3 == event->button )
  {
    Glib::RefPtr<Gtk::TreeView::Selection> selection= _tree_view->get_selection();
    
    if ( selection->count_selected_rows() > 0 )
    {
      std::vector<Gtk::TreeModel::Path> selected= selection->get_selected_rows();
      _popup_menu.emit(selected[0], event->time);
    }
    _popup_menu.emit(Gtk::TreeModel::Path(), event->time);
  }
}


void MultiView::icon_selection_changed()
{
  _selection_changed.emit(_icon_view->get_selected_items());
}


void MultiView::tree_selection_changed()
{
  _selection_changed.emit(_tree_view->get_selection()->get_selected_rows());
}
