//
//  MTogglePane.m
//  MySQLWorkbench
//
//  Created by Alfredo Kojima on 6/Oct/08.
//  Copyright 2008 Sun Microsystems Inc. All rights reserved.
//

#import "MTogglePane.h"


@implementation MTogglePane

- (id)initWithFrame:(NSRect)frame
{
  if ((self= [super initWithFrame:frame]) != nil)
  {
    _header= [[[NSImageView alloc] initWithFrame:NSMakeRect(0, 0, NSWidth(frame), 23)] autorelease];
    [self addSubview:_header];
    [_header setAutoresizingMask:NSViewWidthSizable];
    [_header setImageScaling:NSImageScaleAxesIndependently];
    [_header setImage:[[[NSImage alloc] initWithContentsOfFile:[[NSBundle mainBundle] pathForResource:@"collapsing_panel_header_bg_flat" ofType:@"png" inDirectory:@"images"]] autorelease]];
  
    _toggleButton= [[[NSButton alloc] initWithFrame:NSMakeRect(5, 6, 13, 13)] autorelease];
    [_toggleButton setBezelStyle:NSDisclosureBezelStyle];
    [_toggleButton setButtonType:NSOnOffButton];
    [_toggleButton setTitle:@""];
    [_toggleButton setAction:@selector(toggle:)];
    [_toggleButton setTarget:self];
    [self addSubview:_toggleButton];
    
    _label= [[[NSTextField alloc] initWithFrame:NSMakeRect(20, 4, 20, 20)] autorelease];
    [_label setBordered:NO];
    [_label setEditable:NO];
    [_label setFont:[NSFont boldSystemFontOfSize:12]];
    [_label setDrawsBackground:NO];
    [self addSubview:_label];
  }
  return self;
}

- (BOOL)isFlipped
{
  return YES;
}


- (IBAction)toggle:(id)sender
{
  [self relayout];
}


- (void)setExpanded:(BOOL)flag
{
  [_toggleButton setState:flag ? NSOnState : NSOffState];
  [self relayout];
}


- (void)setLabel:(NSString*)label
{
  [_label setStringValue:label];
  [_label sizeToFit];
}


- (void)contentFrameDidChange:(NSNotification*)notif
{
  [self relayout];
}


- (void)setContentView:(NSView*)view
{
  if (_content)
  {
    [_content removeFromSuperview];
    [[NSNotificationCenter defaultCenter] removeObserver:self name:nil object:_content];
  }
  _content= view;
  [_content setPostsFrameChangedNotifications:YES];
  [[NSNotificationCenter defaultCenter] addObserver:self 
                                           selector:@selector(contentFrameDidChange:)
                                               name:NSViewFrameDidChangeNotification
                                             object:_content];
  [view setFrameOrigin:NSMakePoint(0, NSHeight([_header frame]))];
  [view setAutoresizingMask:NSViewWidthSizable|NSViewMaxYMargin];
  [self addSubview:view];
  
  [self relayout];
}


- (void)resizeSubviewsWithOldSize:(NSSize)oldBoundsSize
{
  [self relayout];
  [super resizeSubviewsWithOldSize:oldBoundsSize];
}


- (void)relayout
{
  NSRect contentRect= [_content frame];
  NSRect newContentRect;
  NSRect rect= [self frame];
  CGFloat headerHeight= NSHeight([_header frame]);
  
  if ([_toggleButton state] == NSOnState)
  {
    rect.size.height= headerHeight + NSHeight(contentRect);

    [_content setHidden:NO];
  }
  else
  {
    rect.size.height= headerHeight;
    
    [_content setHidden:YES];
  }
  
  newContentRect= NSMakeRect(0, headerHeight, NSWidth(rect), NSHeight(contentRect));
  
  if (!NSEqualRects(newContentRect, contentRect))
    [_content setFrame:newContentRect];
  
  if (!NSEqualRects([self frame], rect))
    [self setFrame:rect];
}


@end
