//
//  WBMainController.mm
//  MySQLWorkbench
//
//  Created by Alfredo Kojima on 1/Oct/08.
//  Copyright 2008 Sun Microsystems Inc. All rights reserved.
//

#import "WBMainController.h"
#import "WBMainWindow.h"
#import "MCppUtilities.h"


#include "workbench/wb_context_ui.h"
#import "sigobjc++.h"
#import "GRTShellViewController.h"


static GThread *mainthread= 0;

@implementation WBMainController

static void showInternalError(const std::string &title, const std::string &message)
{
  NSRunAlertPanel([NSString stringWithUTF8String:title.c_str()],
                  [NSString stringWithUTF8String:message.c_str()],
                  @"OK", nil, nil);
}


static void showError(const std::string &title, const std::string &message)
{
  NSRunAlertPanel([NSString stringWithUTF8String:title.c_str()],
                  [NSString stringWithUTF8String:message.c_str()],
                  @"OK", nil, nil);  
}


static void showWarning(const std::string &title, const std::string &message)
{
  NSRunAlertPanel([NSString stringWithUTF8String:title.c_str()],
                  [NSString stringWithUTF8String:message.c_str()],
                  @"OK", nil, nil);  
}


static void showMessage(const std::string &title, const std::string &message)
{
  NSRunAlertPanel([NSString stringWithUTF8String:title.c_str()],
                  [NSString stringWithUTF8String:message.c_str()],
                  @"OK", nil, nil);  
}


static int confirmAction(const std::string &title, const std::string &message, 
                         const std::string &yes_text, const std::string &no_text, const std::string &other_text)
{
  int ret= NSRunAlertPanel([NSString stringWithUTF8String:title.c_str()],
                  [NSString stringWithUTF8String:message.c_str()],
                  yes_text.empty() ? @"OK":[NSString stringWithUTF8String:yes_text.c_str()],
                  no_text.empty()  ? @"Cancel":[NSString stringWithUTF8String:no_text.c_str()],
                  other_text.empty()?nil:[NSString stringWithUTF8String:other_text.c_str()]);  
  
  if (ret == NSAlertDefaultReturn)
    return 1;
  else if (ret == NSAlertAlternateReturn)
    return 0;
  else
    return -1;
}


static std::string showFileDialog(const std::string &type, const std::string &title, const std::string &extensions)
{   
  NSMutableArray *fileTypes= [NSMutableArray array];
  std::vector<std::string> exts(bec::split_string(extensions,","));
   
  for (std::vector<std::string>::const_iterator iter= exts.begin();
       iter != exts.end(); ++iter)
  {   
    if (iter->find('|') != std::string::npos)
    {
      std::string ext= iter->substr(iter->find('|')+1);
      ext= bec::replace_string(ext, "*.", "");
      [fileTypes addObject:[NSString stringWithUTF8String:ext.c_str()]];
    }
    else
      [fileTypes addObject:[NSString stringWithUTF8String:iter->c_str()]];
  }  
  
  if (type == "open")
  {
    NSOpenPanel *panel= [NSOpenPanel openPanel];
    
    [panel setTitle:[NSString stringWithUTF8String:title.c_str()]];
    
    if ([panel runModalForTypes:fileTypes] == NSFileHandlingPanelOKButton)
      return [[panel filename] UTF8String];
  }
  else if (type == "save")
  {
    NSSavePanel *panel= [NSSavePanel savePanel];
    
    [panel setTitle:[NSString stringWithUTF8String:title.c_str()]];
    
    [panel setAllowedFileTypes:fileTypes];

    if ([panel runModal] == NSFileHandlingPanelOKButton)
      return [[panel filename] UTF8String];
  }
  
  return "";
}


static bool requestInput(const std::string &message, int flags, std::string &ret_text)
{
  return false;
}





static void windowShowStatusText(const std::string &text, WBMainWindow *window)
{
  NSString *string= [[NSString alloc] initWithUTF8String:text.c_str()];

  // setStatusText must release the param
  [window performSelectorOnMainThread:@selector(setStatusText:)
                           withObject:string
                        waitUntilDone:NO];
}


static bool windowShowProgress(const std::string &,const std::string &,float, WBMainWindow *window)
{
  return true;
}


static NativeHandle windowOpenPlugin(bec::GRTManager*, 
                                     const std::string &, const std::string &, const std::string &, 
                                     const grt::BaseListRef &, bec::GUIPluginFlags, WBMainWindow *window)
{
  return 0;
}


static void windowShowPlugin(NativeHandle, WBMainWindow *window)
{
}


static void windowHidePlugin(NativeHandle, WBMainWindow *window)
{
}


static void windowPerformCommand(const std::string &command, WBMainWindow *window)
{
  
}


static mdc::CanvasView* windowCreateView(const std::string &oid, const std::string &name, WBMainWindow *window)
{  
  return [window createView:oid.c_str()
                       name:name.c_str()];
}


static void windowDestroyView(mdc::CanvasView *view, WBMainWindow *window)
{  
  [window destroyView:view];
}


static void windowSwitchedView(mdc::CanvasView*, WBMainWindow *window)
{
}


static void windowToolChanged(mdc::CanvasView *canvas, WBMainWindow *window)
{
  [window canvasToolChanged:canvas];
}


static void windowRefreshGui(wb::RefreshType type, const std::string &arg1, NativeHandle arg2, WBMainWindow *window)
{
  [window refreshGUI:type argument1:arg1 argument2:arg2];
}


static void windowLockGui(bool, WBMainWindow *window)
{
}


static void windowPopupMenu(bec::MenuItemList*, int, int, WBMainWindow *window)
{
}


static void windowPrintOutput(const std::string &text, WBMainWindow *window)
{
  NSLog(@"output %s", text.c_str());
}


static void windowShellOutput(const std::string &text, WBMainWindow *window)
{  
  [[window shell] write: [NSString stringWithUTF8String:text.c_str()]];
}


static void windowNextShellCommand(const std::string &prompt, WBMainWindow *window)
{
  [[window shell] nextShellCommand: [NSString stringWithUTF8String:prompt.c_str()]];
}


- (void)setupBackend
{
  mainthread= g_thread_self();
  
  try
  {
    bec::GRTManager *grtm;
    
    // Setup backend stuff
    _wbui = new wb::WBContextUI(true);
    _wb = _wbui->get_wb();
    
    grtm = _wb->get_grt_manager();
    
    [mainWindow setWBContext: _wbui];
    [mainWindow setOwner:self];
    
    // Define a set of methods which backend can call to interact with user and frontend
    wb::WBFrontendCallbacks wbcallbacks;
    
    // Assign those callback methods
    wbcallbacks.show_internal_error= sigc::ptr_fun(showInternalError);
    wbcallbacks.show_error= sigc::ptr_fun(showError);
    wbcallbacks.show_warning= sigc::ptr_fun(showWarning);
    wbcallbacks.show_message= sigc::ptr_fun(showMessage);
    wbcallbacks.confirm_action= sigc::ptr_fun(confirmAction);
    wbcallbacks.show_file_dialog= sigc::ptr_fun(showFileDialog);
    wbcallbacks.show_status_text= sigc::bind(sigc::ptr_fun(windowShowStatusText), mainWindow);
    wbcallbacks.show_progress= sigc::bind(sigc::ptr_fun(windowShowProgress), mainWindow);
    wbcallbacks.request_input= sigc::ptr_fun(requestInput);
    wbcallbacks.open_editor= sigc::bind(sigc::ptr_fun(windowOpenPlugin), mainWindow);
    wbcallbacks.show_editor= sigc::bind(sigc::ptr_fun(windowShowPlugin), mainWindow);
    wbcallbacks.hide_editor= sigc::bind(sigc::ptr_fun(windowHidePlugin), mainWindow);
    wbcallbacks.perform_command= sigc::bind(sigc::ptr_fun(windowPerformCommand), mainWindow);
    wbcallbacks.create_view= sigc::bind(sigc::ptr_fun(windowCreateView), mainWindow);
    wbcallbacks.destroy_view= sigc::bind(sigc::ptr_fun(windowDestroyView), mainWindow);
    wbcallbacks.switched_view= sigc::bind(sigc::ptr_fun(windowSwitchedView), mainWindow);
    wbcallbacks.tool_changed= sigc::bind(sigc::ptr_fun(windowToolChanged), mainWindow);
    wbcallbacks.refresh_gui= sigc::bind(sigc::ptr_fun(windowRefreshGui), mainWindow);
    wbcallbacks.lock_gui= sigc::bind(sigc::ptr_fun(windowLockGui), mainWindow);
    wbcallbacks.popup_menu= sigc::bind(sigc::ptr_fun(windowPopupMenu), mainWindow);
    wbcallbacks.print_output= sigc::bind(sigc::ptr_fun(windowPrintOutput), mainWindow);
    wbcallbacks.shell_output= sigc::bind(sigc::ptr_fun(windowShellOutput), mainWindow);
    wbcallbacks.next_shell_command= sigc::bind(sigc::ptr_fun(windowNextShellCommand), mainWindow);
      
    wb::WBOptions wboptions;
    //wboptions.open_at_startup= open_file;
    wboptions.basedir = [[[NSBundle mainBundle] resourcePath] fileSystemRepresentation];
    wboptions.struct_search_path = wboptions.basedir + "/grt";
    //  wboptions.plugin_search_path = [[[NSBundle mainBundle] 
    //  wboptions.module_search_path = //;
    wboptions.user_data_dir= [[@"~/Library/Application Support/MySQL/Workbench" stringByExpandingTildeInPath] fileSystemRepresentation];
    
    //wboptions.open_at_startup = "";
    
    _wbui->init(&wbcallbacks, &wboptions);
    
    _wb->flush_idle_tasks();
  }
  catch (std::exception &exc)
  {
    MShowCPPException(exc);
  }
}


- (void)awakeFromNib
{  
  mainWindow = [[WBMainWindow alloc] init];
  [mainWindow load];

  [self setupBackend];

  [NSTimer scheduledTimerWithTimeInterval:0.5 target:self selector:@selector(idleTasks:)
                                 userInfo:nil repeats:YES];

  [mainWindow showWindow:nil];
  
  [mainWindow setupReady];
}


- (void)idleTasks:(NSTimer*)timer
{
  _wb->flush_idle_tasks();
}


- (void)requestRefresh
{
  NSAutoreleasePool *pool= [[NSAutoreleasePool alloc] init];
  [NSTimer scheduledTimerWithTimeInterval:0.0
                                   target:self
                                 selector:@selector(idleTasks:)
                                 userInfo:nil repeats:NO];
  [pool release];
}

@end
