//
//  WBMainWindow.mm
//  MySQLWorkbench
//
//  Created by Alfredo Kojima on 1/Oct/08.
//  Copyright 2008 Sun Microsystems Inc. All rights reserved.
//

#import "WBMainWindow.h"
#import "WBMainController.h"
#import "WBMenuManager.h"
#import "WBToolbarManager.h"
#import "WBOverviewPanel.h"
#import "WBModelViewPanel.h"
#import "MCanvasViewer.h"
#import "MCanvasScrollView.h"
#import "GRTShellViewController.h"
#include "workbench/wb_context_ui.h"

@implementation WBMainWindow

- (id)init
{
  self = [super init];
  if (self != nil) 
  {
    _forms= [[NSMutableDictionary alloc] init];
  }
  return self;
}


- (void) dealloc
{
  [_menuManager release];
  [_toolbarManager release];
  [_forms release];
  [_overview release];

  [super dealloc];
}


- (void)setOwner:(WBMainController*)owner
{
  _mainController= owner;
}


- (void)load
{
  if (![NSBundle loadNibNamed:@"MainWindow" owner:self])
    NSLog(@"Failed loading nib");
}


- (void)awakeFromNib
{
  [topTabView removeTabViewItem:[topTabView tabViewItemAtIndex:0]];
  
  [statusBarText setStringValue:@""];  
}


- (void)setWBContext:(wb::WBContextUI*)wbui
{
  _wbui= wbui;
  [shellController setupWithGRTManager: _wbui->get_wb()->get_grt_manager()];

  // setup GRT shell sidebar
  {
    NSTabViewItem *item= [[[NSTabViewItem alloc] initWithIdentifier:@"GRT"] autorelease];

    [item setLabel:@"GRT"];
    [item setView:[shellController sidebar]];

    [sideTabView addTabViewItem:item];
  }
  
  _menuManager= [[WBMenuManager alloc] initWithMenuBar:[NSApp mainMenu] WBContext:_wbui];
  _toolbarManager= [[WBToolbarManager alloc] initWithToolbar:[[self window] toolbar] 
                                                toolsToolbar:sideToolbar
                                                   WBContext:_wbui];
  
  _overview= [[WBOverviewPanel alloc] initWithBE:_wbui->get_overview()];
}


- (void)setupReady
{
  [shellController refresh];
  [_menuManager rebuildMainMenu];
  [_toolbarManager refreshToolbar:nil];
  [_overview setup];

  [self addTopPanel:_overview];
}


- (void)addTopPanel:(WBBasePanel*)panel
{
  NSTabViewItem *tabItem= [[[NSTabViewItem alloc] initWithIdentifier:[panel identifier]] autorelease];
  
  [tabItem setView:[panel topView]];
  [tabItem setLabel:[panel title]];
  
  [_forms setObject:panel forKey:[panel identifier]];
  
  [topTabView addTabViewItem:tabItem];
  [topTabView selectLastTabViewItem:self];
}


- (GRTShellViewController*)shell
{
  return shellController;
}


- (void)refreshGUI:(wb::RefreshType)type
         argument1:(const std::string&)arg1
         argument2:(NativeHandle)arg2
{
  const char* RefreshTypeStr[] = {
    "RefreshNeeded",
    
    "RefreshNothing",
    "RefreshLayers",
    "RefreshSchemaList",
    "RefreshSchema",
    "RefreshNewView",
    "RefreshPrivileges",
    "RefreshSelection",
    "RefreshToolbar",
    "RefreshMessages",
    "RefreshCloseEditor",
    "RefreshFindOutput",
    "RefreshHistory",
    "RefreshOverview",
    "RefreshOverviewNode",
    "RefreshCatalogTreeNode",
    "RefreshAttachments",
    "RefreshViewName",
    "RefreshDocument",
    "RefreshCloseDocument",
    "RefreshZoom",
    "RefreshMenubar",
    "RefreshTimer"
  };
  
  switch (type)
  {
    case wb::RefreshNeeded:
      [_mainController requestRefresh];
      break;
      
    case wb::RefreshDocument:
      // refresh the title of the document
      [[self window] setTitle:[NSString stringWithUTF8String:_wbui->get_title().c_str()]];
      [[self window] setRepresentedFilename:[NSString stringWithUTF8String:_wbui->get_wb()->get_filename().c_str()]];
      break;
      
    case wb::RefreshOverview:
      [_overview refresh];
      NSLog(@"REFRESH OVER");
      break;
      
    case wb::RefreshToolbar:
      [_toolbarManager refreshToolbar: arg1.empty() ? nil : [NSString stringWithUTF8String:arg1.c_str()]];
      break;

    default:
      NSLog(@"unhandled %s", RefreshTypeStr[type]);
      break;
  }
}


- (void)setStatusText:(NSString*)text
{  
  [statusBarText setStringValue:text];
  [statusBarText sizeToFit];
  
  [text release];
}


- (mdc::CanvasView*)createView:(const char*)oid
                          name:(const char*)name
{  
  WBModelViewPanel *panel= [[WBModelViewPanel alloc] initWithId:[NSString stringWithUTF8String:oid]
                                                         formBE:_wbui->get_view_form_for_view_id(oid)];

  [self addTopPanel:panel];
  [panel release];

  return [panel canvas];
}


- (void)destroyView:(mdc::CanvasView*)view
{
  NSString *identifier= [NSString stringWithUTF8String:view->get_tag().c_str()];
  WBModelViewPanel *panel= [_forms objectForKey:identifier];

  if (panel)
  {
    [topTabView removeTabViewItem:[topTabView tabViewItemAtIndex:[topTabView indexOfTabViewItemWithIdentifier:identifier]]];

    [_forms removeObjectForKey:identifier];
  }
}


- (void)canvasToolChanged:(mdc::CanvasView*)canvas
{
  NSString *identifier= [NSString stringWithUTF8String:canvas->get_tag().c_str()];
  WBModelViewPanel *panel= [_forms objectForKey:identifier];
  
  if (panel && [[[topTabView selectedTabViewItem] identifier] isEqualToString:identifier])
  {
    [panel updateCursor];
    [_toolbarManager refreshToolsToolbar];
    [_toolbarManager refreshMainToolbar];
  }
}


- (void)tabView:(NSTabView *)tabView willSelectTabViewItem:(NSTabViewItem *)tabViewItem
{  
  WBBasePanel *panel= [_forms objectForKey:[tabViewItem identifier]];

  _wbui->set_active_form([panel formBE]);
}


@end
