//
//  testapp.mm
//  mdcanvas
//
//  Created by Alfredo Kojima on 07/Mar/6.
//  Copyright 2007 MySQL AB. All rights reserved.
//

#include <string>
#import "testapp.h"
#import "MDCanvasView.h"
#import "MDCanvasScrollView.h"
#include "mdc.h"
#include "mdc_draw_util.h"


using namespace mdc;

mdc::CanvasItem *t= 0;


class Thing : public mdc::Box {
  mdc::RectangleFigure rect;
  mdc::Box title_bar;
  mdc::RectangleFigure title_back;
  mdc::IconTextFigure title;
  mdc::Button title_expander;

  mdc::Box column_box;

  cairo_surface_t *column_icon;
  cairo_surface_t *key_icon;

  std::vector<mdc::IconTextFigure*> columns;

public:
  Thing(mdc::Layer *layer)
  : mdc::Box(layer, Box::Vertical), 
    rect(layer), 
    title_bar(layer, Box::Horizontal),
    title_back(layer),
    title(layer),
    title_expander(layer, ExpanderButton),
    column_box(layer, Box::Vertical, true)
{
    column_icon= cairo_image_surface_create_from_png("/Users/kojima/Development/mysql-workbench-pro/images/grt/structs/db.Column.11x11.png");
    key_icon= cairo_image_surface_create_from_png("/Users/kojima/Development/mysql-workbench-pro/images/grt/structs/db.Column.pk.11x11.png");

//    column_icon= mdc::IconManager::get_instance()->get_icon("column-12x12.png");
    
    n= "Table";
    
    set_allowed_resizing(false, false);
    
    set_background(&rect);
    set_accepts_focus(true);
    set_accepts_selection(true);

    rect.n= "Table rect";
    rect.set_fill_color(Color(1,1,1));
    rect.set_pen_color(Color(0.5,0.5,0.5));
    rect.set_filled(true);
    rect.set_rounded_corners(8, mdc::CTop);
    rect.set_has_shadow(true);

    add(&title_bar, false, false, true);
    title_bar.n="titlebar";
    title_bar.set_padding(6, 4);
    title_bar.add(&title, true, true);
    title_bar.set_background(&title_back);
    title_back.set_fill_color(Color(0.5, 0.7, 0.83));
    title_back.set_pen_color(Color(0.5,0.5,0.5));
    title_back.set_filled(true);
    title_back.set_rounded_corners(8, mdc::CTop);
    
    title_bar.set_draggable(true);

    title.n= "title";
    title.set_icon(cairo_image_surface_create_from_png("/Users/kojima/Development/mysql-workbench-pro/images/grt/structs/db.Table.16x16.png"));
    title.set_font(mdc::FontSpec("Lucida Grande", SNormal, WBold, 11));
    title.set_text("Hello World");
    
    title_expander.set_color(Color(0.4,0.4,0.4));
    title_expander.n="title expdr";
    title_expander.signal_activate().connect(sigc::mem_fun(this, &Thing::toggle_expander));
    title_bar.add(&title_expander, false, true);
    title_expander.set_active(true);

    add(&column_box, false, true, true);
    column_box.set_spacing(2);
    column_box.set_padding(3, 3);
    column_box.n= "colbox";

    t= &column_box;
    
    add_column("id int primary key", key_icon);
    add_column("name varchar(32)", column_icon);
    add_column("address varchar(200)", column_icon);
    add_column("city int", column_icon);
    add_column("country int", column_icon);
    add_column("phone varchar(40)", column_icon);
    add_column("email varchar(80)", column_icon);
}


void toggle_expander()
{
  if (title_expander.get_active())
    column_box.set_visible(true);
  else
    column_box.set_visible(false);
}


void add_column(const std::string &text, cairo_surface_t *icon)
{
  mdc::IconTextFigure *tf;

  tf= new mdc::IconTextFigure(_layer);
  tf->n= ":"+text;
  tf->set_icon(icon);
  tf->set_spacing(1);
  tf->set_font(mdc::FontSpec("Lucida Grande", SNormal, WNormal, 10));
  tf->set_text(text);

  column_box.add(tf, false, true);
}


};


@implementation testapp


- (IBAction)createObjects:(id)sender
{
  mdc::CanvasView *cv= [canvas mdCanvasView];
  mdc::Magnet *magnet;
  
  Thing *thing= new Thing(cv->get_current_layer());
 // thing->set_fixed_size(mdc::Size(100, 100));

  thing->magnetize_bounds();
  
//  mdc::TextFigure *thing= new mdc::TextFigure(cv->get_current_layer());
 // thing->set_text("hello wordl");
  cv->get_current_layer()->add_item(thing);
  thing->move_to(mdc::Point(rand()%500, rand()%500));
  
  thing->set_draggable(true);
  
  static int i= 0;
  if (i++==0)
    t1= thing;
  else
    t2= thing;
}


- (IBAction)group:(id)sender
{
  mdc::CanvasView *cv= [canvas mdCanvasView];
  mdc::Layer *layer= cv->get_current_layer();

  cv->get_selection()->add(layer->create_group_with(cv->get_selected_items()));
}


- (IBAction)ungroup:(id)sender
{
  t->set_visible(false);
}


- (IBAction)createArea:(id)sender
{
  mdc::CanvasView *cv= [canvas mdCanvasView];
  mdc::Layer *layer= cv->get_current_layer();

  RectangleFigure *bg= new RectangleFigure(layer);
  bg->set_filled(true);
//  bg->set_alpha(0.7);
  bg->set_fill_color(mdc::Color(1.0, 1.0, 0.6));
  bg->set_pen_color(mdc::Color(0.7, 0.7, 0.7));

  AreaGroup *ag= layer->create_area_group_with(bg, cv->get_selected_items());
  ag->n= "new area";
  printf("========================================== AREA is %p\n", ag);
}


- (IBAction)createLine:(id)sender
{
  mdc::CanvasView *cv= [canvas mdCanvasView];
  mdc::Layer *layer= cv->get_current_layer();
  
  line= new mdc::Line(layer);
  line->set_line_type(mdc::OrthogonalLine);
  line->add_vertex(mdc::Point(100, 300));
  line->add_vertex(mdc::Point(500, 300));
  
  layer->add_item(line);
  
  line->set_draggable(true);
  
}


- (IBAction)createLinePoint:(id)sender
{
  NSLog(@"HELLO");
  
  if (!t1 || !t2) 
  {
    NSLog(@"not inited");
    return;
  }
  line->get_first_connector()->connect(t1->get_bounds_magnet());
  line->get_last_connector()->connect(t2->get_bounds_magnet());
}


- (void)awakeFromNib
{
  mdc::CanvasView *cview;
  NSRect rect= [scroller frame];
  
  rect.size= [scroller contentSize];
  canvas= [[MDCanvasView alloc] initWithFrame:rect];
  [scroller setContentCanvas:canvas];

  cview= [canvas mdCanvasView];
  cview->set_page_size(mdc::Size(1024, 768));
  cview->set_page_layout(2,2);
}


- (IBAction)zoomIn:(id)sender
{
  float zoom= [canvas mdCanvasView]->get_zoom()+0.5;
  printf("set zoom %f\n", zoom);
  [canvas mdCanvasView]->set_zoom(zoom);
}


- (IBAction)zoomOut:(id)sender
{
  float zoom= [canvas mdCanvasView]->get_zoom()-0.5;
  printf("set zoom %f\n", zoom);
  [canvas mdCanvasView]->set_zoom(zoom);
}


- (IBAction)save:(id)sendr
{
//   [canvas mdCanvasView]->export_png("/tmp/model.png");
  [canvas mdCanvasView]->export_pdf("/tmp/model.pdf");
}

@end




int main(int argc, char **argv)
{
  return NSApplicationMain(argc, (const char**)argv);
}