/* Copyright (C) 2007 - 2008 MySQL AB, 2008 Sun Microsystems, Inc.

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; version 2 of the License.

   There are special exceptions to the terms and conditions of the GPL
   as it is applied to this software. View the full text of the
   exception in file EXCEPTIONS-CONNECTOR-C++ in the directory of this
   software distribution.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#ifndef _MYSQL_DBC_EXCECPTION_H_
#define _MYSQL_DBC_EXCECPTION_H_

#include <cppconn/exception.h>

/*
TODO - FIXME

I've had a chat with Andrey. We are not sure if we need this extra type of
exception. Client side errors cause MySQL_SQLExceptions and server side
errors result in SQLExceptions. However, the interfaces of the exception classes
are identical. The only purpose of the MySQL_SQLException is to transport
the information "comes from the client".

Earlier versions of C/C++ did have a driver - driver manager design. We've almost
(99% sure) given up that design. Therefore its questionable wheter we need a deeply
nested exceptions class structure. Nowadays sql::SQLException is modeled after
JDBC's SQLException. We can use it for any kind of SQL errors (client and server).
It offers all we need for the MySQL driver.

Deriving from it just for the purpose of indicating a MySQL driver error
is pointless - there is only one driver (driver - driver manager
design is almost gone). And if we want to distinguish between server and client
errors we can do it on the level of sql::SQLException - no need to do it on the
sql::mysql level.

Suggestion: clearify driver - driver manager situation, remove MySQL_SQLException after
successful WB test run of C/C++.
*/

namespace sql
{
namespace mysql
{

class MySQL_SQLException : public sql::SQLException
{
public:
	MySQL_SQLException(const MySQL_SQLException& e) : SQLException(e.what(), e.sql_state, e.errNo) { next= e.next; }
	MySQL_SQLException(const char *error);
	MySQL_SQLException(const char *error, const char* sqlerror, unsigned int errNo);

private:
	MEMORY_ALLOC_OPERATORS(MySQL_SQLException)
};

}; /* namespace mysql */
}; /* namespace sql */

#endif // _MYSQL_DBC_EXCECPTION_H_

/*
 * Local variables:
 * tab-width: 4
 * c-basic-offset: 4
 * End:
 * vim600: noet sw=4 ts=4 fdm=marker
 * vim<600: noet sw=4 ts=4
 */
