#ifndef _WF_BOX_H_
#define _WF_BOX_H_


#include "mforms/box.h"

#include "wf_view.h"

using namespace System;
using namespace System::Windows::Forms;
using namespace System::Windows::Forms::Layout;
using namespace System::Runtime::InteropServices;

namespace MySQL {
  namespace Forms {

    public ref class BaseBox : public TableLayoutPanel
    {
      Dictionary<Control^, bool> controlInfo;

    public:
      bool GetControlExpands(Control ^ctl)
      {
        return controlInfo[ctl];
      }

      void Add(Control ^ctl, bool expands)
      {
        controlInfo[ctl]= expands;
        Controls->Add(ctl);
      }
    };


    public ref class HBox : public BaseBox
    {
    public:
      HBox()
      {
        RowCount= 1;
        ColumnCount= 1;
        GrowStyle= TableLayoutPanelGrowStyle::AddColumns;
      }

      virtual Drawing::Size GetPreferredSize(Drawing::Size proposed) override
      {
        Drawing::Size pref;

        for each (Control ^ctl in Controls)
        {
          if (!ctl->Visible) continue;

          if (ctl->Height+ctl->Margin.Top+ctl->Margin.Bottom > pref.Height)
            pref.Height= ctl->Height+ctl->Margin.Top+ctl->Margin.Bottom;
          
          pref.Width+= ctl->PreferredSize.Width + ctl->Margin.Left + ctl->Margin.Right;
        }

        pref.Height+= Padding.Top + Padding.Bottom;

        return pref;
      }

      virtual Drawing::Size GetPreferredSize()
      {
        return GetPreferredSize(Drawing::Size(0, 0));
      }
    };

    public ref class VBox : public BaseBox
    {
    public:
      VBox()
      {
        RowCount= 1;
        ColumnCount= 1;
        GrowStyle= TableLayoutPanelGrowStyle::AddRows;
      }

      virtual Drawing::Size GetPreferredSize(Drawing::Size proposed) override
      {
        Drawing::Size pref;

        for each (Control ^ctl in Controls)
        {
          if (!ctl->Visible) continue;

          if (ctl->Width + ctl->Margin.Left+ctl->Margin.Right > pref.Width)
            pref.Width= ctl->PreferredSize.Width + ctl->Margin.Left+ctl->Margin.Right;
          
          pref.Height+= ctl->PreferredSize.Height + ctl->Margin.Top+ctl->Margin.Bottom;
        }

        pref.Width+= Padding.Left + Padding.Right;

        return pref;
      }
    };


    public ref class BoxImpl : public ViewImpl
    {
    protected:
      BoxImpl(::mforms::Box *self)
        : ViewImpl(self)
      {
        _spacing= 0;
      }

      static bool create(::mforms::Box *self, bool horizontal)
      {
        BoxImpl ^box= gcnew BoxImpl(self);

        if (box != nullptr)
        {
          box->_horizontal= horizontal;

          if (horizontal)
            box->wcontrol= gcnew HBox();
          else
            box->wcontrol= gcnew VBox();
          box->wcontrol->Margin.All= 0;
          box->wcontrol->Padding.All= 0;
          box->wcontrol->AutoSize= true;
          return true;
        }
        return false;
      }


      static void add(mforms::Box *self, mforms::View *child, bool expand, bool fill)
      {
        BoxImpl^ box= (BoxImpl^)ObjectImpl::FromUnmanaged(self);

        if (box != nullptr)
        {
          box->add(((ViewImpl^)ObjectImpl::FromUnmanaged(child))->get_control(), expand, fill);
        }
      }


      static void add_end(mforms::Box *self, mforms::View *child, bool expand, bool fill)
      {
        BoxImpl^ box= (BoxImpl^)ObjectImpl::FromUnmanaged(self);

        if (box != nullptr)
        {
          box->add_end(((ViewImpl^)ObjectImpl::FromUnmanaged(child))->get_control(), expand, fill);
        }
      }


      static void set_spacing(mforms::Box *self, int space)
      {
        BoxImpl^ box= (BoxImpl^)ObjectImpl::FromUnmanaged(self);

        if (box != nullptr)
        {
          box->set_spacing(space);
        }
      }


      static void set_padding(mforms::Box *self, int padding)
      {
        BoxImpl^ box= (BoxImpl^)ObjectImpl::FromUnmanaged(self);

        if (box != nullptr)
        {
          box->set_padding(padding);
        }
      }


      static void set_back_color(mforms::Box *self, const std::string &color)
      {
        BoxImpl^ box= (BoxImpl^)ObjectImpl::FromUnmanaged(self);

        if (box != nullptr)
        {
          box->set_back_color(CppStringToNative(color));
        }
      }


    public:
      int _spacing;
      bool _horizontal;

      void add(Windows::Forms::Control ^control, bool expand, bool fill);
      void add_end(Windows::Forms::Control ^control, bool expand, bool fill);

      void set_spacing(int space);
      void set_padding(int padding);
      
      void set_back_color(String ^color);

      static void init(Manager ^mgr)
      {
        ::mforms::ControlFactory *f= ::mforms::ControlFactory::get_instance();

        DEF_CALLBACK2(bool, ::mforms::Box*, bool, mgr, f->_box_impl, BoxImpl, create);
        DEF_CALLBACK4(void, ::mforms::Box*, ::mforms::View*, bool, bool, mgr, f->_box_impl, BoxImpl, add);
        DEF_CALLBACK4(void, ::mforms::Box*, ::mforms::View*, bool, bool, mgr, f->_box_impl, BoxImpl, add_end);
        DEF_CALLBACK2(void, ::mforms::Box*, int, mgr, f->_box_impl, BoxImpl, set_spacing);
        DEF_CALLBACK2(void, ::mforms::Box*, int, mgr, f->_box_impl, BoxImpl, set_padding);
        DEF_CALLBACK2(void, ::mforms::Box*, const std::string&, mgr, f->_box_impl, BoxImpl, set_back_color);
      }
    };

  };
};

#endif
