#ifndef _WF_FILECHOOSER_H_
#define _WF_FILECHOOSER_H_

#include <mforms/mforms.h>


#include "wf_filechooser.h"
#include "wf_mforms.h"

using namespace System;
using namespace Windows::Forms;


namespace MySQL {
  namespace FileChoosers {

    public ref class FileChooserImpl : public ViewImpl
    {
      static bool create(::mforms::FileChooser *self, ::mforms::FileChooserType type)
      {
        FileChooserImpl ^form= gcnew FileChooserImpl(self);

        if (form != nullptr)
        {
          Windows::Forms::FileDialog ^dlg;
          switch (type)
          {
          case ::mforms::OpenFile:
            dlg= gcnew Windows::Forms::OpenFileDialog();
            break;
          case ::mforms::SaveFile:
            dlg= gcnew Windows::Forms::SaveFileDialog();
            break;
          case ::mforms::OpenDirectory:
            dlg= gcnew Windows::Forms::OpenFileDialog();
            break;
          default:
            return false;
          }
          dlg->RestoreDirectory= true;
          form->wcontrol= dlg;
          return true;
        }
        return false;
      }

      static void set_title(::mforms::FileChooser *self, const std::string &title)
      {
        FileChooserImpl^ form= (FileChooserImpl^)ObjectImpl::FromUnmanaged(self);

        if (form != nullptr)
        {
          ((FileDialog^)form->wcontrol)->Text= CppStringToNative(title);
        }
      }

      static bool show_modal(::mforms::FileChooser *self)
      {
        FileChooserImpl^ form= (FileChooserImpl^)ObjectImpl::FromUnmanaged(self);

        if (form != nullptr)
        {
          Windows::Forms::FileDialog ^f= (FileDialog^)form->wcontrol;

          return f->ShowDialog() == DialogResult::OK;
        }
        return false;
      }

      static void set_path(FileChooser *self, const std::string &path)
      {
        FileChooserImpl^ form= (FileChooserImpl^)ObjectImpl::FromUnmanaged(self);

        if (form != nullptr)
        {
          Windows::Forms::FileDialog ^f= (FileDialog^)form->wcontrol;

          f->FileName= CppStringToNative(path);
        }
      }

      static std::string get_path(FileChooser *self)
      {
        FileChooserImpl^ form= (FileChooserImpl^)ObjectImpl::FromUnmanaged(self);

        if (form != nullptr)
        {
          Windows::Forms::FileDialog ^f= (FileDialog^)form->wcontrol;

          return NativeToCppString(f->FileName);
        }
        return "";
      }

      static void set_extensions(FileChooser *self, const std::string &extensions, const std::string &default_extension)
      {
        FileChooserImpl^ form= (FileChooserImpl^)ObjectImpl::FromUnmanaged(self);

        if (form != nullptr)
        {
          Windows::Forms::FileDialog ^f= (FileDialog^)form->wcontrol;

          f->DefaultExt= CppStringToNative(default_extension);
          f->Filter= CppStringToNative(extensions);
        }
      }

      FileChooserImpl(::mforms::FileChooser *form)
        : ViewImpl(form)
      {
      }

    public:
      static void init(Manager ^mgr)
      {
        ::mforms::ControlFactory *f= ::mforms::ControlFactory::get_instance();

        DEF_CALLBACK2(bool, ::mforms::FileChooser*, ::mforms::FileChooserType, mgr, f->_filechooser_impl, FileChooserImpl, create);
        DEF_CALLBACK2(void, ::mforms::FileChooser*, const std::string&, mgr, f->_filechooser_impl, FileChooserImpl, set_title);
        DEF_CALLBACK1(bool, ::mforms::FileChooser*, mgr, f->_filechooser_impl, FileChooserImpl, show_modal);
        DEF_CALLBACK3(void, ::mforms::FileChooser*, const std::string&, const std::string&, mgr, f->_filechooser_impl, FileChooserImpl, set_extensions);
        DEF_CALLBACK2(void, ::mforms::FileChooser*, const std::string&, mgr, f->_filechooser_impl, FileChooserImpl, set_path);
        DEF_CALLBACK1(std::string, ::mforms::FileChooser*, mgr, f->_filechooser_impl, FileChooserImpl, get_path);
      }
    };

  };
};

#endif
