#ifndef _WF_SELECTOR_H_
#define _WF_SELECTOR_H_


#include "mforms/selector.h"

#include "wf_view.h"

using namespace System;
using namespace Windows::Forms;

namespace MySQL {
  namespace Forms {

    public ref class SelectorImpl : public ViewImpl
    {
      bool _islist;

    protected:
      SelectorImpl(::mforms::Selector *self)
        : ViewImpl(self)
      {
      }

      static bool create(::mforms::Selector *self, bool list)
      {
        SelectorImpl ^selector= gcnew SelectorImpl(self);

        if (selector != nullptr)
        {
          selector->_islist= list;
          if (!list)
          {
            ComboBox ^combo= gcnew Windows::Forms::ComboBox();
            selector->wcontrol= combo;
            combo->DropDownStyle= ComboBoxStyle::DropDownList;
          }
          else
            selector->wcontrol= gcnew Windows::Forms::ListBox();
          return true;
        }
        return false;
      }

      static void clear(::mforms::Selector *self)
      {
        SelectorImpl^ selector= (SelectorImpl^)ObjectImpl::FromUnmanaged(self);

        if (selector != nullptr)
        {
          if (selector->_islist)
            ((ListBox^)selector->wcontrol)->Items->Clear();
          else
            ((ComboBox^)selector->wcontrol)->Items->Clear();
        }
      }

      static int add_item(::mforms::Selector *self, const std::string &item)
      {
        SelectorImpl^ selector= (SelectorImpl^)ObjectImpl::FromUnmanaged(self);

        if (selector != nullptr)
        {
          if (selector->_islist)
            ((ListBox^)selector->wcontrol)->Items->Add(CppStringToNative(item));
          else
            ((ComboBox^)selector->wcontrol)->Items->Add(CppStringToNative(item));
        }
        return 0;
      }

      static void add_items(::mforms::Selector *self, const std::list<std::string> &items)
      {
        SelectorImpl^ selector= (SelectorImpl^)ObjectImpl::FromUnmanaged(self);

        if (selector != nullptr)
        {
          cli::array<String^> ^strings= gcnew cli::array<String^>(items.size());
          int i= 0;

          for (std::list<std::string>::const_iterator iter= items.begin(); iter != items.end(); ++iter)
          {
            strings[i++]= CppStringToNative(*iter);
          }

          if (selector->_islist)
            ((ListBox^)selector->wcontrol)->Items->AddRange(strings);
          else
            ((ComboBox^)selector->wcontrol)->Items->AddRange(strings);
        }
      }

      static std::string get_text(::mforms::Selector *self)
      {
        SelectorImpl^ selector= (SelectorImpl^)ObjectImpl::FromUnmanaged(self);

        if (selector != nullptr)
        {
          if (selector->_islist)
          {
            if (((ListBox^)selector->wcontrol)->SelectedItem!=nullptr)
              return NativeToCppString((String^)((ListBox^)selector->wcontrol)->SelectedItem);
          }
          else
          {
            if (((ComboBox^)selector->wcontrol)->SelectedItem!=nullptr)
              return NativeToCppString((String^)((ComboBox^)selector->wcontrol)->SelectedItem);
          }
        }
        return "";
      }

      static void set_index(::mforms::Selector *self, int index)
      {
        SelectorImpl^ selector= (SelectorImpl^)ObjectImpl::FromUnmanaged(self);

        if (selector != nullptr)
        {
          if (selector->_islist)
            ((ListBox^)selector->wcontrol)->SelectedIndex= index;
          else
            ((ComboBox^)selector->wcontrol)->SelectedIndex= index;
        }
      }

      static int get_index(::mforms::Selector *self)
      {
        SelectorImpl^ selector= (SelectorImpl^)ObjectImpl::FromUnmanaged(self);

        if (selector != nullptr)
        {
          if (selector->_islist)
            return ((ListBox^)selector->wcontrol)->SelectedIndex;
          else
            return ((ComboBox^)selector->wcontrol)->SelectedIndex;
        }
        return -1;
      }


    public:
      static void init(Manager ^mgr)
      {
        ::mforms::ControlFactory *f= ::mforms::ControlFactory::get_instance();

        DEF_CALLBACK2(bool, ::mforms::Selector*, bool, mgr, f->_selector_impl, SelectorImpl, create);
        DEF_CALLBACK1(void, ::mforms::Selector*, mgr, f->_selector_impl, SelectorImpl, clear);
        DEF_CALLBACK2(int, ::mforms::Selector*, const std::string&, mgr, f->_selector_impl, SelectorImpl, add_item);
        DEF_CALLBACK2(void, ::mforms::Selector*, const std::list<std::string>&, mgr, f->_selector_impl, SelectorImpl, add_items);
        DEF_CALLBACK2(void, ::mforms::Selector*, int, mgr, f->_selector_impl, SelectorImpl, set_index);
        DEF_CALLBACK1(std::string, ::mforms::Selector*, mgr, f->_selector_impl, SelectorImpl, get_text);
        DEF_CALLBACK1(int, ::mforms::Selector*, mgr, f->_selector_impl, SelectorImpl, get_index);
      }
    };

  };
};


#endif
