using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.Runtime.InteropServices;
using MySQL.Grt;
using MySQL.Utilities;
using MySQL.GUI.Workbench.Plugins;
using MySQL.GUI.Workbench.Plugins.Db;

namespace MySQL.GUI.Workbench.Plugins.DbMySQL
{
  using System.Collections;
  using Pages = LinkedList<PageInfo>;
  using Db_object_type = MySQL.GUI.Workbench.Plugins.Db.Db_plugin.Db_object_type;
  using DbMySQLScriptSync = MySQL.GUI.Workbench.Plugins.DbMySQLScriptSync.DbMySQLScriptSync;
  //using DiffTreeControl = MySQL.GUI.Workbench.Plugins.Sync.DiffTreeControl;
  using Db_plugin = MySQL.GUI.Workbench.Plugins.Db.Db_plugin;

  public partial class DiffReportingWizardPages : WizardPluginPages
  {
    private DbMySQLDiffReporting pluginBE;

    private SchemaObjectFilter user_filter;
    private SchemaObjectFilter table_filter;
    private SchemaObjectFilter view_filter;
    private SchemaObjectFilter routine_filter;
    private SchemaObjectFilter trigger_filter;

    private List<String> schemataNames;

    // schemata selection
    private int schema_large_image_index;
    private int schema_small_image_index;

    // dbms connection
    private DbConnection dbConnection;

    private bool showAdvancedConnectionLogPage;
    private bool showAdvancedSchemataObjectsExtractionLogPage;

    private Dictionary<Db_object_type, SchemaObjectFilter> dbObjectFilters;

    // diff tree
    //private DiffTreeControl diffTreeControl;
    private DbMySQLScriptSync basePluginBE;

    public DiffReportingWizardPages(WizardPlugin wizard)
      : base(wizard)
    {
      this.basePluginBE = new DbMySQLScriptSync(wizard.GrtManager);
      pluginBE = new DbMySQLDiffReporting(wizard.GrtManager);
      
      schemataNames = new List<String>();

      InitializeComponent();

      // dbms connection
      dbConnection = new DbConnection();
      dbConnection.Parent = connectionPanel;
      dbConnection.Dock = DockStyle.Fill;
      dbConnection.Padding = new System.Windows.Forms.Padding(5);
      dbConnection.Margin = new System.Windows.Forms.Padding(5);

      showAdvancedConnectionLogPage = false;
      showAdvancedSchemataObjectsExtractionLogPage = false;

      // schemata selection/objects filter
      schema_large_image_index = GrtIconManager.get_instance()
        .add_icon_to_imagelist(pluginBE.get_left_db().schema_icon_id(IconSize.Icon32));
      schema_small_image_index = GrtIconManager.get_instance()
        .add_icon_to_imagelist(pluginBE.get_left_db().schema_icon_id(IconSize.Icon16));

      schemataListView.SmallImageList = GrtIconManager.ImageList16;
      schemataListView.LargeImageList = GrtIconManager.ImageList32;

      dbObjectFilters = new Dictionary<Db_object_type, SchemaObjectFilter>();

      AddPage(optionsPanel,
        InitOptionsPage,
        null,
        null,
        null,
        null,
        "Diff Sources",
        "Select Diff Report Sources");

      AddPage(connectionPanel,
        InitializeLeftConnectionPage,
        null,
        ProcessConnectionPage,
        null,
        SkipLeftConnectionPage,
        "Left catalog: Connect to DBMS",
        "Set parameters for connection to DBMS.");

      AddPage(connectionLogPanel,
        InitializeLeftConnectionLogPage,
        CleanupConnectionLogPage,
        null,
        ToggleAdvancedConnectionLogPage,
        SkipLeftConnectionPage,
        "Left catalog: Connect to DBMS (Progress/Results)",
        "See the progress/results of establishing connection to DBMS.");

      AddPage(filterSchemataPanel,
        InitializeLeftFilterSchemataPage,
        null,
        null,
        null,
        SkipLeftConnectionPage,
        "Left catalog: Select Schemata",
        "Choose schemata containing objects of interest.");

      AddPage(schemataObjectsExtractionLogPanel,
        InitializeLeftSchemataObjectsExtractionLogPage,
        CleanupSchemataObjectsExtractionLogPage,
        null,
        ToggleAdvancedSchemataObjectsExtractionLogPage,
        SkipLeftConnectionPage,
        "Left catalog: Select Schemata (Progress/Results)",
        "See the progress/results of retrieving objects description from selected schemata.");

      AddPage(connectionPanel,
        InitializeRightConnectionPage,
        null,
        ProcessConnectionPage,
        null,
        SkipRightConnectionPage,
        "Right catalog: Connect to DBMS",
        "Set parameters for connection to DBMS.");

      AddPage(connectionLogPanel,
        InitializeRightConnectionLogPage,
        CleanupConnectionLogPage,
        null,
        ToggleAdvancedConnectionLogPage,
        SkipRightConnectionPage,
        "Right catalog: Connect to DBMS (Progress/Results)",
        "See the progress/results of establishing connection to DBMS.");

      AddPage(filterSchemataPanel,
        InitializeRightFilterSchemataPage,
        null,
        null,
        null,
        SkipRightConnectionPage,
        "Right catalog: Select Schemata",
        "Choose schemata containing objects of interest.");

      AddPage(schemataObjectsExtractionLogPanel,
        InitializeRightSchemataObjectsExtractionLogPage,
        CleanupSchemataObjectsExtractionLogPage,
        null,
        ToggleAdvancedSchemataObjectsExtractionLogPage,
        SkipRightConnectionPage,
        "Right catalog: Select Schemata (Progress/Results)",
        "See the progress/results of retrieving objects description from selected schemata.");

      AddPage(generatedScriptPanel,
        InitGeneratedScriptPage,
        null,
        null,
        null,
        null,
        "Catalog Diff Report",
        "Review the catalog diff report");
    }

    #region Options Page Routines

    private string InitOptionsPage()
    {
      List<Control> controlsToSkipOnce = new List<Control>();
      controlsToSkipOnce.Add(wizard.NextPageButton);
      wizard.ControlsToSkipOnce(controlsToSkipOnce);

      radioRightSourceTree.Checked = true;
      radioLeftSourceServer.Checked = true;

      textRightFileName.Enabled = false;
      buttonRightFileBrowse.Enabled = false;
      textLeftFileName.Enabled = false;
      buttonLeftFileBrowse.Enabled = false;
      updateNextButtonState();

      return "";
    }

    #endregion

    #region Left/right ConnectionPage common routines

    private string InitializeLeftConnectionPage(Db_plugin db)
    {
      //if (dbConnection.Backend == null)
      {
        connectionPanel.SuspendLayout();
        dbConnection.Backend = db.db_conn();
        connectionPanel.ResumeLayout();
      }

      return "";
    }

    private string ProcessConnectionPage()
    {
      return dbConnection.Backend.validate_driver_params();
    }

    private string InitializeConnectionLogPage(Db_plugin db)
    {
      List<String> messages = new List<String>();
      string errMsg = "";

      taskLblConnToSrcDb.TaskState = MySQLTaskStatusLabelState.TaskOpen;
      taskLblRetrSchemaInfo.TaskState = MySQLTaskStatusLabelState.TaskOpen;
      StepProlog(lConnResult, lConnStage);

      TaskMediator(connectionLogPanel, connectionLogTextBox, messages);

      // next task
      if (0 == errMsg.Length)
      {
        lConnStage.Text = "Connecting to DBMS...";
        connectionLogPanel.Update();

        try
        {
          db.db_conn().test_connection();
          taskLblConnToSrcDb.TaskState = MySQLTaskStatusLabelState.TaskCompleted;
        }
        catch (Exception)
        {
          taskLblConnToSrcDb.TaskState = MySQLTaskStatusLabelState.TaskError;
          errMsg = "Failed to connect database.";
        }
      }
      else
        taskLblConnToSrcDb.TaskState = MySQLTaskStatusLabelState.TaskError;

      TaskMediator(connectionLogPanel, connectionLogTextBox, messages);

      // next task
      if (0 == errMsg.Length)
      {
        lConnStage.Text = "Retrieving schemata description from DBMS...";
        connectionLogPanel.Update();

        try
        {
          db.load_schemata(ref schemataNames);
          taskLblRetrSchemaInfo.TaskState = MySQLTaskStatusLabelState.TaskCompleted;

          // closes bug #35878 (duplicate list contents + crash when diff live+live)
          schemataListView.Items.Clear();

          // fill schemata list
          for (int n = 0; n < schemataNames.Count; ++n)
            schemataListView.Items.Add(schemataNames[n]).ImageIndex = schema_large_image_index;
        }
        catch(Exception)
        {
          errMsg = "Failed to load schemata.";
          taskLblRetrSchemaInfo.TaskState = MySQLTaskStatusLabelState.TaskError;
        }
      }
      else
        taskLblRetrSchemaInfo.TaskState = MySQLTaskStatusLabelState.TaskError;

      messages.Add(errMsg);
      TaskMediator(connectionLogPanel, connectionLogTextBox, messages);

      StepEpilog(errMsg, lConnResult, lConnStage);


      // returning a message here results in a modal dialog with the message, 
      // which is neither needed nor wanted
      //return errMsg;
      return "";
    }

    private string CleanupConnectionLogPage()
    {
      connectionLogTextBox.Text = "";
      schemataListView.Items.Clear();
      return "";
    }

    private string ToggleAdvancedConnectionLogPage()
    {
      showAdvancedConnectionLogPage = !showAdvancedConnectionLogPage;
      connectionLogGroupBox.Visible = showAdvancedConnectionLogPage;
      return "";
    }

    private string InitializeFilterSchemataPage(Db_plugin db)
    {
      List<int> indexes = new List<int>();
      db.def_schemata_selection(ref indexes);
      foreach (int n in indexes)
        schemataListView.Items[n].Selected = true;
      schemataListView.Select();
      return "";
    }

    private void cbShowAsList_CheckedChanged(object sender, EventArgs e)
    {
      CheckBox cb = sender as CheckBox;
      if (null == cb)
        return;

      ListView.SelectedIndexCollection indices = schemataListView.SelectedIndices;
      schemataListView.View = cb.Checked ? View.List : View.LargeIcon;
      int imageIndex = cb.Checked ? schema_small_image_index : schema_large_image_index;
      foreach (ListViewItem item in schemataListView.Items)
        item.ImageIndex = imageIndex;
      foreach (int index in indices)
        schemataListView.Items[index].Selected = true;
      schemataListView.Select();
    }

    private string InitializeSchemataObjectsExtractionLogPage(Db_plugin db)
    {
      List<String> messages = new List<String>();
      string errMsg = "";

      taskLblRetrObjInfo.TaskState = MySQLTaskStatusLabelState.TaskOpen;
      taskLblCheckRetrObjInfoResults.TaskState = MySQLTaskStatusLabelState.TaskOpen;
      StepProlog(lSyncResult, lSyncStage);

      TaskMediator(schemataObjectsExtractionLogPanel, schemataObjectsExtractionLogTextBox, messages);

      // next task
      if (0 == errMsg.Length)
      {
        // restrict objects retrieving by selected schemata
        IEnumerator enumerator = schemataListView.SelectedIndices.GetEnumerator();
        if (null == enumerator)
          taskLblRetrObjInfo.TaskState = MySQLTaskStatusLabelState.TaskCompleted;
        else
        {
          lSyncStage.Text = "Retrieving objects description from selected schemata...";
          schemataObjectsExtractionLogPanel.Update();

          List<int> schemataIndexes = new List<int>(schemataListView.SelectedIndices.Count);
          while (enumerator.MoveNext())
            schemataIndexes.Add((int)enumerator.Current);
          db.schemata_selection(schemataIndexes, false);

          try
          {
            db.load_db_objects(Db_object_type.dbotTable);
            db.load_db_objects(Db_object_type.dbotView);
            db.load_db_objects(Db_object_type.dbotRoutine);
            db.load_db_objects(Db_object_type.dbotTrigger);
              //!            || !pluginBE.load_db_objects(Db_object_type.dbotUser, ref messages)
            taskLblRetrObjInfo.TaskState = MySQLTaskStatusLabelState.TaskCompleted;
          }
          catch (Exception)
          {
            errMsg = "Failed to fetch selected schemata objects.";
            taskLblRetrObjInfo.TaskState = MySQLTaskStatusLabelState.TaskError;
          }
        }
      }
      else
        taskLblRetrObjInfo.TaskState = MySQLTaskStatusLabelState.TaskCompleted;

      TaskMediator(schemataObjectsExtractionLogPanel, schemataObjectsExtractionLogTextBox, messages);

      // next task
      if (0 == errMsg.Length)
      {
        lSyncStage.Text = "Checking results...";
        schemataObjectsExtractionLogPanel.Update();

        taskLblCheckRetrObjInfoResults.TaskState = MySQLTaskStatusLabelState.TaskCompleted;
      }
      else
        taskLblCheckRetrObjInfoResults.TaskState = MySQLTaskStatusLabelState.TaskError;

      messages.Add(errMsg);
      TaskMediator(schemataObjectsExtractionLogPanel, schemataObjectsExtractionLogTextBox, messages);

      StepEpilog(errMsg, lSyncResult, lSyncStage);

      //return errMsg;
      return "";
    }

    private string CleanupSchemataObjectsExtractionLogPage()
    {
      schemataObjectsExtractionLogTextBox.Text = "";
      return "";
    }

    private string ToggleAdvancedSchemataObjectsExtractionLogPage()
    {
      showAdvancedSchemataObjectsExtractionLogPage = !showAdvancedSchemataObjectsExtractionLogPage;
      schemataObjectsExtractionLogGroupBox.Visible = showAdvancedSchemataObjectsExtractionLogPage;
      return "";
    }

    #endregion

    #region LeftConnectionPage

    private string InitializeLeftConnectionPage()
    {
      return InitializeLeftConnectionPage(pluginBE.get_left_db());
    }

    private string SkipLeftConnectionPage()
    {
      return radioLeftSourceServer.Checked ? "" : "skip";
    }

    #endregion LeftConnectionPage

    #region RightConnectionPage

    private string InitializeRightConnectionPage()
    {
      return InitializeLeftConnectionPage(pluginBE.get_right_db());
    }

    private string SkipRightConnectionPage()
    {
      return radioRightSourceServer.Checked ? "" : "skip";
    }

    #endregion ConnectionPage2

    #region LeftConnectionLogPage

    private string InitializeLeftConnectionLogPage()
    {
      return InitializeConnectionLogPage(pluginBE.get_left_db());
    }

    #endregion ConnectionLogPage

    #region rightConnectionLogPage

    private string InitializeRightConnectionLogPage()
    {
      return InitializeConnectionLogPage(pluginBE.get_right_db());
    }

    #endregion ConnectionLogPage

    #region LeftFilterSchemataPage

    private string InitializeLeftFilterSchemataPage()
    {
      return InitializeFilterSchemataPage(pluginBE.get_left_db());
    }
  
    #endregion LeftFilterSchemataPage

    #region RightFilterSchemataPage

    private string InitializeRightFilterSchemataPage()
    {
      return InitializeFilterSchemataPage(pluginBE.get_right_db());
    }

    #endregion RightFilterSchemataPage

    #region LeftSchemataObjectsExtractionLogPage

    private string InitializeLeftSchemataObjectsExtractionLogPage()
    {
      return InitializeSchemataObjectsExtractionLogPage(pluginBE.get_left_db());
    }

    #endregion LeftSchemataObjectsExtractionLogPage

    #region RightSchemataObjectsExtractionLogPage

    private string InitializeRightSchemataObjectsExtractionLogPage()
    {
      return InitializeSchemataObjectsExtractionLogPage(pluginBE.get_right_db());
    }

    #endregion RightSchemataObjectsExtractionLogPage

    #region Generated Script Page

    private string InitGeneratedScriptPage()
    {
      MySQL.Grt.GrtValue left_cat = radioLeftSourceServer.Checked ?
        pluginBE.get_left_db().db_catalog() : new MySQL.Grt.GrtValue();

      MySQL.Grt.GrtValue right_cat = radioRightSourceServer.Checked ?
        pluginBE.get_right_db().db_catalog() : new MySQL.Grt.GrtValue();

      outputScriptTextBox.Text = pluginBE.generate_report(
        radioLeftSourceFile.Checked ? textLeftFileName.Text : "",
        radioRightSourceFile.Checked ? textRightFileName.Text : "",
        left_cat, right_cat);

      return "";
    }

    #endregion

    private void updateNextButtonState()
    {
      wizard.NextPageButton.Enabled =
        (!textRightFileName.Enabled || System.IO.File.Exists(textRightFileName.Text)) &&
        (!textLeftFileName.Enabled || System.IO.File.Exists(textLeftFileName.Text));
    }

    private void textFileName_TextChanged(object sender, EventArgs e)
    {
      updateNextButtonState();
    }

    private void buttonLeftFileBrowse_click(object sender, EventArgs e)
    {
      OpenFileDialog openFileDialog = new OpenFileDialog();
      openFileDialog.DefaultExt = "sql";
      openFileDialog.Filter = "SQL Script Files (*.sql)|*.sql|All Files (*.*)|*.*";
      openFileDialog.Title = "Open SQL Script File";
      openFileDialog.RestoreDirectory = true;

      DialogResult res = openFileDialog.ShowDialog();
      if (res == DialogResult.OK)
      {
        textLeftFileName.Text = openFileDialog.FileName;
      }
    }

    private void buttonRightFileBrowse_click(object sender, EventArgs e)
    {
      OpenFileDialog openFileDialog = new OpenFileDialog();
      openFileDialog.DefaultExt = "sql";
      openFileDialog.Filter = "SQL Script Files (*.sql)|*.sql|All Files (*.*)|*.*";
      openFileDialog.Title = "Open SQL Script File";
      openFileDialog.RestoreDirectory = true;

      DialogResult res = openFileDialog.ShowDialog();
      if (res == DialogResult.OK)
      {
        textRightFileName.Text = openFileDialog.FileName;
      }
    }

    private void radioLeftSourceTree_check(object sender, EventArgs e)
    {
      textLeftFileName.Enabled = false;
      buttonLeftFileBrowse.Enabled = false;
      radioRightSourceTree.Enabled = false;
      updateNextButtonState();
    }

    private void radioRightSourceTree_check(object sender, EventArgs e)
    {
      textRightFileName.Enabled = false;
      buttonRightFileBrowse.Enabled = false;
      radioLeftSourceTree.Enabled = false;
      updateNextButtonState();
    }

    private void radioLeftSourceFile_check(object sender, EventArgs e)
    {
      textLeftFileName.Enabled = true;
      buttonLeftFileBrowse.Enabled = true;
      radioRightSourceTree.Enabled = true;
      updateNextButtonState();
    }

    private void radioRightSourceFile_check(object sender, EventArgs e)
    {
      textRightFileName.Enabled = true;
      buttonRightFileBrowse.Enabled = true;
      radioLeftSourceTree.Enabled = true;
      updateNextButtonState();
    }

    private void radioLeftSourceServer_check(object sender, EventArgs e)
    {
      textLeftFileName.Enabled = false;
      buttonLeftFileBrowse.Enabled = false;
      radioRightSourceTree.Enabled = true;
      updateNextButtonState();
    }

    private void radioRightSourceServer_check(object sender, EventArgs e)
    {
      textRightFileName.Enabled = false;
      buttonRightFileBrowse.Enabled = false;
      radioLeftSourceTree.Enabled = true;
      updateNextButtonState();
    }

    private void SaveAsButton_click(object sender, EventArgs e)
    {
      SaveFileDialog saveFileDialog = new SaveFileDialog();
      saveFileDialog.DefaultExt = "txt";
      saveFileDialog.Filter = "SQL Script Files (*.txt)|*.txt|All Files (*.*)|*.*";
      saveFileDialog.Title = "Output Report File";
      saveFileDialog.RestoreDirectory = true;

      if (saveFileDialog.ShowDialog() == DialogResult.OK)
      {
        String err = basePluginBE.save_script_to_file(outputScriptTextBox.Text, saveFileDialog.FileName);
        if (err.Length > 0)
          System.Windows.Forms.MessageBox.Show(err, "Error saving file", MessageBoxButtons.OK, MessageBoxIcon.Error);
      }
    }
  }
}