using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using WeifenLuo.WinFormsUI.Docking;
using Aga.Controls.Tree;
using MySQL.Grt;
using MySQL.Utilities;
using MySQL.Grt.Db;

namespace MySQL.GUI.Workbench.Plugins
{
  public partial class DbMysqlViewEditor : ObjectEditorPlugin
  {
    #region Member Variables

    private MySQLViewEditorBE viewEditorBE { get { return editorBE as MySQLViewEditorBE; } } 
    private DbObjectEditorPages dbObjectEditorPages;
    protected Timer sqlEditorTimer;

    #endregion

    #region Constructors

    protected DbMysqlViewEditor()
    {
      InitializeComponent();
    }

    public DbMysqlViewEditor(GrtManager GrtManager, GrtValue GrtList)
      : base(new MySQLViewEditorBE(GrtManager, GrtList))
    {
      InitializeComponent();

      // Init BE
      viewEditorBE.set_sql_parser_log_cb(new DBObjectEditorBE.VoidStringListDelegate(sqlEditorParseLogCallback));

      // Init timer
      sqlEditorTimer = new Timer();
      sqlEditorTimer.Interval = 2000;
      sqlEditorTimer.Tick += new EventHandler(sqlEditorTimerHandler);

      InitializingControls = true;
      try
      {
        InitFormData();

        CallRefreshFormData();
      }
      finally
      {
        InitializingControls = false;
      }
      viewEditorBE.add_refresh_ui_handler(RefreshFormDataInvoke);
    }

    #endregion

    #region IWorkbenchDocument implementation

    public override UIForm BackendClass
    {
      get { return viewEditorBE; }
    }

    #endregion

    #region ObjectEditorPlugin Overrides

    public override bool ChangeGrtList(GrtManager GrtManager, GrtValue GrtList)
    {
      // Make sure the changes are stored
      DbMysqlViewEditor_FormClosing(null, null);

      InitializingControls = true;

      try
      {
        grtManager = GrtManager;
        grtList = GrtList;

        if (editorBE != null)
          editorBE.disable_auto_refresh();
        editorBE = new MySQLViewEditorBE(GrtManager, GrtList);

        InitFormData();

        RefreshFormData();
      }
      finally
      {
        InitializingControls = false;
      }

      viewEditorBE.add_refresh_ui_handler(RefreshFormDataInvoke);

      Invalidate();

      return true;
    }

    #endregion

    #region Form implementation

    private void DbMysqlViewEditor_Load(object sender, EventArgs e)
    {
      ddlEditor.GrtManager = grtManager;
      ddlEditor.SqlEditorBE = viewEditorBE.get_sql_editor();
      ddlEditor.Initialize();

      ActiveControl = ddlEditor;
    }

    private void DbMysqlViewEditor_FormClosing(object sender, FormClosingEventArgs e)
    {
      sqlEditorTimerHandler(null, null);
    }

    protected void InitFormData()
    {
      dbObjectEditorPages = new DbObjectEditorPages(grtManager, viewEditorBE);
    }

    protected override void RefreshFormData()
    {
      nameTextBox.Text = viewEditorBE.get_name();
      TabText = nameTextBox.Text;
      commentTextBox.Text = viewEditorBE.get_comment();

      String ddl = viewEditorBE.get_query();
      int caretPos = ddlEditor.Caret.Position;
      if (0 < ddl.Length)
      {
        syntaxCheckLabel.Visible = true;
        if (viewEditorBE.has_syntax_error())
        {
          syntaxCheckLabel.Text = "DDL Syntax Error";
          syntaxCheckLabel.Image = Properties.Resources.parse_indicator_red;
          syntaxCheckLabel.Cursor = Cursors.Hand;
        }
        else
        {
          syntaxCheckLabel.Text = "DDL Syntax OK";
          syntaxCheckLabel.Image = Properties.Resources.parse_indicator_green;
          syntaxCheckLabel.Cursor = Cursors.Default;
        }
      }
      else
      {
        ddl = String(viewEditorBE.get_query());
        //ddl += String.Format("CREATE VIEW `{0}`.`{1}` AS\n",
        //  viewEditorBE.get_schema_name(), viewEditorBE.get_name());
        caretPos = ddl.Length;
      }
      ddlEditor.Text = ddl;
      ddlEditor.Caret.Position = caretPos;
      ddlEditor.Selection.Start = caretPos;
      ddlEditor.Selection.End = caretPos;
    }

    private void sqlEditorTimerHandler(Object myObject, EventArgs myEventArgs)
    {
      if (sqlEditorTimer.Enabled)
      {
        sqlEditorTimer.Stop();
        viewEditorBE.set_query(ddlEditor.Text);
      }
    }

    private void nameTextBox_TextChanged(object sender, EventArgs e)
    {
      if (!InitializingControls && !nameTextBox.Text.Equals(viewEditorBE.get_name()))
        viewEditorBE.set_name(nameTextBox.Text);

      TabText = nameTextBox.Text;
    }

    private void ddlEditor_DocumentChange(object sender, ScintillaNet.NativeScintillaEventArgs e)
    {
      if (!InitializingControls && !insideRefreshFormData)
      {
        sqlEditorTimer.Stop();
        sqlEditorTimer.Start();
      }
    }

    private void ddlEditor_TextModified(object sender, ScintillaNet.TextModifiedEventArgs e)
    {
      ddlEditor_DocumentChange(sender, null);
    }

    private void commentTextBox_TextChanged(object sender, EventArgs e)
    {
      if (!InitializingControls && !commentTextBox.Text.Equals(viewEditorBE.get_comment()))
        viewEditorBE.set_comment(commentTextBox.Text);
    }

    private void sqlEditorParseLogCallback(List<String> strlist)
    {
      // if there are more than 2 lines, the 2nd line contains the first error
      if (strlist.Count >= 3)
        MySQLApplicationContext.MainFormStatusBarText = strlist[1];
      else
        MySQLApplicationContext.MainFormStatusBarText = "";

      // name field could change...
      if (viewEditorBE.get_name() != nameTextBox.Text)
        nameTextBox.Text = viewEditorBE.get_name();
    }

    private void undoItem_Click(object sender, EventArgs e)
    {
      ddlEditor.UndoRedo.Undo();
    }

    private void cutItem_Click(object sender, EventArgs e)
    {
      ddlEditor.Clipboard.Cut();
    }

    private void copyItem_Click(object sender, EventArgs e)
    {
      ddlEditor.Clipboard.Copy();
    }

    private void pasteItem_Click(object sender, EventArgs e)
    {
      ddlEditor.Clipboard.Paste();
    }

    private void selectAllItem_Click(object sender, EventArgs e)
    {
      ddlEditor.Selection.SelectAll();
    }

    private void redoItem_Click(object sender, EventArgs e)
    {
      ddlEditor.UndoRedo.Redo();
    }

    private void editorContextMenu_Opening(object sender, CancelEventArgs e)
    {
      // Enable/Disable menu items depending on the edit content.
      undoItem.Enabled = ddlEditor.UndoRedo.CanUndo;
      redoItem.Enabled = ddlEditor.UndoRedo.CanRedo;
      cutItem.Enabled = ddlEditor.Selection.Length > 0;
      copyItem.Enabled = ddlEditor.Selection.Length > 0;
      pasteItem.Enabled = ddlEditor.Clipboard.CanPaste;
      selectAllItem.Enabled = ddlEditor.TextLength > 0;
    }

    private void syntaxCheckLabel_Click(object sender, EventArgs e)
    {
      if (viewEditorBE.has_syntax_error())
        grtManager.get_grt().make_output_visible();
    }

    #endregion

  }
}