
#include "grtui/gui_plugin_base.h"
#include "grtui/grt_wizard_form.h"
#include "grtui/wizard_progress_page.h"
#include "grtui/grtdb_connect_panel.h"

using namespace grtui;
using namespace mforms;


class ExportWizardForm : public WizardForm
{
  Db_rev_eng _db_rev_eng;

public:
  ExportWizardForm(grt::GRTManager *grtm)
    : WizardForm(grtm)
  {
    _db_rev_eng.db_plugin()->grtm(grtm);
  }

  Db_plugin *db_plugin() { return _db_rev_eng.db_plugin(); }
  Sql_import *sql_import() { return _db_rev_eng.sql_import(); }
};

//--------------------------------------------------------------------------------

class ConnectionPage : public WizardPage
{
public:
  ConnectionPage(ExportWizardForm *form)
    : WizardPage(form, "connection"), _contents(true)
  {
    add(&_connect, false, true);
  }
  
  virtual std::string get_title()
  {
    return "Connect to DBMS";
  }
  
  virtual std::string get_subtitle()
  {
    return "Set parameters for connecting to a DBMS.";
  }
  
  
  virtual void load(grt::DictRef values)
  {
    _connect.set_backend(((ExportWizardForm*)_form)->db_plugin()->db_conn());
  }


  virtual void do_validate()
  {

    _form->clear_problem();
  }
  
  virtual bool allow_next()
  {
    return false;
    //return !_filename.get_string_value().empty();
  }

protected:
  DbConnectPanel _connect;
};

//--------------------------------------------------------------------------------

class ExportProgressPage : public WizardProgressPage
{
public:
  ExportProgressPage(WizardForm *form) : WizardProgressPage(form, "progress")
  {
    /*
    add_task(false,
             _("Export"), 
             sigc::mem_fun(this, &ExportProgressPage::do_export),
             _("Exporting objects..."));

    add_task(false,
             _("Verify Results"), 
             sigc::mem_fun(this, &ExportProgressPage::verify_results),
             _("Verifying imported objects..."));
*/
    end_adding_tasks(true,
                     _("Forward Engineer Finished Successfully"));
    
    set_status_text("");
  }
  
  virtual std::string get_title()
  {
    return "Script Export Progress";
  }
  
  virtual std::string get_subtitle()
  {
    return "Progress and results of SQL script reverse engineering.";
  }
};


//--------------------------------------------------------------------------------


class ExportFinishPage : public WizardPage
{
public:
  virtual std::string get_title()
  {
    return "Finished";
  }
  
  virtual std::string get_subtitle()
  {
    return "";
  }
};


//--------------------------------------------------------------------------------


class WbPluginDbExport : public GUIPluginBase, public WizardForm
{
  ExportInputPage *_input_page;
  ExportProgressPage *_progress_page;
  
public:
  WbPluginDbExport(bec::GRTManager *grtm, const grt::BaseListRef &args)
    : GUIPluginBase(grtm, args), WizardForm(grtm)
  {
    _input_page= new ExportInputPage(this);
    _progress_page= new ExportProgressPage(this);
    
    add_page(_input_page);
    add_page(_progress_page);
    
    set_title("Forward Engineer to SQL");
  }


  virtual ~WbPluginDbExport()
  {
    delete _input_page;
    delete _progress_page;
  }


  virtual void execute()
  {
    grt::DictRef values(GUIPluginBase::grtm()->get_grt());
    
    WizardForm::show(values);
  }

  virtual void show()
  {
    execute();
  }

  virtual void hide()
  {
    //XXX clean this mess by renaming the show(DictRef) method from WizardForm
    mforms::Form::show(false);
  }
};






extern "C" 
{
  GUIPluginBase *createWBPluginDbFrwEng(bec::GRTManager *grtm, const grt::BaseListRef &args)
  {
    return new WbPluginDbExport(grtm, args);
  }
};
