using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Text;
using System.Windows.Forms;
using Aga.Controls.Tree;
using MySQL.Grt;
using MySQL.Grt.Db;
using MySQL.Utilities;
using MySQL.GUI.Workbench.Plugins;

namespace MySQL.GUI.Workbench.Plugins.Db
{
  public partial class SchemaObjectFilter : UserControl
  {
    protected string objectType;
    protected DBObjectFilterBE dbObjectFilterBE;
    protected GrtManager grtManager;

    protected SchemaObjectFilter()
    {
      InitializeComponent();
    }

    public SchemaObjectFilter(GrtManager GrtManager)
      : this()
    {
      grtManager = GrtManager;
      dbObjectFilterBE = new DBObjectFilterBE(GrtManager);

      // Hide detailed selection at the beginning
      DetailedSelection = false;
    }

    public DBObjectFilterBE Backend
    {
      get
      {
        return dbObjectFilterBE;
      }
    }

    public string GetObjectType()
    {
      return objectType;
    }

    public void SetObjectType(string ObjectType)
    {
      objectType = ObjectType;

      dbObjectFilterBE.set_object_type_name(ObjectType);

      string full_type_name= dbObjectFilterBE.get_full_type_name();
      activatedCheckBox.Text = String.Format("Object of type {0}",
        full_type_name);
      typeLabel.Text = full_type_name;

      int icon_id = dbObjectFilterBE.icon_id(MySQL.Grt.IconSize.Icon32);
      typePictureBox.Image = GrtIconManager.get_instance().get_icon(icon_id);
    }

    private void SetupModel(TreeViewAdv view,
      GrtStringListModel modelBE,
      Aga.Controls.Tree.NodeControls.NodeTextBox nameNodeControl,
      Aga.Controls.Tree.NodeControls.NodeIcon iconNodeControl)
    {
      SimpleGrtListModel model = new SimpleGrtListModel(view, modelBE, iconNodeControl);
      model.AddColumn(nameNodeControl, (int)GrtStringListModel.Columns.Name, false);
      view.Model = model;
    }

    public void SetupModels(GrtStringListModel selectionModelBE, GrtStringListModel exclusionModelBE)
    {
      SetupModel(selectionTreeView, selectionModelBE, selectedObjNameNode, selectedObjIconNode);
      SetupModel(ignorTreeView, exclusionModelBE, ignoredObjNameNode, ignoredObjIconNode);
      dbObjectFilterBE.filter_model((ignorTreeView.Model as SimpleGrtListModel).GrtList as GrtStringListModel);
      ReloadStoredFilterSetList("");
    }

    public void RefreshModels()
    {
      SimpleGrtListModel ignorModel = ignorTreeView.Model as SimpleGrtListModel;
      SimpleGrtListModel selectionModel = selectionTreeView.Model as SimpleGrtListModel;
      ignorModel.RefreshModel();
      selectionModel.RefreshModel();

      GrtStringListModel selectionModelBE = selectionModel.GrtList as GrtStringListModel;
      selectionCountLabel.Text = String.Format("{0} / {1}",
        selectionModelBE.active_items_count(),
        selectionModelBE.total_items_count());

      activatedCheckBox.Checked = (selectionModelBE.active_items_count() != 0);
    }

    private void RemoveExclusionMasks(bool selectedOnly)
    {
      cbStoredFilterSet.SelectedIndex = -1;
      GrtStringListModel srcModelBE = (ignorTreeView.Model as SimpleGrtListModel).GrtList as GrtStringListModel;
      GrtStringListModel destModelBE = (selectionTreeView.Model as SimpleGrtListModel).GrtList as GrtStringListModel;
      IEnumerable<TreeNodeAdv> nodes = selectedOnly ? ignorTreeView.SelectedNodes : ignorTreeView.AllNodes;
      int nodesCount = selectedOnly ? ignorTreeView.SelectedNodes.Count : srcModelBE.count();
      List<int> itemIndexes = new List<int>(nodesCount);
      foreach (TreeNodeAdv node in nodes)
        itemIndexes.Add(node.Index);
      srcModelBE.remove_items(itemIndexes);
      destModelBE.invalidate();
      RefreshModels();
    }

    private void AddToIgnoreMasks(bool selectedOnly)
    {
      cbStoredFilterSet.SelectedIndex = -1;
      GrtStringListModel srcModelBE = (selectionTreeView.Model as SimpleGrtListModel).GrtList as GrtStringListModel;
      GrtStringListModel destModelBE = (ignorTreeView.Model as SimpleGrtListModel).GrtList as GrtStringListModel;
      IEnumerable<TreeNodeAdv> nodes = selectedOnly ? selectionTreeView.SelectedNodes : selectionTreeView.AllNodes;
      int nodesCount = selectedOnly ? selectionTreeView.SelectedNodes.Count : srcModelBE.count();
      List<int> itemIndexes = new List<int>(nodesCount);
      foreach (TreeNodeAdv node in nodes)
        itemIndexes.Add(node.Index);
      srcModelBE.copy_items_to_val_masks_list(itemIndexes);
      srcModelBE.invalidate();
      RefreshModels();
    }

    #region Properties

    public bool TypeActivated
    {
      get { return activatedCheckBox.Checked; }
      set
      {
        activatedCheckBox.Checked = value;

        foreach (Control c in mainGroupBox.Controls)
        {
          if (c != activatedCheckBox)
            c.Enabled = activatedCheckBox.Checked;
        }
      }
    }

    public bool DetailedSelection
    {
      get { return detailsPanel.Visible; }
      set
      {
        if (detailsPanel.Visible != value)
        {
          detailsPanel.Visible = value;
          showDetailsButton.Visible = !value;

          Height = Height + (detailsPanel.Height * (Convert.ToInt16(value) * 2 - 1));
        }
      }
    }

    #endregion

    #region UI Event Handling

    private void showDetailsButton_Click(object sender, EventArgs e)
    {
      DetailedSelection = true;
    }

    private void hideDetailsButton_Click(object sender, EventArgs e)
    {
      DetailedSelection = false;
    }

    private void addToIgnoreListButton_Click(object sender, EventArgs e)
    {
      AddToIgnoreMasks(true);
    }

    private void removeFromIgnoreListButton_Click(object sender, EventArgs e)
    {
      RemoveExclusionMasks(true);
    }

    private void ignoreAllButton_Click(object sender, EventArgs e)
    {
      AddToIgnoreMasks(false);
    }

    private void removeAllFromIgnoreListButton_Click(object sender, EventArgs e)
    {
      RemoveExclusionMasks(false);
    }

    private void selectionTreeView_DoubleClick(object sender, EventArgs e)
    {
      AddToIgnoreMasks(true);
    }

    private void ignorTreeView_DoubleClick(object sender, EventArgs e)
    {
      RemoveExclusionMasks(true);
    }

    private void addIgnoreWildcardButton_Click(object sender, EventArgs e)
    {
      string itemsValMask;
      if (StringInputForm.ShowModal("Add Ignore Pattern",
        "Please enter the patter to ignore. You can use * and ? to match the objects you would like to ignore." + System.Console.Out.NewLine +
          "Please note that the string is matched case sensitive.",
        "Ignore Mask:",
        out itemsValMask) == DialogResult.OK)
      {
        GrtStringListModel srcModelBE = (selectionTreeView.Model as SimpleGrtListModel).GrtList as GrtStringListModel;
        GrtStringListModel destModelBE = (ignorTreeView.Model as SimpleGrtListModel).GrtList as GrtStringListModel;
        destModelBE.add_item(itemsValMask, -1);
        srcModelBE.invalidate();
        RefreshModels();
      }
    }

    private void searchTextBox_Leave(object sender, EventArgs e)
    {
      SimpleGrtListModel model = selectionTreeView.Model as SimpleGrtListModel;
      GrtStringListModel modelBE = model.GrtList as GrtStringListModel;
    //  modelBE.items_val_mask(searchTextBox.Text);
      RefreshModels();
    }

    private void searchTextBox_KeyDown(object sender, KeyEventArgs e)
    {
      if (e.KeyCode == Keys.Return)
      {
        SimpleGrtListModel model = selectionTreeView.Model as SimpleGrtListModel;
        GrtStringListModel modelBE = model.GrtList as GrtStringListModel;
        modelBE.items_val_mask(searchTextBox.Text);
        RefreshModels();
      }
    }
    #endregion

    public void AddStoredFilterSet(String storedFilterSetName)
    {
      if (0 == storedFilterSetName.Length)
        return;

      dbObjectFilterBE.add_stored_filter_set(storedFilterSetName);

      GrtStringListModel srcModelBE = (selectionTreeView.Model as SimpleGrtListModel).GrtList as GrtStringListModel;
      srcModelBE.invalidate();
      RefreshModels();

      ReloadStoredFilterSetList(storedFilterSetName);
    }

    private void btnAddStoredFilterSet_Click(object sender, EventArgs e)
    {
      string storedFilterSetName;
      if (StringInputForm.ShowModal("Adding new stored filter set", "", "Filter Set Name:", out storedFilterSetName) == DialogResult.OK)
        AddStoredFilterSet(storedFilterSetName);
    }

    private void btnRemoveStoredFilterSet_Click(object sender, EventArgs e)
    {
      if (cbStoredFilterSet.Text.Length != 0)
        dbObjectFilterBE.remove_stored_filter_set(cbStoredFilterSet.SelectedIndex);
      ReloadStoredFilterSetList("");
    }

    private void cbStoredFilterSet_SelectionChangeCommitted(object sender, EventArgs e)
    {
      dbObjectFilterBE.load_stored_filter_set(cbStoredFilterSet.SelectedIndex);

      GrtStringListModel srcModelBE = (selectionTreeView.Model as SimpleGrtListModel).GrtList as GrtStringListModel;
      srcModelBE.invalidate();
      RefreshModels();
    }

    private void ReloadStoredFilterSetList(String selectedItemName)
    {
      cbStoredFilterSet.Items.Clear();
      List<String> names= new List<String>();
      dbObjectFilterBE.load_stored_filter_set_list(ref names);
      foreach (String name in names)
      {
        cbStoredFilterSet.Items.Add(name);
        if (0 == selectedItemName.CompareTo(name))
          cbStoredFilterSet.SelectedIndex = cbStoredFilterSet.Items.Count - 1;
      }
    }

    public ComboBox StoredFilterSetControl
    {
      get { return cbStoredFilterSet; }
    }

    public int StoredFilterSetIndex
    {
      set
      {
        cbStoredFilterSet.SelectedIndex = value;

        GrtStringListModel srcModelBE = (selectionTreeView.Model as SimpleGrtListModel).GrtList as GrtStringListModel;
        srcModelBE.invalidate();
        RefreshModels();
      }
    }

    public void ResetModels()
    {
      dbObjectFilterBE.load_stored_filter_set(-1);
      GrtStringListModel srcModelBE = (selectionTreeView.Model as SimpleGrtListModel).GrtList as GrtStringListModel;
      srcModelBE.invalidate();
    }

  }
}
