using System;
using System.Collections;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Text;
using System.Windows.Forms;
using MySQL.Grt;
using MySQL.Grt.Db;
using MySQL.Utilities;
using MySQL.GUI.Workbench.Plugins;
using MySQL.GUI.Workbench.Plugins.DbMySQLScriptSync;
using Aga.Controls.Tree;
using Aga.Controls.Tree.NodeControls;

namespace MySQL.GUI.Workbench.Plugins.Sync
{
  public partial class ValidationControl : UserControl
  {
    public delegate void RunValidationDelegate();
    
    private GrtMessageList messageListBE;
    private SimpleGrtListModel messageListModel;
    private int currentStep;
    private RunValidationDelegate validationDelegate;

    public ValidationControl(GrtMessageList message_list, RunValidationDelegate deleg)
    {
      InitializeComponent();

      validationDelegate = deleg;
      messageListBE = message_list;
      messageListModel = new SimpleGrtListModel(messageTreeView, messageListBE, nodeIcon);
      messageListModel.AddColumn(nodeMessage, (int)GrtMessageList.Columns.Message, false);
      messageListModel.AddColumn(nodeTime, (int)GrtMessageList.Columns.Time, false);
      messageTreeView.Model = messageListModel;
    }

    public void RefreshOutput()
    {
      messageListModel.RefreshModel();
    }

    public void OnStartValidation()
    {
      currentStep = 0;
      LblCurrentStep.Text = "General catalog validation";
      LblValidationResult.Text = "Executing...";
      taskLblGeneralValidation.TaskState = MySQLTaskStatusLabelState.TaskOpen;
      taskLblMySQLValidation.TaskState = MySQLTaskStatusLabelState.TaskOpen;
      messageListBE.clear();
      RefreshOutput();
    }

    public void OnValidationStepCompleted(int res)
    {
      switch (currentStep)
      {
        case 0:
          LblCurrentStep.Text = "";
          LblValidationResult.Text = "Done";
          taskLblGeneralValidation.TaskState = 
            res == 1 ? MySQLTaskStatusLabelState.TaskCompleted : MySQLTaskStatusLabelState.TaskError;
          break;
        case 1:
          LblCurrentStep.Text = "MySQL-specific catalog validation";
          taskLblMySQLValidation.TaskState =
            res == 0 ? MySQLTaskStatusLabelState.TaskCompleted : MySQLTaskStatusLabelState.TaskError;
          break;
      }
      currentStep++;
    }

    private void runValidationBtn_Click(object sender, EventArgs e)
    {
      validationDelegate();
    }
  }
}