using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.Runtime.InteropServices;
using System.IO;
using MySQL.Grt;
using MySQL.Utilities;
using MySQL.GUI.Workbench.Plugins;

namespace MySQL.GUI.Workbench.Plugins.Db
{
  public partial class WBPluginSqlImportPages : WizardPluginPages
  {
    private Sql_import sqlImport;
    private bool showAdvancedImportLogPage;

    public WBPluginSqlImportPages(WizardPlugin wizard)
      : base(wizard)
    {
      InitializeComponent();

      sqlImport = new Sql_import();
      sqlImport.grtm(wizard.GrtManager);

      showAdvancedImportLogPage = false;
      caseSensitiveNamesCB.Checked = sqlImport.get_bool_option("SqlIdentifiersCS");

      AddPage(optionsPanel,
        null,
        null,
        ProcessOptionsPage,
        null,
        null,
        "SQL Import Options",
        "Set SQL import options");

      AddPage(objectsImportLogPanel,
        InitializeObjectsImportLogPage,
        null,
        null,
        ToggleAdvancedImportLogPage,
        null,
        "Import (Progress/Results)",
        "See the progress/results of import.");

      AddPage(postOptionsPanel,
        null,
        null,
        ProcessPostOptionsPage,
        null,
        null,
        "Post-procedure options",
        "Set post-procedure options.");
    }

    #region OptionsPage

    private string ProcessOptionsPage()
    {
      string errMsg = "";
      string fpath = inputPathTextBox.Text;
      if (0 == fpath.Length || !(new FileInfo(fpath)).Exists)
        errMsg = "Invalid path to SQL script file";
      else
        sqlImport.sql_script(inputPathTextBox.Text);
      return errMsg;
    }

    private void inputBrowseButton_Click(object sender, EventArgs e)
    {
      OpenFileDialog fd = new OpenFileDialog();
      fd.RestoreDirectory = true;
      fd.Filter = "SQL script (*.sql)|*.sql";
      if (DialogResult.OK == fd.ShowDialog())
        inputPathTextBox.Text = fd.FileName;
    }

    #endregion OptionsPage

    #region ObjectsImportLogPage

    private void PrepareTask(PageTask pageTask)
    {
      sqlImport.task_msg_cb(pageTask.ProcessTaskMsg);
      sqlImport.task_progress_cb(pageTask.ProcessTaskProgress);
      sqlImport.task_fail_cb(pageTask.ProcessTaskFail);
      sqlImport.task_finish_cb(pageTask.ProcessTaskFinish);
    }

    private String OnImport()
    {
      sqlImport.exec_task();
      return "";
    }

    private void AfterImport(PageTask pageTask)
    {
      if ((pageTask.ErrMsgExists || pageTask.WarnMsgExists) && !showAdvancedImportLogPage)
        ToggleAdvancedImportLogPage();
    }

    private String OnCheckImportResults()
    {
      //! TODO:
      /*
      optional validation (warnings for mixing letter case when referencing same objects)
      with regard to sqlImport.get_bool_option("SqlIdentifiersCS")
      */
      return "";
    }

    private string InitializeObjectsImportLogPage()
    {
      objectsImportLogTextBox.Text = "";
      lImportObjName.Text = "";
      pbImport.Minimum = 0;
      pbImport.Maximum = 100;

      pageTasks.Setup(lImportResult, null);

      sqlImport.set_option("case_sensitive_names", caseSensitiveNamesCB.Checked);
      sqlImport.set_option("processing_create_statements", procCreateStmtCB.Checked);
      sqlImport.set_option("processing_alter_statements", procAlterStmtCB.Checked);
      sqlImport.set_option("processing_drop_statements", procDropStmtCB.Checked);

      pageTasks.Add(
        true,
        true,
        "Importing SQL script...",
        taskLblExecImportProc,
        lImportStage,
        lImportResult,
        objectsImportLogPanel,
        objectsImportLogTextBox,
        PrepareTask,
        OnImport,
        AfterImport,
        AfterImport,
        pbImport,
        lImportObjName);

      pageTasks.Add(
        false,
        true,
        "Checking results...",
        taskLblCheckImportResults,
        lImportStage,
        lImportResult,
        objectsImportLogPanel,
        objectsImportLogTextBox,
        PrepareTask,
        OnCheckImportResults,
        null,
        null,
        null,
        null);

      return pageTasks.Exec();
    }

    private string ToggleAdvancedImportLogPage()
    {
      showAdvancedImportLogPage = !showAdvancedImportLogPage;
      importGroupBox.Visible = showAdvancedImportLogPage;
      return "";
    }

    #endregion ObjectsImportLogPage

    #region PostOptionsPage

    private string ProcessPostOptionsPage()
    {
      if (cbAutoplacement.Checked)
        sqlImport.autoplace_on_diagram();
      return "";
    }

    #endregion PostOptionsPage
  }
}