using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.Runtime.InteropServices;
using MySQL.Grt;
using MySQL.GUI.Workbench.Plugins;
using MySQL.GUI.Workbench.Plugins.WbModelReporting.Properties;

namespace MySQL.GUI.Workbench.Plugins.WbModelReporting
{
  using Pages = LinkedList<PageInfo>;
  using System.Collections;

  //class CatalogObjectGrtStringList : GrtStringListModel
  //{ 
    
  //}

  public partial class ReportingWizardPages : WizardPluginPages
  {
    private WbModelReporting pluginBE;

    public ReportingWizardPages(WizardPlugin wizard)
      : base(wizard)
    {
      pluginBE = new WbModelReporting(wizard.GrtManager);

      InitializeComponent();

      AddPage(optionsPanel,
        InitOptionsPage,
        CleanupOptionsPage,
        ProcessOptionsPage,
        null,
        null,
        "DBDoc - Model Reporting",
        "DBDoc - Model Reporting Options");
    }

    #region Options Page Routines

    private string InitOptionsPage()
    {
      // fill template combo box
      List<string> templates = pluginBE.get_available_reporting_templates();

      templatesListBox.Items.Clear();
      for (int i = 0; i < templates.Count; i++)
        templatesListBox.Items.Add(templates[i]);

      if (templatesListBox.Items.Count > 0)
        templatesListBox.SelectedIndex = 0;
      else
        templatesListBox.SelectedIndex = -1;

      outputDirTextBox.Text = System.IO.Path.Combine(
        Environment.GetFolderPath(Environment.SpecialFolder.MyDocuments), "MySQL Workbench Report");

      return "";
    }

    private void templatesListBox_SelectedIndexChanged(object sender, EventArgs e)
    {
      styleComboBox.Items.Clear();

      // fill style combo box
      if (templatesListBox.SelectedItem != null)
      {
        pluginBE.set_active_template(templatesListBox.SelectedItem.ToString());

        styleDescLabel.Text = pluginBE.get_reporting_template_description();

        List<string> styleNames = pluginBE.get_available_reporting_template_style_names();

        if (styleNames.Count > 0)
        {
          styleComboBox.Enabled = true;

          for (int i = 0; i < styleNames.Count; i++)
            styleComboBox.Items.Add(styleNames[i]);

          if (styleComboBox.Items.Count > 0)
            styleComboBox.SelectedIndex = 0;
        }
        else
        {
          previewPictureBox.Image = Resources.note_128x128;
          styleComboBox.Enabled = false;
        }
      }
      else
      {
        previewPictureBox.Image = Resources.note_128x128;
        styleComboBox.Enabled = false;
      }
    }

    private void styleComboBox_SelectedIndexChanged(object sender, EventArgs e)
    {
      if (styleComboBox.SelectedItem != null)
      {
        pluginBE.set_active_template_style(styleComboBox.SelectedItem.ToString());

        // display image
        string previewImageFileName = pluginBE.get_reporting_template_style_preview_image_filename();
        if (!previewImageFileName.Equals(""))
        {
          if (System.IO.File.Exists(previewImageFileName))
            previewPictureBox.Image = new Bitmap(previewImageFileName);
          else
            previewPictureBox.Image = Resources.note_128x128;
        }

        styleDescLabel.Text = pluginBE.get_reporting_template_description()
          + Environment.NewLine + Environment.NewLine
          + pluginBE.get_reporting_template_style_description();
      }
      else
        previewPictureBox.Image = Resources.note_128x128;
    }

    private string CleanupOptionsPage()
    {
      return "";
    }

    private void outputBrowseButton_Click(object sender, EventArgs e)
    {
      FolderBrowserDialog dialog = new FolderBrowserDialog();
      dialog.ShowNewFolderButton = true;

      if (dialog.ShowDialog() == DialogResult.OK)
      {
        outputDirTextBox.Text = dialog.SelectedPath;
      }
    }

    private string ProcessOptionsPage()
    {
      // set options
      pluginBE.set_option("template_name", templatesListBox.SelectedItem.ToString());
      pluginBE.set_option("template_style_name", styleComboBox.Text);
      pluginBE.set_option("title", titleTextBox.Text);
      pluginBE.set_option("output_dirname", outputDirTextBox.Text);
      pluginBE.set_option("columns_show", showColumnsCheckBox.Checked ? "1" : "0");
      pluginBE.set_option("indices_show", showIndicesCheckBox.Checked ? "1" : "0");
      pluginBE.set_option("fks_show", showFksCheckBox.Checked ? "1" : "0");
      pluginBE.set_option("fks_show_referred_fks", showReferredFksCheckBox.Checked ? "1" : "0");

      // start background task
      pluginBE.start_sync();

      return "";
    }

    #endregion

  }
}