<html>
<?

function locateCodeFiles($path, $ignoreList)
{
    $dircontents= scandir($path);
    $files= array();
    foreach ($dircontents as $file)
    {
        if ($file == "." or $file == "..")
          continue;

        $filep= $path."/".$file;
        if (in_array($file, $ignoreList) or in_array($filep, $ignoreList))
          continue;

        if (is_dir($filep))
          $files= array_merge($files, locateCodeFiles($filep, $ignoreList));
        elseif (in_array(end(split("\.", $file)), array("c", "cc", "cpp")))
          $files[]= $filep;
    }
    return $files;
}


function getCovStats($path)
{
    $lines= file($path);

    $notExecuted= 0;
    $executed= 0;
    $untested= array();

    foreach ($lines as $line)
    {
        $line= trim($line);
        if ($line == "")
          continue;
        $parts= split(":", $line);
        $execCount= trim($parts[0]);
        $lineNum= trim($parts[1]);

        if ($execCount == "-")
          continue;
        if ($execCount[0] == "#")
        {
            $untested[]= $lineNum;
            $notExecuted+= 1;
        }
        elseif (ctype_digit($execCount[0]))
          $executed+= 1;
        else
          assert(0);
    }
    return array("covered"=>$executed, "notCovered"=>$notExecuted, "untestedCode"=>$untested);
}


function checkTestedFiles($basedir, $pathList)
{
    $tested= array();
    $notTested= array();
    foreach ($pathList as $f)
        $notTested[]= $f;
    foreach ($pathList as $f)
    {
        $file= end(split("/", $f));
        if (file_exists($basedir."/".$file.".gcov"))
        {
            $stats= getCovStats($basedir."/".$file.".gcov");

            unset($notTested[array_search($f, $notTested)]);
            $tested[$f]= $stats;
        }
    }
    return array("tested"=>$tested, "notTested"=>$notTested);
}



function processProject($basedir, $ignoreFiles)
{
    $files= locateCodeFiles($basedir, $ignoreFiles);
    $testData= checkTestedFiles($basedir."/test", $files);

    $libraryStats= array();
    $fileStats= array();

    foreach ($testData["notTested"] as $f)
    {
        $parts= split("/", substr($f, strlen($basedir)));

        $library= $parts[1];

        if (array_key_exists($library, $libraryStats))
        {
            $lstats= $libraryStats[$library];
            $lstats["fileCount"]+= 1;
            $lstats["files"][]= $f;
            $libraryStats[$library]= $lstats;
        }
        else
        {
            $lstats= array();
            $lstats["fileCount"]= 1;
            $lstats["files"]= array($f);
            $lstats["testedFileCount"]= 0;
            $lstats["covered"]= 0;
            $lstats["notCovered"]= 0;
            $lstats["tested"]= 0;
            $libraryStats[$library]= $lstats;
        }
    }

    $covered= 0;
    $notCovered= 0;
    $total= 0;
    foreach ($testData["tested"] as $f => $stats)
    {        
        $fileStats[$f]= array("covered"=> $stats["covered"],
                              "notCovered"=> $stats["notCovered"],
                              "tested"=> $stats["covered"]+$stats["notCovered"],
                              "untestedCode"=>$stats["untestedCode"]);
        $covered+= $fileStats[$f]["covered"];
        $notCovered+= $fileStats[$f]["notCovered"];
        $total+= $fileStats[$f]["tested"];

        $parts= split("/", substr($f, strlen($basedir)));

        $library= $parts[1];

        if (array_key_exists($library, $libraryStats))
        {
            $lstats= $libraryStats[$library];
            $lstats["fileCount"]+= 1;
            $lstats["files"][]= $f;
            $lstats["testedFileCount"]+= 1;
            $lstats["covered"]+= $fileStats[$f]["covered"];
            $lstats["notCovered"]+= $fileStats[$f]["notCovered"];
            $lstats["tested"]+= $fileStats[$f]["tested"];
            $libraryStats[$library]= $lstats;
        }
        else
        {
            $lstats= array();
            $lstats["fileCount"]= 1;
            $lstats["files"]= array($f);
            $lstats["testedFileCount"]= 1;
            $lstats["covered"]= $fileStats[$f]["covered"];
            $lstats["notCovered"]= $fileStats[$f]["notCovered"];
            $lstats["tested"]= $fileStats[$f]["tested"];
            $libraryStats[$library]= $lstats;
        }
    }

    $globalStats= array("files"=>count($files),
                        "testedFiles"=>count($testData["tested"]),
                        "coveredLines"=>$covered,
                        "notCoveredLines"=>$notCovered,
                        "testedTotalLines"=>$total);
    
    return array("global"=>$globalStats, "perLibrary"=>$libraryStats, "perFile"=>$fileStats);
}



function showProjectStats($projectInfo)
{
    $basedir= $projectInfo["basedir"];
    $ignoreFiles= $projectInfo["ignoreFiles"];

    $result= processProject($basedir, $ignoreFiles);
    
    $globalStats= $result["global"];
    $fileStats= $result["perFile"];
    
    echo "<title>{$projectInfo['name']} Test Coverage Report</title><body>\n";
    echo "<h1>Test Coverage Report For {$projectInfo['name']}</h1>\n";
    
    echo "<h2>Summary</h2>\n";
    printf("<p>Total Tested Source Files: {$globalStats['testedFiles']} of {$globalStats['files']} files (%.2f%%)\n", $globalStats['testedFiles']*100.0/$globalStats['files']);
    printf("<p>Coverage in Tested Files: {$globalStats['coveredLines']} of {$globalStats['testedTotalLines']} lines (%.2f%%)\n", $globalStats['coveredLines']*100.0/$globalStats['testedTotalLines']);
    
    echo "<p><small><em>Coverage in Tested Files means the number of lines that were actually executed in tests,\n";
    echo "in relation to the total number of lines in <em>files that were tested at all</em>.\n";
    echo "Files that were not handled in tests, are not counted.</em></small>\n";
    
    
    echo "<h2>Coverage by Library</h2>";
    foreach ($result["perLibrary"] as $library => $lstats)
    {    
        if ($lstats["tested"] > 0)
          $covered_pct= $lstats["covered"]*100.0/$lstats["tested"];
        else
          $covered_pct= 0;
        $filecov_pct= $lstats["testedFileCount"]*100.0/$lstats["fileCount"];
        
        echo "<h2><a href='#$library'>$library</a></h2>\n";
        
        printf("<p>Number of Files Tested: {$lstats['testedFileCount']} of {$lstats['fileCount']} files    <b>(%.2f%%)</b>\n", $filecov_pct);
        printf("<p>Coverage in Tested Files: {$lstats['covered']} of {$lstats['tested']} lines   (%.2f%%)\n", $covered_pct);
    }
    
    
    echo "<h2>Details by File</h2>\n";
    foreach($result["perLibrary"] as $library => $lstats)
    {
        echo "<a name='$library'><h3>$library</h3></a>\n";
        
        echo "<table border=0 width=100%>\n";
        foreach($lstats["files"] as $f)
        {
            echo "<tr>";
            $flag= 0;
            if (array_key_exists($f, $fileStats))
            {
                $fs= $fileStats[$f];
                if ($fs["tested"] > 0)
                {
                    echo "<td><a href=covcheck.php?action=viewfile&file=".urlencode($f).">".end(split("/", $f))."</a></td>";
                    echo "<td width=100%%>{$fs['covered']} of {$fs['tested']} lines</td>";
                    $flag= 1;
                }
            }
            if (!$flag)
            {
                echo "<td>".end(split("/", $f))."</td>";
                echo "<td width=100%%>Not Tested</td>";
            }
            echo "</tr>\n";
        }
        echo "</table>\n";
    }
}



function showFile($projectInfo, $path)
{
    $basedir= $projectInfo["basedir"];
    $ignoreFiles= $projectInfo["ignoreFiles"];

    $files= locateCodeFiles($basedir, $ignoreFiles);

    if (!in_array($path, $files))
    {
        echo "Invalid file.";
        exit(1);
    }

    echo "<p>".$path."<br>\n";
    echo "<li>Lines in black were executed.";
    echo "<li>Lines in <font color=red>red</font> were not executed.";
    echo "<li>Lines in <font color='#a0a0a0'>gray</font> lines are not code.";
    echo "<br><br>";
    
    echo "<a href=covcheck.php>Return to Main Page</a><br><br>";

    $filename= end(split("/", $path));

    $covfile= $basedir."/test/".$filename.".gcov";

    $code= file($covfile);

    $left= array();
    $right= array();
    $color= array();

    foreach($code as $line)
    {
        $line= trim($line);
        if ($line == "")
          continue;
        $parts= split(":", $line, 3);
        $execCount= trim($parts[0]);
        $lineNum= trim($parts[1]);
        $stuff= rtrim($parts[2]);
        
        if ($lineNum == 0)
          continue;

        if ($execCount == "-")
        {
            $color[]= "#a0a0a0";
            $left[]= $lineNum;
            $right[]= htmlentities($stuff);
        }
        elseif ($execCount[0] == "#")
        {
            $color[]= "red";
            $left[]= $lineNum;
            $right[]= htmlentities($stuff);
        }
        elseif (ctype_digit($execCount[0]))
        {
            $color[]= "";
            $left[]= $lineNum;
            $right[]= htmlentities($stuff);
        }
        else
          assert(0);
    }
    
    echo "<table border=0 width=100%>\n";
    /*
    for ($i= 0; $i < count($left); $i++)
    {
        echo "<tr><td align=right>";
        echo "<pre>".$left[$i]."</pre>";
        echo "</td><td>";
        echo "<pre>".$right[$i]."</pre>";
        echo "</td></tr>";
    }*/
    print "<tr><td align=right><pre>";
    for ($i= 0; $i < count($left); $i++)
    {
        if ($color[$i] != "")
          echo "<font color={$color[$i]}>{$left[$i]}</font>\n";
        else
          echo $left[$i]."\n";
    }
    echo "</ptr></td><td width=10></td><td><pre>";
    for ($i= 0; $i < count($left); $i++)
    {
        if ($color[$i] != "")
          echo "<font color={$color[$i]}>{$right[$i]}</font>\n";
        else
          echo $right[$i]."\n";
    }
    echo "</ptr></td></tr></table>\n";
    
    echo "<a href=covcheck.php>Return to Main Page</a><br><br>";
}


$projectInfo=array("name"=>"GUI Common",
                   "basedir"=> "/home/kojima/svn/mysql-gui-common/library",
                   "ignoreFiles"=> array(".svn", ".deps", ".libs", "test", "ftgl", "tests", "newt"));

if ($_GET['action'] == "viewfile")
  showFile($projectInfo, $_GET['file']);
else
  showProjectStats($projectInfo);  

?>
</body></html>
      
