#ifndef __SYNTETIC_WB_DOCUMENT_H
#define __SYNTETIC_WB_DOCUMENT_H


#include "grt_manager.h"
#include "grts/structs.h"
#include "grts/structs.db.mgmt.h"
#include "grts/structs.db.mysql.h"
#include "grts/structs.workbench.h"
#include "grts/structs.workbench.physical.h"

namespace tut
{


struct SynteticMySQLModel
{
  db_mgmt_RdbmsRef rdbms;
  workbench_physical_ModelRef model;
  workbench_physical_ViewRef physicalView;

  db_mysql_CatalogRef catalog;
  db_mysql_SchemaRef schema;
  db_mysql_TableRef table;
  db_mysql_ColumnRef column;
  db_mysql_ColumnRef column2;
  db_mysql_ColumnRef columnText;
  db_mysql_ColumnRef columnDate;
  db_mysql_ColumnRef columnDouble;
  db_mysql_IndexRef primaryKey;
  db_mysql_IndexColumnRef indexColumn;
  db_mysql_ForeignKeyRef foreignKey;
  db_mysql_TriggerRef trigger;

  db_mysql_ViewRef view;
  db_mysql_RoutineRef routine;
  db_mysql_RoutineGroupRef routineGroup;
  db_UserRef user;
  db_RoleRef role;

  db_RolePrivilegeRef tablePrivilege;
  db_RolePrivilegeRef viewPrivilege;
  db_RolePrivilegeRef routinePrivilege;

  workbench_physical_TableFigureRef tableFigure;
  workbench_physical_ViewFigureRef viewFigure;
  workbench_physical_RoutineGroupFigureRef routineGroupFigure;

  SynteticMySQLModel(grt::GRT* grt)
    : model(grt),
    catalog(grt),
    physicalView(grt),
    schema(grt),
    table(grt),
    column(grt),
    column2(grt),
    columnText(grt),
    columnDate(grt),
    columnDouble(grt),
    primaryKey(grt),
    indexColumn(grt),
    foreignKey(grt),
    trigger(grt),
    view(grt),
    routine(grt),
    routineGroup(grt),
    user(grt),
    role(grt),
    tablePrivilege(grt),
    viewPrivilege(grt),
    routinePrivilege(grt),
    tableFigure(grt),
    viewFigure(grt),
    routineGroupFigure(grt)
  {
    // Set up a basic root structure.
    workbench_WorkbenchRef wb(grt);
    workbench_DocumentRef doc(grt);
    doc->owner(wb);
    wb->doc(doc);
    grt->set("/wb", wb);

    app_OptionsRef options(grt);
    options->owner(wb);
    wb->options(options);

    // Fill model with data.
    model->catalog(catalog);             
    catalog->owner(model);
    model->views().insert(physicalView); 
    physicalView->owner(model);

    grt::ListRef<db_DatatypeGroup> grouplist= 
      grt::ListRef<db_DatatypeGroup>::cast_from(grt->unserialize("../../bin/debug/data/db_datatype_groups.xml"));

    // unfortunately we have to do these lines to make sure that references (links) will be resolved during import
    grt::ValueRef root= grt->root();
    assure(root.type() == grt::DictType);
    grt::DictRef::cast_from(root).set("grouplist", grouplist);

    rdbms= db_mgmt_RdbmsRef::cast_from(grt->unserialize("../../bin/debug/modules/data/mysql_rdbms_info.xml"));
    ensure("db_mgmt_RdbmsRef initialization", rdbms.is_valid());

    model->rdbms(rdbms);

    catalog->name("test_catalog");
    catalog->schemata().insert(schema); schema->owner(catalog);
    catalog->users().insert(user);      user->owner(catalog);
    catalog->roles().insert(role);      role->owner(catalog);
    catalog->simpleDatatypes(rdbms->simpleDatatypes());
    catalog->characterSets(rdbms->characterSets());

    schema->name("test_schema");
    schema->tables().insert(table);     table->owner(schema);
    schema->views().insert(view);       view->owner(schema);
    schema->routines().insert(routine); routine->owner(schema);
    schema->routineGroups().insert(routineGroup);  routineGroup->owner(schema);

    table->name("t1");
    table->columns().insert(column);    column->owner(table);
    table->columns().insert(column2);    column2->owner(table);
    table->columns().insert(columnText);    columnText->owner(table);
    table->columns().insert(columnDate);    columnDate->owner(table);
    table->columns().insert(columnDouble);  columnDouble->owner(table);
    table->indices().insert(primaryKey); primaryKey->owner(table);
    table->foreignKeys().insert(foreignKey);  foreignKey->owner(table);
    table->primaryKey(primaryKey);
    table->triggers().insert(trigger);   trigger->owner(table);

    table->tableEngine("InnoDB");

    view->name("v1");
    view->sqlDefinition("select id from t1");

    routine->name("r1");
#ifndef NL
#define NL "\n"
#endif
    routine->sqlDefinition("DELIMITER //"NL
"CREATE FUNCTION get_count(less_than INT, greather_than INT) RETURNS INT"NL
"    DETERMINISTIC"NL
"    READS SQL DATA"NL
"BEGIN"NL
"       #OK, here some comment"NL
"  DECLARE res INTEGER; #FEES PAID TO RENT THE VIDEOS INITIALLY"NL
"  SELECT count(*) INTO res"NL
"    FROM t1"NL
"    WHERE id > less_tnan AND id < greather_than;"NL
"  RETURN res;"NL
"END //"NL
"DELIMITER ;");

    routineGroup->name("group1");
    routineGroup->routines().insert(routine);

    column->name("id");
    column->simpleType(model->rdbms()->simpleDatatypes().get(0));
    column->defaultValueIsNull(0);

    column2->name("parent_id");
    column2->simpleType(model->rdbms()->simpleDatatypes().get(0));
    column2->defaultValueIsNull(0);

    columnText->name("desc");
    columnText->simpleType(model->rdbms()->simpleDatatypes().get(8));
    columnText->defaultValueIsNull(0);

    columnDate->name("datetimefield");
    columnDate->simpleType(model->rdbms()->simpleDatatypes().get(20));
    columnDate->defaultValueIsNull(0);

    columnDouble->name("doublefield");
    columnDouble->simpleType(model->rdbms()->simpleDatatypes().get(6));
    columnDouble->defaultValueIsNull(0);

    primaryKey->name("PK");
    primaryKey->columns().insert(indexColumn);
    primaryKey->isPrimary(1);

    indexColumn->owner(primaryKey);
    indexColumn->referencedColumn(column);

    foreignKey->name("FK");
    foreignKey->columns().insert(column2);
    foreignKey->referencedColumns().insert(column);
    foreignKey->referencedTable(table);

    trigger->name("trig");
    trigger->sqlDefinition("CREATE "
"TRIGGER trig BEFORE INSERT "
"ON t1 FOR EACH ROW SET id = new.id;");

    user->name("User1");
    user->roles().insert(role);

    role->name("Role1");
    role->privileges().insert(tablePrivilege);  tablePrivilege->owner(role);
    role->privileges().insert(viewPrivilege);   viewPrivilege->owner(role);
    role->privileges().insert(routinePrivilege);routinePrivilege->owner(role);

    tablePrivilege->databaseObject(table);
    tablePrivilege->privileges().insert("ALL");
    viewPrivilege->databaseObject(view);
    viewPrivilege->privileges().insert("ALL");
    routinePrivilege->databaseObject(routine);
    routinePrivilege->privileges().insert("ALL");

    tableFigure->table(table);
    tableFigure->owner(physicalView);
    physicalView->figures().insert(tableFigure);

    viewFigure->view(view);
    viewFigure->owner(physicalView);
    physicalView->figures().insert(viewFigure);

    routineGroupFigure->routineGroup(routineGroup);
    routineGroupFigure->owner(physicalView);
    physicalView->figures().insert(routineGroupFigure);
  }
};


template
<
class X,
class Model
>
void add_to_model(X&, Model&);

template <>
inline void add_to_model<db_RoleRef, SynteticMySQLModel>(db_RoleRef &role, SynteticMySQLModel &model)
{
  model.catalog->roles().insert(role);
}

template <>
inline void add_to_model(db_UserRef &user, SynteticMySQLModel &model)
{
  model.catalog->users().insert(user);
}


template 
<
class X,
class Model= SynteticMySQLModel
>
struct xWrap : public X
{
  xWrap(const char* n, Model &model)
    : X(model.catalog->get_grt())
  {
    content().owner(model.schema);
    content().name(n);
    add_to_model((X&)*this, model);
  };
};


typedef xWrap<db_RoleRef> xRole;
typedef xWrap<db_UserRef> xUser;


inline void add_privilege(SynteticMySQLModel &model, db_RoleRef &role, db_DatabaseObjectRef obj, const char *priv)
{
  db_RolePrivilegeRef privilege(model.catalog->get_grt());
  role->privileges().insert(privilege);
  privilege->owner(role);

  privilege->databaseObject(obj);
  privilege->privileges().insert(priv);
}

inline void add_privilege(SynteticMySQLModel &model, db_RoleRef &role, const char* objectType, const char* objectName, const char *priv)
{
  db_RolePrivilegeRef privilege(model.catalog->get_grt());
  role->privileges().insert(privilege);
  privilege->owner(role);

  privilege->databaseObjectName(objectName);
  privilege->databaseObjectType(objectType);
  privilege->privileges().insert(priv);
}

inline void assign_role(db_UserRef user, db_RoleRef role)
{
  user->roles().insert(role);
}


}

#endif
