#ifndef _WB_HELPERS_H_
#define _WB_HELPERS_H_


//#include "grt_test_utility.h"

#include "grt/common.h"

#include "grts/structs.db.mysql.h"
#include "grts/structs.db.h"
#include "model_model_impl.h"

#include "workbench/wb_context.h"
#include "workbench/wb_context_ui.h"
#include "workbench/wb_model_view_form.h"
#include "workbench/wb_component_basic.h"
#include "workbench/wb_component_physical.h"
#include "workbench/wb_overview.h"

#include "interfaces/interfaces.h"

#include "mdc_canvas_view_image.h"
#include "mdc.h"

// These are only needed to register some classes explicitely.
#include "sqlgenerator.h"
#include "wb_model.h"

using namespace wb;


static void print_output(const std::string &str)
{
  if (getenv("TEST_VERBOSE"))
    printf("%s", str.c_str());
}


static void show_error(const std::string &str, const std::string &detail)
{
//  if (getenv("TEST_VERBOSE"))
    printf("[ERROR] %s\n  %s", str.c_str(), detail.c_str());
}


static void show_warning(const std::string &str, const std::string &detail)
{
  if (getenv("TEST_VERBOSE"))
    printf("[WARNING] %s\n  %s", str.c_str(), detail.c_str());
}


static void show_status(const std::string &str)
{
  if (getenv("TEST_VERBOSE"))
    printf("[Status] %s\n", str.c_str());
}


class WBTester
{
  std::map<std::string, mdc::CanvasView*> view_by_id;

  mdc::CanvasView *create_view_cb(const std::string &id, const std::string &name)
  {
    mdc::CanvasView *nview;

    nview= new mdc::ImageCanvasView(800, 600);
    nview->initialize();

    nview->set_page_size(page_size);
    
    view_by_id[id]= nview;
        
    last_view= nview;
    return nview;
  }
  
public:
  grt::GRT *grt;
  WBContextUI *wbui;
  WBContext *wb;
  mdc::CanvasView *last_view;
  mdc::Size page_size;
  WBOptions wboptions;
  
  WBTester(const mdc::Size &apage_size= mdc::Size(800,600), const WBFrontendCallbacks &callbacks= WBFrontendCallbacks())
    : last_view(0), page_size(apage_size)
  {
    WBFrontendCallbacks wbcallbacks= callbacks;

    //wbcallbacks.show_internal_error= sigc::ptr_fun();
    if (!wbcallbacks.show_error)
      wbcallbacks.show_error= sigc::ptr_fun(show_error);
    if (!wbcallbacks.show_warning)
      wbcallbacks.show_warning= sigc::ptr_fun(show_warning);
    //wbcallbacks.show_message= sigc::ptr_fun();
    //wbcallbacks.confirm_action= sigc::ptr_fun();
    if (!wbcallbacks.show_status_text)
      wbcallbacks.show_status_text= sigc::ptr_fun(show_status);
    //wbcallbacks.show_progress= sigc::ptr_fun();
    //wbcallbacks.open_editor= sigc::ptr_fun();
    //wbcallbacks.show_editor= sigc::ptr_fun();
    //wbcallbacks.hide_editor= sigc::ptr_fun();
    if (!wbcallbacks.create_view)
      wbcallbacks.create_view= sigc::mem_fun(this, &WBTester::create_view_cb);
    //wbcallbacks.destroy_view= sigc::ptr_fun();
    //wbcallbacks.switched_view= sigc::ptr_fun();
    //wbcallbacks.tool_changed= sigc::ptr_fun();
    //wbcallbacks.option_changed= sigc::ptr_fun();
    //wbcallbacks.refresh_gui= sigc::ptr_fun();
    if (!wbcallbacks.print_output)
      wbcallbacks.print_output= sigc::ptr_fun(print_output);
    //wbcallbacks.next_shell_command= sigc::ptr_fun();
    

#ifdef _WIN32
    wboptions.basedir= "../../bin/debug";
#else
    wboptions.basedir= "wbdata";
#endif
    //wboptions.plugin_search_path;
    wboptions.struct_search_path= "../../res/grt";
#ifdef _WIN32
    wboptions.module_search_path= "../../bin/debug";
#else
    wboptions.module_search_path= "";
#endif
    wboptions.user_data_dir= "data";
  
  
    wbui= new WBContextUI(true);
    wb= wbui->get_wb();

    grt= wb->get_grt_manager()->get_grt();
    grt->load_metaclasses("data/structs.test.xml");
    wbui->init(&wbcallbacks, &wboptions);

    // focus the Physical model in the overview by default
    wbui->get_overview()->focus_node(1);
  }

  ~WBTester()
  {
    wb->finalize();
    delete wbui;
  }


  void add_view()
  {    
    wb->add_new_diagram(wb->get_document()->physicalModels()[0]);

    sync_view();

    wbui->set_active_form(wbui->get_view_form(last_view));
  }

  void activate_overview()
  {
    wb->get_ui()->set_active_form(wb->get_ui()->get_overview());
  }


  workbench_physical_ModelRef get_pmodel()
  {
    return wb->get_document()->physicalModels()[0];
  }

  db_mgmt_RdbmsRef get_rdbms()
  {
    return get_pmodel()->rdbms();
  }

  workbench_physical_ViewRef get_pview()
  {
    return get_pmodel()->views().get(0);
  }

  db_CatalogRef get_catalog()
  {
    return wb->get_document()->physicalModels().get(0)->catalog();
  }

  db_SchemaRef get_schema()
  {
    return wb->get_document()->physicalModels().get(0)->catalog()->schemata().get(0);
  }


  void interactive_place_db_objects(int x, int y, std::list<db_DatabaseObjectRef> &objects)
  {
    wb->get_component<WBComponentPhysical>()->
      interactive_place_db_objects(wbui->get_view_form(last_view), x, y, objects);
  }

  db_mysql_TableRef add_table_figure(const std::string &name, int x, int y)
  {
    db_SchemaRef schema(get_schema());

    db_mysql_TableRef table(grt);
    table->name(name);
    table->owner(schema);
    schema->tables().insert(table);

    ModelViewForm *vform= dynamic_cast<ModelViewForm *>(wbui->get_active_main_form());

    if(!vform)
      throw std::logic_error(__FILE__ "::174");

    wb->get_component<WBComponentPhysical>()->place_db_object(
      vform, vform->get_view()->window_to_canvas(x, y), table);

    sync_view();

    return table;
  }

  
  void sync_view()
  {
    int i= 0;
    g_usleep((int)(10000));
    i=1;
    while (last_view==0 || wb->get_grt_manager()->get_dispatcher()->get_busy())
    {
      i++;
      g_usleep((int)(10000));
      wb->flush_idle_tasks();
    }
  }

  void flush_until(float timeout)
  {
    time_t start= time(NULL);
    while (time(NULL) - start < timeout)
    {
      g_usleep((int)(100000*timeout));
      wb->flush_idle_tasks();
    }
  }

  void flush_until(float timeout, sigc::slot<bool> condition)
  {
    time_t start= time(NULL);
    while (time(NULL) - start < timeout && !condition())
    {
      g_usleep((int)(100000*timeout));
      wb->flush_idle_tasks();
    }
  }

  void flush_while(float timeout, sigc::slot<bool> condition)
  {
    time_t start= time(NULL);
    while (time(NULL) - start < timeout && condition())
    {
      g_usleep((int)(100000*timeout));
      wb->flush_idle_tasks();
    }
  }

  void flush_until(float timeout, sigc::slot<int> condition, int value)
  {
    time_t start= time(NULL);
    while (time(NULL) - start < timeout && condition() != value)
    {
      g_usleep((int)(100000*timeout));
      wb->flush_idle_tasks();
    }
  }


  void export_png(const std::string &path)
  {
    wb->export_png(path);
  }
  
  void export_pdf(const std::string &path)
  {
    last_view->get_selection()->clear();
    
    wb->export_pdf(path);
  }

};

class GRTManagerTest : public bec::GRTManager
{
public:
  GRTManagerTest(bool verbose = false) : GRTManager(false, verbose)
  {
    // QQQ remove reference to built binary (search path)?
    set_search_paths("../../bin/debug/", "../../res/grt/", "");

    std::list<std::string> exts;
    exts.push_back(".grt");
    exts.push_back(".wbp.be");
    set_module_extensions(exts);

    register_interfaces(get_grt());


    initialize();
  }
};


#endif
