
#include "stdafx.h"

#include "grt_test_utility.h"

// Do NOT auto register here, but on project level. We need this in several places
// which might not use this file!
#ifndef _WIN32 // ^-- huh?
#define AUTO_REGISTER_GRT_CLASSES
#endif

#include "structs.test.h"

// register test GRT object classes
#include "grts/structs.app.h"
#include "grts/structs.db.h"
#include "grts/structs.db.maxdb.h"
#include "grts/structs.db.mgmt.h"
//#include "grts/structs.db.migration.h"
//#include "grts/structs.db.mssql.h"
#include "grts/structs.db.mysql.h"
//#include "grts/structs.db.oracle.h"
#include "grts/structs.db.query.h"
//#include "grts/structs.db.sybase.h"
#include "grts/structs.eer.h"
#include "grts/structs.h"
#include "grts/structs.model.h"
#include "grts/structs.workbench.h"
#include "grts/structs.workbench.logical.h"
#include "grts/structs.workbench.model.h"
#include "grts/structs.workbench.model.reporting.h"
#include "grts/structs.workbench.physical.h"

#ifdef _WIN32
#undef strncasecmp
#define strncasecmp _strnicmp
#endif

std::string grt_type_to_str(grt::Type grt_type)
{
  std::string res;

  switch (grt_type)
  {
    case grt::AnyType: {res= "AnyType"; break;}
    case grt::IntegerType: {res= "IntegerType"; break;}
    //case DoubleType:
    case grt::DoubleType: {res= "DoubleType"; break;}
    case grt::StringType: {res= "StringType"; break;}
    case grt::ObjectType: {res= "ObjectType"; break;}
    case grt::ListType: {res= "ListType"; break;}
    case grt::DictType: {res= "DictType"; break;}
    default: {res= "UNKNOW"; break;}
  }

  return res;
};


std::string format_msg(
  const char* major_msg,
  const char* minor_msg,
  grt::Type grt_type,
  int item_index)
{
  std::ostringstream oss;
  oss << "[" << major_msg << " ";
  if (NULL != minor_msg)
    oss << "(" << minor_msg << ") ";
  if (-1 != item_index)
    oss << "(item: " << item_index << ") ";

  oss << "(type: " << grt_type_to_str(grt_type) << ")]" << ", ";
  return oss.str();
}


class Followed_obj_stack_helper
{
public:
  Followed_obj_stack_helper(bool clear_stack_on_destroy) : _clear_stack_on_destroy(clear_stack_on_destroy) {}
  ~Followed_obj_stack_helper()
  {
    if (_clear_stack_on_destroy)
      _followed_obj_stack.clear();
  }
  static std::list<std::string> * followed_obj_stack() { return &_followed_obj_stack; }
private:
  static std::list<std::string> _followed_obj_stack;
  bool _clear_stack_on_destroy;
};

std::list<std::string> Followed_obj_stack_helper::_followed_obj_stack;



static std::string fmtmsg(const std::string &msg, const std::list<std::string> &l)
{
  std::string p;
  
  for (std::list<std::string>::const_iterator iter= l.begin(); iter != l.end(); ++iter)
    p.append(*iter);

  return msg+" @ "+p;
}


void grt_ensure_equals(
  const char* major_msg,
  const ValueRef& res_val,
  const ValueRef& test_val,
  bool compare_obj_id,
  int item_index,
  std::list<std::string> *followed_obj_stack,
  std::list<std::string> &followed_path)
{
  static const char* MSGD_TYPE_CHECK= "type_check";
  static const char* MSGD_VALUE_CHECK= "value_check";
  static const char* MSGD_CONTAINER_SIZE_CHECK= "container_size_check";
  static const char* MSGD_KEY_CHECK= "key_check";
  static const char* MSGD_OBJ_ID_CHECK= "object_id_check";
  static const char* MSGD_STRUCT_NAME_CHECK= "structure_name_check";
  static const char* MSGD_MEMBER_COUNT_CHECK= "member_count_check";
  static const char* MSGD_MEMBER_NAME_CHECK= "member_name_check";

  Followed_obj_stack_helper followed_obj_stack_helper(!followed_obj_stack);
  if (!followed_obj_stack)
  {
    followed_obj_stack= Followed_obj_stack_helper::followed_obj_stack();
  }

  std::string msg= format_msg(major_msg, MSGD_TYPE_CHECK, AnyType, item_index);

  tut::ensure_equals(
    fmtmsg(msg, followed_path).c_str(),
    grt_type_to_str(res_val.type()),
    grt_type_to_str(test_val.type()));

  msg= format_msg(major_msg, MSGD_VALUE_CHECK, test_val.type(), item_index); // inflates messages

  switch (res_val.type())
  {
    case grt::IntegerType:
    {
      tut::ensure_equals(fmtmsg(msg, followed_path).c_str(),
        (int)IntegerRef::cast_from(res_val),
        (int)IntegerRef::cast_from(test_val));
      break;
    }

    case grt::DoubleType:
    {
      tut::ensure_equals(fmtmsg(msg, followed_path).c_str(),
        (double)DoubleRef::cast_from(res_val),
        (double)DoubleRef::cast_from(test_val));
      break;
    }

    case grt::StringType:
    {
      tut::ensure_equals(fmtmsg(msg, followed_path).c_str(),
        (std::string)StringRef::cast_from(res_val),
        (std::string)StringRef::cast_from(test_val));
      break;
    }

    case grt::ObjectType:
    {
      ObjectRef res_obj(ObjectRef::cast_from(res_val));
      ObjectRef test_obj(ObjectRef::cast_from(test_val));

      if (followed_obj_stack->end() != std::find(followed_obj_stack->begin(), followed_obj_stack->end(), test_obj.id()))
        break;
      followed_obj_stack->push_back(test_obj.id());

      tut::ensure_equals(
        fmtmsg(format_msg(msg.c_str(), MSGD_STRUCT_NAME_CHECK), followed_path).c_str(),
        res_obj.class_name(),
        test_obj.class_name());


      if (compare_obj_id)
      {
        tut::ensure_equals(
          fmtmsg(format_msg(msg.c_str(), MSGD_OBJ_ID_CHECK), followed_path).c_str(),
          res_obj->id(),
          test_obj->id());
      }

      static const std::string member_name= "name";
      if (test_obj.has_member(member_name))
      {
        std::string obj_name;
        std::string xx= test_obj.id();
        obj_name.append(member_name).append(": ").append(test_obj.get_string_member(member_name));
        msg= format_msg(msg.c_str(), obj_name.c_str(), test_obj.get_member(member_name).type());
      }

      std::string mem_attr;
      grt::Type mem_type;
      bool skip_member_cmp= false;
      
      MetaClass *res_obj_meta  = res_obj.get_metaclass();
      MetaClass *test_obj_meta = test_obj.get_metaclass();

      tut::ensure_equals(
      fmtmsg(format_msg(msg.c_str(), MSGD_MEMBER_COUNT_CHECK), followed_path).c_str(),
      res_obj_meta->name(),
      test_obj_meta->name());

      do
      {
        for (MetaClass::MemberList::const_iterator mem= res_obj_meta->get_members_partial().begin();
             mem != res_obj_meta->get_members_partial().end(); ++mem)
        {
          const MetaClass::Member &res_mem= mem->second;

          if (res_mem.overrides) continue;
          
          skip_member_cmp= false;

          mem_type = res_mem.type.base.type; // ValueRef.TypeSpec.SimpleTypeSpec.Type
          if (grt::ObjectType == mem_type)
          {
            if (0 == strncasecmp("owner", res_mem.name.c_str(), 6))
              skip_member_cmp= true;
            else
            {
              if (!res_mem.owned_object)
                skip_member_cmp= true;
            }
          }

          if (!skip_member_cmp)
          {
            mem_attr = res_obj_meta->get_member_attribute(res_mem.name, "dontdiff");
            if (0 == strncasecmp("1", mem_attr.c_str(), 2))
              skip_member_cmp= true;
          }
          
          if (!skip_member_cmp)
          {
            followed_path.push_back("."+res_mem.name);

            grt_ensure_equals(
                              format_msg(msg.c_str(), std::string("attr: " + res_mem.name).c_str(), test_obj.get_member(res_mem.name).type()).c_str(),
                              res_obj.get_member(res_mem.name),
                              test_obj.get_member(res_mem.name),
                              compare_obj_id,
                              -1,
                              followed_obj_stack,
                              followed_path);
            
            followed_path.pop_back();
          }
        }

        res_obj_meta= res_obj_meta->parent();
      }
      while (res_obj_meta != 0);

      followed_obj_stack->pop_back();
      break;
    }

    case grt::ListType:
    {
      BaseListRef res_list= BaseListRef::cast_from(res_val);
      BaseListRef test_list= BaseListRef::cast_from(test_val);
      
      std::string list_msg= format_msg(msg.c_str(), MSGD_CONTAINER_SIZE_CHECK, AnyType, -1);
      tut::ensure_equals(
        fmtmsg(list_msg, followed_path).c_str(),
        res_list.count(),
        test_list.count());

      for (unsigned n= 0; n<res_list.count(); n++)
      {
        followed_path.push_back(strfmt("[%i]", n));
        
        grt_ensure_equals(
          msg.c_str(),
          res_list.get(n),
          test_list.get(n),
          compare_obj_id,
          n,
          followed_obj_stack,
          followed_path);

        followed_path.pop_back();
      }

      break;
    }

    case grt::DictType:
    {
      DictRef res_dict= DictRef::cast_from(res_val);
      DictRef test_dict= DictRef::cast_from(test_val);
      
      std::string dict_msg= format_msg(msg.c_str(), MSGD_CONTAINER_SIZE_CHECK, AnyType);
      tut::ensure_equals(
        fmtmsg(dict_msg, followed_path).c_str(),
        res_dict.count(),
        test_dict.count());

      DictRef::const_iterator res_dict_it         = res_dict.begin();
      const DictRef::const_iterator res_dict_last = res_dict.end();

      DictRef::const_iterator test_dict_it         = test_dict.begin();
      const DictRef::const_iterator test_dict_last = test_dict.end();
      
      int n = 0;
      for (; res_dict_last != res_dict_it; res_dict_it++, test_dict_it++ )
      {
        tut::ensure("Dicts differ", test_dict_last != test_dict_it);
      
        const std::string &res_key  = res_dict_it->first;
        const std::string &test_key = test_dict_it->first;
        ValueRef res_val = res_dict_it->second;
        ValueRef test_val = test_dict_it->second;

        dict_msg= format_msg(msg.c_str(), MSGD_KEY_CHECK, AnyType);
        tut::ensure_equals(
          fmtmsg(dict_msg, followed_path).c_str(),
          res_key,
          test_key);

        followed_path.push_back("["+res_key+"]");

        grt_ensure_equals(
            msg.c_str(),
            res_val,
            test_val,
            compare_obj_id,
            n,
            followed_obj_stack,
            followed_path);

        followed_path.pop_back();

        ++n;
      }

      break;
    }

    case grt::AnyType:
    default:
    {
      //ANY is considered as empty value
      break;
    }
  }
}


void grt_ensure_equals(
  const char* major_msg,
  const ValueRef& res_val,
  const ValueRef& test_val,
  bool compare_obj_id,
  int item_index,
  std::list<std::string> *followed_obj_stack)
{
  std::list<std::string> path;
  grt_ensure_equals(major_msg, res_val, test_val, compare_obj_id, item_index, followed_obj_stack, path);
}


void check_module_function_return(
  Module *test_module,
  const char *func_name,
  BaseListRef &args,
  const ValueRef &test_val,
  bool compare_obj_id)
{
  ValueRef res_val;
  res_val= test_module->call_function(func_name, args);
  grt_ensure_equals(func_name, res_val, test_val, compare_obj_id);
}





static void dump_tree_node(FILE *f, bec::TreeModel *tree, const bec::NodeId &node, const std::vector<int> &columns)
{
  std::string value;
  bool ret;

  fprintf(f, "%s:%i:", node.repr().c_str(), tree->count_children(node));

  for (std::vector<int>::const_iterator column= columns.begin(); column != columns.end(); ++column)
  {
    ret= tree->get_field(node, *column, value);
    if (!ret)
    {
      int i;
      ret= tree->get_field(node, *column, i);
      fprintf(f, "::%i:%i", ret, i);
    }
    else
      fprintf(f, "::%i:%s", ret, value.c_str());
  }

  fprintf(f, "\n");

  for (int c= tree->count_children(node), i= 0; i < c; i++)
  {
    bec::NodeId child(tree->get_child(node, i));

    dump_tree_node(f, tree, child, columns);
  }
}



void dump_tree_model(const std::string &path, bec::TreeModel *tree, const std::vector<int> &columns)
{
  FILE *f= fopen(path.c_str(), "w+");

  bec::NodeId node= tree->get_root();
  
  dump_tree_node(f, tree, node, columns);

  fclose(f);
}

