//---------------------------------------------------------------------------


#include "stdafx.h"

#ifndef _WIN32
#include <memory>
#include <algorithm>
#include <stdio.h>
#include <glib/gthread.h>
#endif

#include "test.h" // test.h is located in common/library/test/include

//#if _WIN32_WINNT >= 0x0502
//void ImportMysqlCanvasDll(bool);
//#endif

//---------------------------------------------------------------------------

namespace tut
{
  test_runner_singleton runner;
  mysql_reporter *tut_rep= NULL;
}

void my_log_handler(const gchar *log_domain, GLogLevelFlags log_level, const gchar *message, gpointer user_data)
{
  // If program execution arrives here then we found a critical or fatal error in glib.
  // The log call is usually triggered by g_return_if_fail() and g_return_value_if_fail().
  // We cannot raise an exception here (as we'd do with tut:fail()) because this prevents glib
  // logging from proper unwinding. So just trigger the output as the TUT runner would do.
  /*
  tut::mysql_reporter* reporter = (tut::mysql_reporter) user_data;
  tut::test_result result(name_,ti->first,test_result::fail,ex);
  reporter->test_completed(result);
  */
}

Global_test_parameters *test_params= NULL;

int main(int argc, char* argv[])
{
#ifdef _WIN32
  // This is supposed to suppress error dialogs for exceptions, which don't make sense in a command line app
  // anyway. Tests will simply fail if an exception appeared.
  // It does not seem to have an effect (at least when running in a VM), though.
  SetErrorMode(SEM_FAILCRITICALERRORS);
#endif

  g_thread_init(NULL);
  
  test_params= new Global_test_parameters(argc, argv);
  tut::mysql_reporter rep(test_params->get_output_mode() == Global_test_parameters::DEBUGGER, test_params->report_passed());
  tut::tut_rep= &rep;
  tut::runner.get().set_callback(&rep);

  // Set glib error log to own handler to trigger a test failure if an error is logged in glib.
  GLogLevelFlags log_flags= GLogLevelFlags(G_LOG_LEVEL_WARNING | G_LOG_LEVEL_CRITICAL | G_LOG_LEVEL_ERROR | G_LOG_FLAG_FATAL
     | G_LOG_FLAG_RECURSION);
  //g_log_set_handler(NULL, log_flags, my_log_handler, &rep);

  // remove wb_options.xml to recreate it using latest versions of default values
  remove("./data/wb_options.xml");

  //#if _WIN32_WINNT >= 0x0502
    // Select temporary between modifications of mysql.canvas.dll
//    ImportMysqlCanvasDll(false);
  //#endif
  try
  {
    switch (test_params->get_run_mode())
    {
    case Global_test_parameters::ALL:
       tut::runner.get().run_tests();
       break;
    case Global_test_parameters::LIST:
       {
         tut::groupnames names= tut::runner.get().list_groups();
         for (tut::groupnames::const_iterator i= names.begin(); i != names.end(); ++i)
         {
            printf("%s\n", i->c_str());
         }
       }
       break;
    case Global_test_parameters::SELECTED_ONLY:
       {
         std::vector<std::string> only= test_params->get_groups();
         for (tut::groupnames::const_iterator i= only.begin(); i != only.end(); ++i)
         {
           tut::runner.get().run_tests(*i);
         }
       }
       break;
    case Global_test_parameters::SELECTED_SKIP:
        {
         tut::groupnames names= tut::runner.get().list_groups();
         std::vector<std::string> skip= test_params->get_groups();
         for (tut::groupnames::const_iterator i= names.begin(); i != names.end(); ++i)
         {
            if (std::find(skip.begin(), skip.end(), *i) == skip.end())
              tut::runner.get().run_tests(*i);
         }
       }
       break;
    }

    rep.print_summary();
  }
  catch(std::exception& e)
  {
    printf("Unhandled std exception %s", e.what());
  }
  catch(...)
  {
    printf("Unhandled std exception");
  }
#ifdef __WIN__
  if (test_params->wait())
  {
    printf("Tests finished. Press <enter>");
    while(!_kbhit())
      Sleep(1000);
  };
#endif
  return rep.succeeded() ? 0 : 1;
}

//---------------------------------------------------------------------------

#if 0

#if _WIN32_WINNT >= 0x0502

#include <delayimp.h>
#include <tchar.h>
#include <string>
#include <windows.h>
#include <shlwapi.h>

LONG WINAPI DelayLoadDllExceptionFilter(PEXCEPTION_POINTERS pep, BOOL critical)
{
  // Assume we recognize this exception
  LONG lDisposition = EXCEPTION_EXECUTE_HANDLER;  

  // If this is a Delay-load problem, ExceptionInformation[0] points 
  // to a DelayLoadInfo structure that has detailed error info
  PDelayLoadInfo pdli = PDelayLoadInfo(pep->ExceptionRecord->ExceptionInformation[0]);

  // Create a buffer where we construct error messages
  TCHAR sz[500] = { 0 };

  switch (pep->ExceptionRecord->ExceptionCode)
  {
  case VcppException(ERROR_SEVERITY_ERROR, ERROR_MOD_NOT_FOUND):
    // The DLL module was not found at runtime
    wsprintf(sz, _T("Dll not found: %hs"), pdli->szDll);
    break;

  case VcppException(ERROR_SEVERITY_ERROR, ERROR_PROC_NOT_FOUND):
    // The DLL module was found but it doesn't contain the function
    if (pdli->dlp.fImportByName)
    {
      wsprintf(sz, _T("Function %hs was not found in %hs"),
        pdli->dlp.szProcName, pdli->szDll);
    }
    else
    {
      wsprintf(sz, _T("Function ordinal %d was not found in %hs"),
        pdli->dlp.dwOrdinal, pdli->szDll);
    }
    break; 

  default:
    lDisposition = EXCEPTION_CONTINUE_SEARCH;  // We don't recognize this exception
    break;
  }

  if (lDisposition == EXCEPTION_EXECUTE_HANDLER)
  {
    // We recognized this error and constructed a message, show it
    if (critical)
      MessageBox(NULL, sz, NULL, MB_OK);
  }

  return(lDisposition);
}

BOOL LoadAllImportsForDll(LPCTSTR dllDir, LPCSTR dllName, BOOL critical)
{
  __try
  {
    return SetDllDirectory(dllDir) && SUCCEEDED(__HrLoadAllImportsForDll(dllName));
  }
 __except (DelayLoadDllExceptionFilter(GetExceptionInformation(), critical))
 {
 }
  return FALSE;
}


/*
Chooses canvas library based on glu or mesa implementation of opengl.
*/
void ImportMysqlCanvasDll(bool use_mesa_library)
{
  static const CHAR* dllName = "mysql.canvas.dll";

  TCHAR moduleFileName[MAX_PATH];
  TCHAR dllDir[MAX_PATH];

  // unload if it was already loaded
  //(void) __FUnloadDelayLoadedDLL2(dllName);

  (void) GetModuleFileName(NULL, moduleFileName, MAX_PATH);
  (void) PathRemoveFileSpec(moduleFileName);

  // store previous DllDirectory
  if (GetDllDirectory(MAX_PATH, dllDir) == 0)
    dllDir[0] = '\0';

  // try to link with glu first
  std::basic_string<TCHAR> newDllDir(moduleFileName);
  newDllDir.append(_T("\\")).append(_T("mysql.canvas"));
  if (use_mesa_library || !LoadAllImportsForDll(newDllDir.c_str(), dllName, false))
  {
    // rollback to mesa
    newDllDir= moduleFileName;
    newDllDir.append(_T("\\")).append(_T("mysql.canvas.mesa"));
    if (!LoadAllImportsForDll(newDllDir.c_str(), dllName, true))
    {
      //! handle bad scenario here
    }
  }

  // restore original DllDirectory
  SetDllDirectory((dllDir[0] != '\0') ? dllDir : (LPTSTR)NULL);
}

#endif
#endif
